@@ -63,7 +63,7 @@ bool GDExtensionSpecialCompatHashes::get_legacy_hashes(const StringName &p_class
 			if (p_check_valid) {
 				MethodBind *mb = ClassDB::get_method_with_compatibility(p_class, p_method, mapping.current_hash);
 				if (!mb) {
-					WARN_PRINT(vformat("Compatibility hash %d for %s::%s() mapped to non-existent hash %d. Please update gdextension_special_compat_hashes.cpp.", mapping.legacy_hash, p_class, p_method, mapping.current_hash));
+					WARN_PRINT(vformat("Compatibility hash %d for %s::%s() mapped to non-existent hash %d in gdextension_special_compat_hashes.cpp.", mapping.legacy_hash, p_class, p_method, mapping.current_hash));
 					continue;
 				}
 			}
@@ -331,6 +331,12 @@ void GDExtensionSpecialCompatHashes::initialize() {
 	});
 	mappings.insert("EditorFileDialog", {
 		{ "add_filter", 233059325, 3388804757 },
+		{ "get_access", 778734016, 3344081076 },
+		{ "set_access", 3882893764, 4104413466 },
+		{ "get_display_mode", 3517174669, 1092104624 },
+		{ "set_display_mode", 3049004050, 2692197101 },
+		{ "get_file_mode", 2681044145, 4074825319 },
+		{ "set_file_mode", 274150415, 3654936397 },
 	});
 	mappings.insert("EditorImportPlugin", {
 		{ "append_import_external_resource", 3645925746, 320493106 },
