@@ -80,7 +80,7 @@ public:
 
 	virtual String as_text() const = 0;
 
-	virtual Ref<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const;
+	virtual RequiredResult<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const;
 
 	virtual bool action_match(const Ref<InputEvent> &p_event, bool p_exact_match, float p_deadzone, bool *r_pressed, float *r_strength, float *r_raw_strength) const;
 	virtual bool is_match(const Ref<InputEvent> &p_event, bool p_exact_match = true) const;
@@ -90,8 +90,6 @@ public:
 	virtual bool accumulate(const Ref<InputEvent> &p_event) { return false; }
 
 	virtual InputEventType get_type() const { return InputEventType::INVALID; }
-
-	InputEvent() {}
 };
 
 class InputEventFromWindow : public InputEvent {
@@ -105,8 +103,6 @@ protected:
 public:
 	void set_window_id(int64_t p_id);
 	int64_t get_window_id() const;
-
-	InputEventFromWindow() {}
 };
 
 class InputEventWithModifiers : public InputEventFromWindow {
@@ -146,9 +142,7 @@ public:
 	BitField<KeyModifierMask> get_modifiers_mask() const;
 
 	virtual String as_text() const override;
-	virtual String to_string() override;
-
-	InputEventWithModifiers() {}
+	virtual String _to_string() override;
 };
 
 class InputEventKey : public InputEventWithModifiers {
@@ -200,13 +194,11 @@ public:
 	virtual String as_text_key_label() const;
 	virtual String as_text_location() const;
 	virtual String as_text() const override;
-	virtual String to_string() override;
+	virtual String _to_string() override;
 
 	static Ref<InputEventKey> create_reference(Key p_keycode_with_modifier_masks, bool p_physical = false);
 
 	InputEventType get_type() const final override { return InputEventType::KEY; }
-
-	InputEventKey() {}
 };
 
 class InputEventMouse : public InputEventWithModifiers {
@@ -229,8 +221,6 @@ public:
 
 	void set_global_position(const Vector2 &p_global_pos);
 	Vector2 get_global_position() const;
-
-	InputEventMouse() {}
 };
 
 class InputEventMouseButton : public InputEventMouse {
@@ -256,18 +246,16 @@ public:
 	void set_double_click(bool p_double_click);
 	bool is_double_click() const;
 
-	virtual Ref<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const override;
+	virtual RequiredResult<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const override;
 
 	virtual bool action_match(const Ref<InputEvent> &p_event, bool p_exact_match, float p_deadzone, bool *r_pressed, float *r_strength, float *r_raw_strength) const override;
 	virtual bool is_match(const Ref<InputEvent> &p_event, bool p_exact_match = true) const override;
 
 	virtual bool is_action_type() const override { return true; }
 	virtual String as_text() const override;
-	virtual String to_string() override;
+	virtual String _to_string() override;
 
 	InputEventType get_type() const final override { return InputEventType::MOUSE_BUTTON; }
-
-	InputEventMouseButton() {}
 };
 
 class InputEventMouseMotion : public InputEventMouse {
@@ -306,15 +294,13 @@ public:
 	void set_screen_velocity(const Vector2 &p_velocity);
 	Vector2 get_screen_velocity() const;
 
-	virtual Ref<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const override;
+	virtual RequiredResult<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const override;
 	virtual String as_text() const override;
-	virtual String to_string() override;
+	virtual String _to_string() override;
 
 	virtual bool accumulate(const Ref<InputEvent> &p_event) override;
 
 	InputEventType get_type() const final override { return InputEventType::MOUSE_MOTION; }
-
-	InputEventMouseMotion() {}
 };
 
 class InputEventJoypadMotion : public InputEvent {
@@ -337,13 +323,12 @@ public:
 
 	virtual bool is_action_type() const override { return true; }
 	virtual String as_text() const override;
-	virtual String to_string() override;
+	virtual String _to_string() override;
 
-	static Ref<InputEventJoypadMotion> create_reference(JoyAxis p_axis, float p_value);
+	// The default device ID is `InputMap::ALL_DEVICES`.
+	static Ref<InputEventJoypadMotion> create_reference(JoyAxis p_axis, float p_value, int p_device = -1);
 
 	InputEventType get_type() const final override { return InputEventType::JOY_MOTION; }
-
-	InputEventJoypadMotion() {}
 };
 
 class InputEventJoypadButton : public InputEvent {
@@ -369,13 +354,12 @@ public:
 	virtual bool is_action_type() const override { return true; }
 
 	virtual String as_text() const override;
-	virtual String to_string() override;
+	virtual String _to_string() override;
 
-	static Ref<InputEventJoypadButton> create_reference(JoyButton p_btn_index);
+	// The default device ID is `InputMap::ALL_DEVICES`.
+	static Ref<InputEventJoypadButton> create_reference(JoyButton p_btn_index, int p_device = -1);
 
 	InputEventType get_type() const final override { return InputEventType::JOY_BUTTON; }
-
-	InputEventJoypadButton() {}
 };
 
 class InputEventScreenTouch : public InputEventFromWindow {
@@ -400,13 +384,11 @@ public:
 	void set_double_tap(bool p_double_tap);
 	bool is_double_tap() const;
 
-	virtual Ref<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const override;
+	virtual RequiredResult<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const override;
 	virtual String as_text() const override;
-	virtual String to_string() override;
+	virtual String _to_string() override;
 
 	InputEventType get_type() const final override { return InputEventType::SCREEN_TOUCH; }
-
-	InputEventScreenTouch() {}
 };
 
 class InputEventScreenDrag : public InputEventFromWindow {
@@ -452,15 +434,13 @@ public:
 	void set_screen_velocity(const Vector2 &p_velocity);
 	Vector2 get_screen_velocity() const;
 
-	virtual Ref<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const override;
+	virtual RequiredResult<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const override;
 	virtual String as_text() const override;
-	virtual String to_string() override;
+	virtual String _to_string() override;
 
 	virtual bool accumulate(const Ref<InputEvent> &p_event) override;
 
 	InputEventType get_type() const final override { return InputEventType::SCREEN_DRAG; }
-
-	InputEventScreenDrag() {}
 };
 
 class InputEventAction : public InputEvent {
@@ -493,11 +473,9 @@ public:
 	virtual bool is_action_type() const override { return true; }
 
 	virtual String as_text() const override;
-	virtual String to_string() override;
+	virtual String _to_string() override;
 
 	InputEventType get_type() const final override { return InputEventType::ACTION; }
-
-	InputEventAction() {}
 };
 
 class InputEventGesture : public InputEventWithModifiers {
@@ -524,13 +502,11 @@ public:
 	void set_factor(real_t p_factor);
 	real_t get_factor() const;
 
-	virtual Ref<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const override;
+	virtual RequiredResult<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const override;
 	virtual String as_text() const override;
-	virtual String to_string() override;
+	virtual String _to_string() override;
 
 	InputEventType get_type() const final override { return InputEventType::MAGNIFY_GESTURE; }
-
-	InputEventMagnifyGesture() {}
 };
 
 class InputEventPanGesture : public InputEventGesture {
@@ -544,13 +520,11 @@ public:
 	void set_delta(const Vector2 &p_delta);
 	Vector2 get_delta() const;
 
-	virtual Ref<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const override;
+	virtual RequiredResult<InputEvent> xformed_by(const Transform2D &p_xform, const Vector2 &p_local_ofs = Vector2()) const override;
 	virtual String as_text() const override;
-	virtual String to_string() override;
+	virtual String _to_string() override;
 
 	InputEventType get_type() const final override { return InputEventType::PAN_GESTURE; }
-
-	InputEventPanGesture() {}
 };
 
 class InputEventMIDI : public InputEvent {
@@ -594,11 +568,9 @@ public:
 	int get_controller_value() const;
 
 	virtual String as_text() const override;
-	virtual String to_string() override;
+	virtual String _to_string() override;
 
 	InputEventType get_type() const final override { return InputEventType::MIDI; }
-
-	InputEventMIDI() {}
 };
 
 class InputEventShortcut : public InputEvent {
@@ -614,7 +586,7 @@ public:
 	Ref<Shortcut> get_shortcut();
 
 	virtual String as_text() const override;
-	virtual String to_string() override;
+	virtual String _to_string() override;
 
 	InputEventType get_type() const final override { return InputEventType::SHORTCUT; }
 
