@@ -75,8 +75,6 @@ public:
 	Error pack(const Variant &p_data);
 
 	int size() const;
-
-	PackedDataContainer() {}
 };
 
 class PackedDataContainerRef : public RefCounted {
@@ -96,6 +94,4 @@ public:
 
 	int size() const;
 	virtual Variant getvar(const Variant &p_key, bool *r_valid = nullptr) const override;
-
-	PackedDataContainerRef() {}
 };
