@@ -54,7 +54,9 @@ private:
 		bool saved_to_cache = false;
 	};
 
-	HashMap<ID, Cache> unique_ids; //unique IDs and utf8 paths (less memory used)
+	HashMap<ID, Cache> unique_ids; // Unique IDs and utf8 paths (less memory used).
+	bool use_reverse_cache = false;
+	HashMap<CharString, ID> reverse_cache; // Used at runtime.
 	static ResourceUID *singleton;
 
 	uint32_t cache_entries = 0;
@@ -75,6 +77,7 @@ public:
 	void add_id(ID p_id, const String &p_path);
 	void set_id(ID p_id, const String &p_path);
 	String get_id_path(ID p_id) const;
+	ID get_path_id(const String &p_path) const;
 	void remove_id(ID p_id);
 
 	static String uid_to_path(const String &p_uid);
@@ -86,6 +89,7 @@ public:
 	Error update_cache();
 	static String get_path_from_cache(Ref<FileAccess> &p_cache_file, const String &p_uid_string);
 
+	void enable_reverse_cache() { use_reverse_cache = true; }
 	void clear();
 
 	static ResourceUID *get_singleton() { return singleton; }
