@@ -30,37 +30,54 @@
 
 #pragma once
 
+#include "core/templates/a_hash_map.h"
 #include "core/templates/hash_map.h"
 #include "core/templates/paged_allocator.h"
+#include "core/templates/rb_map.h"
 #include "core/templates/self_list.h"
 #include "rendering_shader_container_d3d12.h"
 #include "servers/rendering/rendering_device_driver.h"
 
-#ifndef _MSC_VER
+#if !defined(_MSC_VER) && !defined(__REQUIRED_RPCNDR_H_VERSION__)
 // Match current version used by MinGW, MSVC and Direct3D 12 headers use 500.
 #define __REQUIRED_RPCNDR_H_VERSION__ 475
-#endif
+#endif // !defined(_MSC_VER) && !defined(__REQUIRED_RPCNDR_H_VERSION__)
 
-#include <d3dx12.h>
-#include <dxgi1_6.h>
-#define D3D12MA_D3D12_HEADERS_ALREADY_INCLUDED
-#include <D3D12MemAlloc.h>
+GODOT_GCC_WARNING_PUSH
+GODOT_GCC_WARNING_IGNORE("-Wimplicit-fallthrough")
+GODOT_GCC_WARNING_IGNORE("-Wmissing-field-initializers")
+GODOT_GCC_WARNING_IGNORE("-Wnon-virtual-dtor")
+GODOT_GCC_WARNING_IGNORE("-Wshadow")
+GODOT_GCC_WARNING_IGNORE("-Wswitch")
+GODOT_CLANG_WARNING_PUSH
+GODOT_CLANG_WARNING_IGNORE("-Wimplicit-fallthrough")
+GODOT_CLANG_WARNING_IGNORE("-Wmissing-field-initializers")
+GODOT_CLANG_WARNING_IGNORE("-Wnon-virtual-dtor")
+GODOT_CLANG_WARNING_IGNORE("-Wstring-plus-int")
+GODOT_CLANG_WARNING_IGNORE("-Wswitch")
+
+#include <thirdparty/directx_headers/include/directx/d3dx12.h>
+
+GODOT_GCC_WARNING_POP
+GODOT_CLANG_WARNING_POP
 
 #include <wrl/client.h>
 
-#if defined(_MSC_VER) && defined(MemoryBarrier)
-// Annoying define from winnt.h. Reintroduced by some of the headers above.
-#undef MemoryBarrier
-#endif
-
-using Microsoft::WRL::ComPtr;
-
 #ifdef DEV_ENABLED
 #define CUSTOM_INFO_QUEUE_ENABLED 0
 #endif
 
 class RenderingContextDriverD3D12;
 
+namespace D3D12MA {
+class Allocation;
+class Allocator;
+class VirtualBlock;
+}; // namespace D3D12MA
+
+struct IDXGIAdapter;
+struct IDXGISwapChain3;
+
 // Design principles:
 // - D3D12 structs are zero-initialized and fields not requiring a non-zero value are omitted (except in cases where expresivity reasons apply).
 class RenderingDeviceDriverD3D12 : public RenderingDeviceDriver {
@@ -97,10 +114,6 @@ class RenderingDeviceDriverD3D12 : public RenderingDeviceDriver {
 		bool native_16bit_ops = false;
 	};
 
-	struct StorageBufferCapabilities {
-		bool storage_buffer_16_bit_access_is_supported = false;
-	};
-
 	struct FormatCapabilities {
 		bool relaxed_casting_supported = false;
 	};
@@ -115,9 +128,8 @@ class RenderingDeviceDriverD3D12 : public RenderingDeviceDriver {
 
 	RenderingContextDriverD3D12 *context_driver = nullptr;
 	RenderingContextDriver::Device context_device;
-	ComPtr<IDXGIAdapter> adapter;
-	DXGI_ADAPTER_DESC adapter_desc;
-	ComPtr<ID3D12Device> device;
+	Microsoft::WRL::ComPtr<IDXGIAdapter> adapter;
+	Microsoft::WRL::ComPtr<ID3D12Device> device;
 	DeviceLimits device_limits;
 	RDD::Capabilities device_capabilities;
 	uint32_t feature_level = 0; // Major * 10 + minor.
@@ -126,49 +138,63 @@ class RenderingDeviceDriverD3D12 : public RenderingDeviceDriver {
 	FragmentShadingRateCapabilities fsr_capabilities;
 	FragmentDensityMapCapabilities fdm_capabilities;
 	ShaderCapabilities shader_capabilities;
-	StorageBufferCapabilities storage_buffer_capabilities;
 	FormatCapabilities format_capabilities;
 	BarrierCapabilities barrier_capabilities;
 	MiscFeaturesSupport misc_features_support;
 	RenderingShaderContainerFormatD3D12 shader_container_format;
 	String pipeline_cache_id;
+	D3D12_HEAP_TYPE dynamic_persistent_upload_heap = D3D12_HEAP_TYPE_UPLOAD;
 
-	class DescriptorsHeap {
-		D3D12_DESCRIPTOR_HEAP_DESC desc = {};
-		ComPtr<ID3D12DescriptorHeap> heap;
-		uint32_t handle_size = 0;
-
-	public:
-		class Walker { // Texas Ranger.
-			friend class DescriptorsHeap;
-
-			uint32_t handle_size = 0;
-			uint32_t handle_count = 0;
-			D3D12_CPU_DESCRIPTOR_HANDLE first_cpu_handle = {};
-			D3D12_GPU_DESCRIPTOR_HANDLE first_gpu_handle = {};
-			uint32_t handle_index = 0;
-
-		public:
-			D3D12_CPU_DESCRIPTOR_HANDLE get_curr_cpu_handle();
-			D3D12_GPU_DESCRIPTOR_HANDLE get_curr_gpu_handle();
-			_FORCE_INLINE_ void rewind() { handle_index = 0; }
-			void advance(uint32_t p_count = 1);
-			uint32_t get_current_handle_index() const { return handle_index; }
-			uint32_t get_free_handles() { return handle_count - handle_index; }
-			bool is_at_eof() { return handle_index == handle_count; }
+	struct DescriptorHeap {
+		struct Allocation {
+			uint64_t virtual_alloc_handle = {}; // This is the handle value in "D3D12MA::VirtualAllocation".
+			D3D12_CPU_DESCRIPTOR_HANDLE cpu_handle = {};
+			D3D12_GPU_DESCRIPTOR_HANDLE gpu_handle = {};
 		};
 
-		Error allocate(ID3D12Device *m_device, D3D12_DESCRIPTOR_HEAP_TYPE m_type, uint32_t m_descriptor_count, bool p_for_gpu);
-		uint32_t get_descriptor_count() const { return desc.NumDescriptors; }
-		ID3D12DescriptorHeap *get_heap() const { return heap.Get(); }
+		Microsoft::WRL::ComPtr<ID3D12DescriptorHeap> heap;
+		D3D12_CPU_DESCRIPTOR_HANDLE cpu_handle = {};
+		D3D12_GPU_DESCRIPTOR_HANDLE gpu_handle = {};
+		uint32_t increment_size = 0;
 
-		Walker make_walker() const;
+		Microsoft::WRL::ComPtr<D3D12MA::VirtualBlock> virtual_block;
+
+		Error initialize(ID3D12Device *p_device, D3D12_DESCRIPTOR_HEAP_TYPE p_type, uint32_t p_num_descriptors, bool p_shader_visible);
+
+		Error allocate(uint32_t p_descriptor_count, Allocation &r_allocation);
+		void free(const Allocation &p_allocation);
 	};
 
+	// Some IHVs do not allow creating descriptor heaps beyond a certain limit, so they must be pooled.
+	struct CPUDescriptorHeapPool {
+		struct Allocation : DescriptorHeap::Allocation {
+			uint32_t heap_index = UINT_MAX;
+		};
+
+		BinaryMutex mutex;
+		LocalVector<DescriptorHeap> heaps;
+
+		D3D12_DESCRIPTOR_HEAP_TYPE type = {};
+		uint32_t increment_size = 0;
+
+		void initialize(ID3D12Device *p_device, D3D12_DESCRIPTOR_HEAP_TYPE p_type);
+
+		Error allocate(uint32_t p_descriptor_count, ID3D12Device *p_device, Allocation &r_allocation);
+		void free(const Allocation &p_allocation);
+	};
+
+	DescriptorHeap resource_descriptor_heap;
+	DescriptorHeap sampler_descriptor_heap;
+	CPUDescriptorHeapPool resource_descriptor_heap_pool;
+	CPUDescriptorHeapPool rtv_descriptor_heap_pool;
+	CPUDescriptorHeapPool dsv_descriptor_heap_pool;
+
+	CPUDescriptorHeapPool::Allocation null_rtv_alloc;
+
 	struct {
-		ComPtr<ID3D12CommandSignature> draw;
-		ComPtr<ID3D12CommandSignature> draw_indexed;
-		ComPtr<ID3D12CommandSignature> dispatch;
+		Microsoft::WRL::ComPtr<ID3D12CommandSignature> draw;
+		Microsoft::WRL::ComPtr<ID3D12CommandSignature> draw_indexed;
+		Microsoft::WRL::ComPtr<ID3D12CommandSignature> dispatch;
 	} indirect_cmd_signatures;
 
 	static void STDMETHODCALLTYPE _debug_message_func(D3D12_MESSAGE_CATEGORY p_category, D3D12_MESSAGE_SEVERITY p_severity, D3D12_MESSAGE_ID p_id, LPCSTR p_description, void *p_context);
@@ -188,7 +214,7 @@ private:
 	/**** MEMORY ****/
 	/****************/
 
-	ComPtr<D3D12MA::Allocator> allocator;
+	Microsoft::WRL::ComPtr<D3D12MA::Allocator> allocator;
 
 	/******************/
 	/**** RESOURCE ****/
@@ -203,8 +229,8 @@ private:
 
 		ID3D12Resource *resource = nullptr; // Non-null even if not owned.
 		struct {
-			ComPtr<ID3D12Resource> resource;
-			ComPtr<D3D12MA::Allocation> allocation;
+			Microsoft::WRL::ComPtr<ID3D12Resource> resource;
+			Microsoft::WRL::ComPtr<D3D12MA::Allocation> allocation;
 			States states;
 		} owner_info; // All empty if the resource is not owned.
 		States *states_ptr = nullptr; // Own or from another if it doesn't own the D3D12 resource.
@@ -238,20 +264,34 @@ private:
 	/*****************/
 
 	struct BufferInfo : public ResourceInfo {
+		D3D12_GPU_VIRTUAL_ADDRESS gpu_virtual_address = {};
 		DataFormat texel_format = DATA_FORMAT_MAX;
 		uint64_t size = 0;
 		struct {
-			bool usable_as_uav : 1;
+			bool is_dynamic : 1; // Only used for tracking (e.g. Vulkan needs these checks).
 		} flags = {};
+
+		bool is_dynamic() const { return flags.is_dynamic; }
+	};
+
+	struct BufferDynamicInfo : BufferInfo {
+		uint32_t frame_idx = UINT32_MAX;
+		uint8_t *persistent_ptr = nullptr;
+#ifdef DEBUG_ENABLED
+		// For tracking that a persistent buffer isn't mapped twice in the same frame.
+		uint64_t last_frame_mapped = 0;
+#endif
 	};
 
 public:
-	virtual BufferID buffer_create(uint64_t p_size, BitField<BufferUsageBits> p_usage, MemoryAllocationType p_allocation_type) override final;
+	virtual BufferID buffer_create(uint64_t p_size, BitField<BufferUsageBits> p_usage, MemoryAllocationType p_allocation_type, uint64_t p_frames_drawn) override final;
 	virtual bool buffer_set_texel_format(BufferID p_buffer, DataFormat p_format) override final;
 	virtual void buffer_free(BufferID p_buffer) override final;
 	virtual uint64_t buffer_get_allocation_size(BufferID p_buffer) override final;
 	virtual uint8_t *buffer_map(BufferID p_buffer) override final;
 	virtual void buffer_unmap(BufferID p_buffer) override final;
+	virtual uint8_t *buffer_persistent_map_advance(BufferID p_buffer, uint64_t p_frames_drawn) override final;
+	virtual uint64_t buffer_get_dynamic_offsets(Span<BufferID> p_buffers) override final;
 	virtual uint64_t buffer_get_device_address(BufferID p_buffer) override final;
 
 	/*****************/
@@ -273,13 +313,10 @@ private:
 
 		TextureInfo *main_texture = nullptr;
 
-		UINT mapped_subresource = UINT_MAX;
-		SelfList<TextureInfo> pending_clear{ this };
 #ifdef DEBUG_ENABLED
 		bool created_from_extension = false;
 #endif
 	};
-	SelfList<TextureInfo>::List textures_pending_clear;
 
 	HashMap<DXGI_FORMAT, uint32_t> format_sample_counts_mask_cache;
 	Mutex format_sample_counts_mask_cache_mutex;
@@ -303,8 +340,7 @@ public:
 	virtual void texture_free(TextureID p_texture) override final;
 	virtual uint64_t texture_get_allocation_size(TextureID p_texture) override final;
 	virtual void texture_get_copyable_layout(TextureID p_texture, const TextureSubresource &p_subresource, TextureCopyableLayout *r_layout) override final;
-	virtual uint8_t *texture_map(TextureID p_texture, const TextureSubresource &p_subresource) override final;
-	virtual void texture_unmap(TextureID p_texture) override final;
+	virtual Vector<uint8_t> texture_get_data(TextureID p_texture, uint32_t p_layer) override final;
 	virtual BitField<TextureUsageBits> texture_get_usages_supported_by_format(DataFormat p_format, bool p_cpu_readable) override final;
 	virtual bool texture_can_make_shared_with_format(TextureID p_texture, DataFormat p_format, bool &r_raw_reinterpretation) override final;
 
@@ -318,6 +354,13 @@ public:
 private:
 	LocalVector<D3D12_SAMPLER_DESC> samplers;
 
+	struct SamplerDescriptorHeapAllocation : DescriptorHeap::Allocation {
+		uint32_t key = 0;
+		uint32_t use_count = 1;
+	};
+
+	RBMap<uint32_t, SamplerDescriptorHeapAllocation> sampler_descriptor_heap_allocations;
+
 public:
 	virtual SamplerID sampler_create(const SamplerState &p_state) final override;
 	virtual void sampler_free(SamplerID p_sampler) final override;
@@ -333,7 +376,7 @@ private:
 	};
 
 public:
-	virtual VertexFormatID vertex_format_create(VectorView<VertexAttribute> p_vertex_attribs) override final;
+	virtual VertexFormatID vertex_format_create(Span<VertexAttribute> p_vertex_attribs, const VertexAttributeBindingsMap &p_vertex_bindings) override final;
 	virtual void vertex_format_free(VertexFormatID p_vertex_format) override final;
 
 	/******************/
@@ -344,7 +387,7 @@ public:
 			CommandBufferID p_cmd_buffer,
 			BitField<PipelineStageBits> p_src_stages,
 			BitField<PipelineStageBits> p_dst_stages,
-			VectorView<RDD::MemoryBarrier> p_memory_barriers,
+			VectorView<RDD::MemoryAccessBarrier> p_memory_barriers,
 			VectorView<RDD::BufferBarrier> p_buffer_barriers,
 			VectorView<RDD::TextureBarrier> p_texture_barriers) override final;
 
@@ -354,7 +397,7 @@ private:
 	/****************/
 
 	struct FenceInfo {
-		ComPtr<ID3D12Fence> d3d_fence = nullptr;
+		Microsoft::WRL::ComPtr<ID3D12Fence> d3d_fence = nullptr;
 		HANDLE event_handle = nullptr;
 		UINT64 fence_value = 0;
 	};
@@ -370,7 +413,7 @@ private:
 	/********************/
 
 	struct SemaphoreInfo {
-		ComPtr<ID3D12Fence> d3d_fence = nullptr;
+		Microsoft::WRL::ComPtr<ID3D12Fence> d3d_fence = nullptr;
 		UINT64 fence_value = 0;
 	};
 
@@ -389,7 +432,7 @@ private:
 	// ----- QUEUE -----
 
 	struct CommandQueueInfo {
-		ComPtr<ID3D12CommandQueue> d3d_queue;
+		Microsoft::WRL::ComPtr<ID3D12CommandQueue> d3d_queue;
 	};
 
 public:
@@ -419,11 +462,21 @@ private:
 	struct FramebufferInfo;
 	struct RenderPassInfo;
 	struct RenderPassState {
+		struct AttachmentLayout {
+			struct AspectLayout {
+				TextureLayout cur_layout = TEXTURE_LAYOUT_UNDEFINED;
+				TextureLayout expected_layout = TEXTURE_LAYOUT_UNDEFINED;
+			};
+
+			AspectLayout aspect_layouts[TEXTURE_ASPECT_MAX];
+		};
+
 		uint32_t current_subpass = UINT32_MAX;
 		const FramebufferInfo *fb_info = nullptr;
 		const RenderPassInfo *pass_info = nullptr;
 		CD3DX12_RECT region_rect = {};
 		bool region_is_all = false;
+		LocalVector<AttachmentLayout> attachment_layouts;
 
 		const VertexFormatInfo *vf_info = nullptr;
 		D3D12_VERTEX_BUFFER_VIEW vertex_buffer_views[8] = {};
@@ -436,8 +489,8 @@ private:
 		// Store a self list reference to be used by the command pool.
 		SelfList<CommandBufferInfo> command_buffer_info_elem{ this };
 
-		ComPtr<ID3D12CommandAllocator> cmd_allocator;
-		ComPtr<ID3D12GraphicsCommandList> cmd_list;
+		Microsoft::WRL::ComPtr<ID3D12CommandAllocator> cmd_allocator;
+		Microsoft::WRL::ComPtr<ID3D12GraphicsCommandList> cmd_list;
 
 		ID3D12PipelineState *graphics_pso = nullptr;
 		ID3D12PipelineState *compute_pso = nullptr;
@@ -452,6 +505,10 @@ private:
 		LocalVector<D3D12_RESOURCE_BARRIER> res_barriers;
 		uint32_t res_barriers_count = 0;
 		uint32_t res_barriers_batch = 0;
+
+		CPUDescriptorHeapPool::Allocation uav_alloc;
+		CPUDescriptorHeapPool::Allocation rtv_alloc;
+		CPUDescriptorHeapPool::Allocation dsv_alloc;
 	};
 
 public:
@@ -467,7 +524,7 @@ private:
 	/********************/
 
 	struct SwapChain {
-		ComPtr<IDXGISwapChain3> d3d_swap_chain;
+		Microsoft::WRL::ComPtr<IDXGISwapChain3> d3d_swap_chain;
 		RenderingContextDriver::SurfaceID surface = RenderingContextDriver::SurfaceID();
 		UINT present_flags = 0;
 		UINT sync_interval = 1;
@@ -497,9 +554,10 @@ private:
 	struct FramebufferInfo {
 		bool is_screen = false;
 		Size2i size;
+
 		TightLocalVector<uint32_t> attachments_handle_inds; // RTV heap index for color; DSV heap index for DSV.
-		DescriptorsHeap rtv_heap;
-		DescriptorsHeap dsv_heap; // Used only if not for screen and some depth-stencil attachments.
+		CPUDescriptorHeapPool::Allocation rtv_alloc;
+		CPUDescriptorHeapPool::Allocation dsv_alloc; // Used only for depth-stencil attachments.
 
 		TightLocalVector<TextureID> attachments; // Color and depth-stencil. Used if not screen.
 		TextureID vrs_attachment;
@@ -507,7 +565,7 @@ private:
 
 	D3D12_RENDER_TARGET_VIEW_DESC _make_rtv_for_texture(const TextureInfo *p_texture_info, uint32_t p_mipmap_offset, uint32_t p_layer_offset, uint32_t p_layers, bool p_add_bases = true);
 	D3D12_UNORDERED_ACCESS_VIEW_DESC _make_ranged_uav_for_texture(const TextureInfo *p_texture_info, uint32_t p_mipmap_offset, uint32_t p_layer_offset, uint32_t p_layers, bool p_add_bases = true);
-	D3D12_DEPTH_STENCIL_VIEW_DESC _make_dsv_for_texture(const TextureInfo *p_texture_info);
+	D3D12_DEPTH_STENCIL_VIEW_DESC _make_dsv_for_texture(const TextureInfo *p_texture_info, uint32_t p_mipmap_offset, uint32_t p_layer_offset, uint32_t p_layers, bool p_add_bases = true);
 
 	FramebufferID _framebuffer_create(RenderPassID p_render_pass, VectorView<TextureID> p_attachments, uint32_t p_width, uint32_t p_height, bool p_is_screen);
 
@@ -546,25 +604,18 @@ private:
 			ResourceClass res_class = RES_CLASS_INVALID;
 			UniformType type = UNIFORM_TYPE_MAX;
 			uint32_t length = UINT32_MAX;
-#ifdef DEV_ENABLED
 			bool writable = false;
-#endif
-			struct RootSignatureLocation {
-				uint32_t root_param_idx = UINT32_MAX;
-				uint32_t range_idx = UINT32_MAX;
-			};
-			struct {
-				RootSignatureLocation resource;
-				RootSignatureLocation sampler;
-			} root_sig_locations;
+			uint32_t resource_descriptor_offset = UINT32_MAX;
+			uint32_t sampler_descriptor_offset = UINT32_MAX;
+			uint32_t root_param_idx = UINT32_MAX;
 		};
 
 		struct UniformSet {
 			TightLocalVector<UniformBindingInfo> bindings;
-			struct {
-				uint32_t resources = 0;
-				uint32_t samplers = 0;
-			} num_root_params;
+			uint32_t resource_root_param_idx = UINT32_MAX;
+			uint32_t resource_descriptor_count = 0;
+			uint32_t sampler_root_param_idx = UINT32_MAX;
+			uint32_t sampler_descriptor_count = 0;
 		};
 
 		TightLocalVector<UniformSet> sets;
@@ -580,8 +631,8 @@ private:
 
 		HashMap<ShaderStage, Vector<uint8_t>> stages_bytecode;
 
-		ComPtr<ID3D12RootSignature> root_signature;
-		ComPtr<ID3D12RootSignatureDeserializer> root_signature_deserializer;
+		Microsoft::WRL::ComPtr<ID3D12RootSignature> root_signature;
+		Microsoft::WRL::ComPtr<ID3D12RootSignatureDeserializer> root_signature_deserializer;
 		const D3D12_ROOT_SIGNATURE_DESC *root_signature_desc = nullptr; // Owned by the deserializer.
 		uint32_t root_signature_crc = 0;
 	};
@@ -602,16 +653,16 @@ public:
 	/*********************/
 
 private:
-	struct RootDescriptorTable {
-		uint32_t root_param_idx = UINT32_MAX;
-		D3D12_GPU_DESCRIPTOR_HANDLE start_gpu_handle = {};
-	};
-
 	struct UniformSetInfo {
-		struct {
-			DescriptorsHeap resources;
-			DescriptorsHeap samplers;
-		} desc_heaps;
+		DescriptorHeap::Allocation resource_descriptor_heap_alloc;
+		SamplerDescriptorHeapAllocation *sampler_descriptor_heap_alloc = nullptr;
+
+		struct DynamicBuffer {
+			BufferDynamicInfo const *info = nullptr;
+			uint32_t binding = UINT_MAX;
+		};
+
+		TightLocalVector<DynamicBuffer> dynamic_buffers;
 
 		struct StateRequirement {
 			ResourceInfo *resource = nullptr;
@@ -619,31 +670,14 @@ private:
 			D3D12_RESOURCE_STATES states = {};
 			uint64_t shader_uniform_idx_mask = 0;
 		};
+
 		TightLocalVector<StateRequirement> resource_states;
-
-		struct RecentBind {
-			uint64_t segment_serial = 0;
-			uint32_t root_signature_crc = 0;
-			struct {
-				TightLocalVector<RootDescriptorTable> resources;
-				TightLocalVector<RootDescriptorTable> samplers;
-			} root_tables;
-			int uses = 0;
-		} recent_binds[4]; // A better amount may be empirically found.
-
-#ifdef DEV_ENABLED
-		// Filthy, but useful for dev.
-		struct ResourceDescInfo {
-			D3D12_DESCRIPTOR_RANGE_TYPE type;
-			D3D12_SRV_DIMENSION srv_dimension;
-		};
-		TightLocalVector<ResourceDescInfo> resources_desc_info;
-#endif
 	};
 
 public:
 	virtual UniformSetID uniform_set_create(VectorView<BoundUniform> p_uniforms, ShaderID p_shader, uint32_t p_set_index, int p_linear_pool_index) override final;
 	virtual void uniform_set_free(UniformSetID p_uniform_set) override final;
+	virtual uint32_t uniform_sets_get_dynamic_offsets(VectorView<UniformSetID> p_uniform_sets, ShaderID p_shader, uint32_t p_first_set_index, uint32_t p_set_count) const override final;
 
 	// ----- COMMANDS -----
 
@@ -651,8 +685,7 @@ public:
 
 private:
 	void _command_check_descriptor_sets(CommandBufferID p_cmd_buffer);
-	void _command_bind_uniform_set(CommandBufferID p_cmd_buffer, UniformSetID p_uniform_set, ShaderID p_shader, uint32_t p_set_index, bool p_for_compute);
-	void _command_bind_uniform_sets(CommandBufferID p_cmd_buffer, VectorView<UniformSetID> p_uniform_sets, ShaderID p_shader, uint32_t p_first_set_index, uint32_t p_set_count, bool p_for_compute);
+	DescriptorHeap::Allocation _command_allocate_per_frame_descriptor();
 
 public:
 	/******************/
@@ -665,6 +698,7 @@ public:
 	virtual void command_copy_texture(CommandBufferID p_cmd_buffer, TextureID p_src_texture, TextureLayout p_src_texture_layout, TextureID p_dst_texture, TextureLayout p_dst_texture_layout, VectorView<TextureCopyRegion> p_regions) override final;
 	virtual void command_resolve_texture(CommandBufferID p_cmd_buffer, TextureID p_src_texture, TextureLayout p_src_texture_layout, uint32_t p_src_layer, uint32_t p_src_mipmap, TextureID p_dst_texture, TextureLayout p_dst_texture_layout, uint32_t p_dst_layer, uint32_t p_dst_mipmap) override final;
 	virtual void command_clear_color_texture(CommandBufferID p_cmd_buffer, TextureID p_texture, TextureLayout p_texture_layout, const Color &p_color, const TextureSubresourceRange &p_subresources) override final;
+	virtual void command_clear_depth_stencil_texture(CommandBufferID p_cmd_buffer, TextureID p_texture, TextureLayout p_texture_layout, float p_depth, uint8_t p_stencil, const TextureSubresourceRange &p_subresources) override final;
 
 public:
 	virtual void command_copy_buffer_to_texture(CommandBufferID p_cmd_buffer, BufferID p_src_buffer, TextureID p_dst_texture, TextureLayout p_dst_texture_layout, VectorView<BufferTextureCopyRegion> p_regions) override final;
@@ -687,7 +721,7 @@ public:
 	};
 
 	struct PipelineInfo {
-		ID3D12PipelineState *pso = nullptr;
+		Microsoft::WRL::ComPtr<ID3D12PipelineState> pso;
 		const ShaderInfo *shader_info = nullptr;
 		RenderPipelineInfo render_info;
 	};
@@ -729,6 +763,7 @@ public:
 	virtual void command_begin_render_pass(CommandBufferID p_cmd_buffer, RenderPassID p_render_pass, FramebufferID p_framebuffer, CommandBufferType p_cmd_buffer_type, const Rect2i &p_rect, VectorView<RenderPassClearValue> p_clear_values) override final;
 
 private:
+	void _render_pass_enhanced_barriers_flush(CommandBufferID p_cmd_buffer);
 	void _end_render_pass(CommandBufferID p_cmd_buffer);
 
 public:
@@ -741,8 +776,7 @@ public:
 
 	// Binding.
 	virtual void command_bind_render_pipeline(CommandBufferID p_cmd_buffer, PipelineID p_pipeline) override final;
-	virtual void command_bind_render_uniform_set(CommandBufferID p_cmd_buffer, UniformSetID p_uniform_set, ShaderID p_shader, uint32_t p_set_index) override final;
-	virtual void command_bind_render_uniform_sets(CommandBufferID p_cmd_buffer, VectorView<UniformSetID> p_uniform_sets, ShaderID p_shader, uint32_t p_first_set_index, uint32_t p_set_count) override final;
+	virtual void command_bind_render_uniform_sets(CommandBufferID p_cmd_buffer, VectorView<UniformSetID> p_uniform_sets, ShaderID p_shader, uint32_t p_first_set_index, uint32_t p_set_count, uint32_t p_dynamic_offsets) override final;
 
 	// Drawing.
 	virtual void command_render_draw(CommandBufferID p_cmd_buffer, uint32_t p_vertex_count, uint32_t p_instance_count, uint32_t p_base_vertex, uint32_t p_first_instance) override final;
@@ -753,7 +787,7 @@ public:
 	virtual void command_render_draw_indirect_count(CommandBufferID p_cmd_buffer, BufferID p_indirect_buffer, uint64_t p_offset, BufferID p_count_buffer, uint64_t p_count_buffer_offset, uint32_t p_max_draw_count, uint32_t p_stride) override final;
 
 	// Buffer binding.
-	virtual void command_render_bind_vertex_buffers(CommandBufferID p_cmd_buffer, uint32_t p_binding_count, const BufferID *p_buffers, const uint64_t *p_offsets) override final;
+	virtual void command_render_bind_vertex_buffers(CommandBufferID p_cmd_buffer, uint32_t p_binding_count, const BufferID *p_buffers, const uint64_t *p_offsets, uint64_t p_dynamic_offsets) override final;
 	virtual void command_render_bind_index_buffer(CommandBufferID p_cmd_buffer, BufferID p_buffer, IndexBufferFormat p_format, uint64_t p_offset) override final;
 
 private:
@@ -789,8 +823,7 @@ public:
 
 	// Binding.
 	virtual void command_bind_compute_pipeline(CommandBufferID p_cmd_buffer, PipelineID p_pipeline) override final;
-	virtual void command_bind_compute_uniform_set(CommandBufferID p_cmd_buffer, UniformSetID p_uniform_set, ShaderID p_shader, uint32_t p_set_index) override final;
-	virtual void command_bind_compute_uniform_sets(CommandBufferID p_cmd_buffer, VectorView<UniformSetID> p_uniform_sets, ShaderID p_shader, uint32_t p_first_set_index, uint32_t p_set_count) override final;
+	virtual void command_bind_compute_uniform_sets(CommandBufferID p_cmd_buffer, VectorView<UniformSetID> p_uniform_sets, ShaderID p_shader, uint32_t p_first_set_index, uint32_t p_set_count, uint32_t p_dynamic_offsets) override final;
 
 	// Dispatching.
 	virtual void command_compute_dispatch(CommandBufferID p_cmd_buffer, uint32_t p_x_groups, uint32_t p_y_groups, uint32_t p_z_groups) override final;
@@ -808,9 +841,9 @@ public:
 
 private:
 	struct TimestampQueryPoolInfo {
-		ComPtr<ID3D12QueryHeap> query_heap;
+		Microsoft::WRL::ComPtr<ID3D12QueryHeap> query_heap;
 		uint32_t query_count = 0;
-		ComPtr<D3D12MA::Allocation> results_buffer_allocation;
+		Microsoft::WRL::ComPtr<D3D12MA::Allocation> results_buffer_allocation;
 	};
 
 public:
@@ -841,35 +874,12 @@ public:
 	/********************/
 private:
 	struct FrameInfo {
-		struct {
-			DescriptorsHeap resources;
-			DescriptorsHeap samplers;
-			DescriptorsHeap aux;
-			DescriptorsHeap rtv;
-		} desc_heaps;
-		struct {
-			DescriptorsHeap::Walker resources;
-			DescriptorsHeap::Walker samplers;
-			DescriptorsHeap::Walker aux;
-			DescriptorsHeap::Walker rtv;
-		} desc_heap_walkers;
-		struct {
-			bool resources = false;
-			bool samplers = false;
-			bool aux = false;
-			bool rtv = false;
-		} desc_heaps_exhausted_reported;
-		CD3DX12_CPU_DESCRIPTOR_HANDLE null_rtv_handle = {}; // For [[MANUAL_SUBPASSES]].
-		uint32_t segment_serial = 0;
-
-#ifdef DEV_ENABLED
-		uint32_t uniform_set_reused = 0;
-#endif
+		LocalVector<DescriptorHeap::Allocation> descriptor_allocations;
+		uint32_t descriptor_allocation_count = 0;
 	};
 	TightLocalVector<FrameInfo> frames;
 	uint32_t frame_idx = 0;
 	uint32_t frames_drawn = 0;
-	uint32_t segment_serial = 0;
 	bool segment_begun = false;
 	HashMap<uint64_t, bool> has_comp_alpha;
 
