@@ -35,7 +35,7 @@
 #include "core/os/mutex.h"
 #include "core/os/thread.h"
 #include "core/templates/safe_refcount.h"
-#include "servers/audio_server.h"
+#include "servers/audio/audio_server.h"
 
 #ifdef SOWRAP_ENABLED
 #include "pulse-so_wrap.h"
