@@ -31,6 +31,7 @@
 #include "editor_profiler.h"
 
 #include "core/io/image.h"
+#include "core/string/translation_server.h"
 #include "editor/editor_string_names.h"
 #include "editor/run/editor_run_bar.h"
 #include "editor/settings/editor_settings.h"
@@ -116,29 +117,31 @@ void EditorProfiler::clear() {
 	emit_signal(SNAME("enable_profiling"), activate->is_pressed());
 }
 
-static String _get_percent_txt(float p_value, float p_total) {
-	if (p_total == 0) {
-		p_total = 0.00001;
-	}
-
-	return TS->format_number(String::num((p_value / p_total) * 100, 1)) + TS->percent_sign();
-}
-
 String EditorProfiler::_get_time_as_text(const Metric &m, float p_time, int p_calls) {
-	const int dmode = display_mode->get_selected();
+	const String &lang = _get_locale();
+	const TranslationServer *ts = TranslationServer::get_singleton();
 
-	if (dmode == DISPLAY_FRAME_TIME) {
-		return TS->format_number(rtos(p_time * 1000).pad_decimals(2)) + " " + TTR("ms");
-	} else if (dmode == DISPLAY_AVERAGE_TIME) {
-		if (p_calls == 0) {
-			return TS->format_number("0.00") + " " + TTR("ms");
-		} else {
-			return TS->format_number(rtos((p_time / p_calls) * 1000).pad_decimals(2)) + " " + TTR("ms");
-		}
-	} else if (dmode == DISPLAY_FRAME_PERCENT) {
-		return _get_percent_txt(p_time, m.frame_time);
-	} else if (dmode == DISPLAY_PHYSICS_FRAME_PERCENT) {
-		return _get_percent_txt(p_time, m.physics_frame_time);
+	switch (display_mode->get_selected()) {
+		case DISPLAY_FRAME_TIME: {
+			return ts->format_number(rtos(p_time * 1000).pad_decimals(2), lang) + " " + TTR("ms");
+		} break;
+
+		case DISPLAY_AVERAGE_TIME: {
+			if (p_calls == 0) {
+				return ts->format_number("0.00", lang) + " " + TTR("ms");
+			}
+			return ts->format_number(rtos((p_time / p_calls) * 1000).pad_decimals(2), lang) + " " + TTR("ms");
+		} break;
+
+		case DISPLAY_FRAME_PERCENT: {
+			float total = m.frame_time == 0 ? 0.00001 : m.frame_time;
+			return ts->format_number(String::num((p_time / total) * 100, 1), lang) + ts->get_percent_sign(lang);
+		} break;
+
+		case DISPLAY_PHYSICS_FRAME_PERCENT: {
+			float total = m.physics_frame_time == 0 ? 0.00001 : m.physics_frame_time;
+			return ts->format_number(String::num((p_time / total) * 100, 1), lang) + ts->get_percent_sign(lang);
+		} break;
 	}
 
 	return "err";
@@ -356,6 +359,7 @@ void EditorProfiler::_update_frame() {
 		category->set_editable(0, true);
 		category->set_metadata(0, m.categories[i].signature);
 		category->set_text(0, String(m.categories[i].name));
+		category->set_auto_translate_mode(0, AUTO_TRANSLATE_MODE_DISABLED);
 		category->set_text(1, _get_time_as_text(m, m.categories[i].total_time, 1));
 
 		if (plot_sigs.has(m.categories[i].signature)) {
@@ -373,6 +377,7 @@ void EditorProfiler::_update_frame() {
 			item->set_cell_mode(0, TreeItem::CELL_MODE_CHECK);
 			item->set_editable(0, true);
 			item->set_text(0, it.name);
+			item->set_auto_translate_mode(0, AUTO_TRANSLATE_MODE_DISABLED);
 			item->set_metadata(0, it.signature);
 			item->set_metadata(1, it.script);
 			item->set_metadata(2, it.line);
@@ -401,10 +406,10 @@ void EditorProfiler::_update_frame() {
 void EditorProfiler::_update_button_text() {
 	if (activate->is_pressed()) {
 		activate->set_button_icon(get_editor_theme_icon(SNAME("Stop")));
-		activate->set_text(TTR("Stop"));
+		activate->set_text(TTRC("Stop"));
 	} else {
 		activate->set_button_icon(get_editor_theme_icon(SNAME("Play")));
-		activate->set_text(TTR("Start"));
+		activate->set_text(TTRC("Start"));
 	}
 }
 
@@ -435,9 +440,14 @@ void EditorProfiler::_autostart_toggled(bool p_toggled_on) {
 
 void EditorProfiler::_notification(int p_what) {
 	switch (p_what) {
-		case NOTIFICATION_LAYOUT_DIRECTION_CHANGED:
-		case NOTIFICATION_THEME_CHANGED:
 		case NOTIFICATION_TRANSLATION_CHANGED: {
+			if (is_ready()) {
+				_update_frame();
+			}
+			[[fallthrough]];
+		}
+		case NOTIFICATION_LAYOUT_DIRECTION_CHANGED:
+		case NOTIFICATION_THEME_CHANGED: {
 			activate->set_button_icon(get_editor_theme_icon(SNAME("Play")));
 			clear_button->set_button_icon(get_editor_theme_icon(SNAME("Clear")));
 
@@ -670,18 +680,18 @@ EditorProfiler::EditorProfiler() {
 	activate = memnew(Button);
 	activate->set_toggle_mode(true);
 	activate->set_disabled(true);
-	activate->set_text(TTR("Start"));
+	activate->set_text(TTRC("Start"));
 	activate->connect(SceneStringName(pressed), callable_mp(this, &EditorProfiler::_activate_pressed));
 	container->add_child(activate);
 
 	clear_button = memnew(Button);
-	clear_button->set_text(TTR("Clear"));
+	clear_button->set_text(TTRC("Clear"));
 	clear_button->connect(SceneStringName(pressed), callable_mp(this, &EditorProfiler::_clear_pressed));
 	clear_button->set_disabled(true);
 	container->add_child(clear_button);
 
 	CheckBox *autostart_checkbox = memnew(CheckBox);
-	autostart_checkbox->set_text(TTR("Autostart"));
+	autostart_checkbox->set_text(TTRC("Autostart"));
 	autostart_checkbox->set_pressed(EditorSettings::get_singleton()->get_project_metadata("debug_options", "autostart_profiler", false));
 	autostart_checkbox->connect(SceneStringName(toggled), callable_mp(this, &EditorProfiler::_autostart_toggled));
 	container->add_child(autostart_checkbox);
@@ -690,14 +700,14 @@ EditorProfiler::EditorProfiler() {
 	hb_measure->add_theme_constant_override(SNAME("separation"), 2 * EDSCALE);
 	container->add_child(hb_measure);
 
-	hb_measure->add_child(memnew(Label(TTR("Measure:"))));
+	hb_measure->add_child(memnew(Label(TTRC("Measure:"))));
 
 	display_mode = memnew(OptionButton);
 	display_mode->set_accessibility_name(TTRC("Measure:"));
-	display_mode->add_item(TTR("Frame Time (ms)"));
-	display_mode->add_item(TTR("Average Time (ms)"));
-	display_mode->add_item(TTR("Frame %"));
-	display_mode->add_item(TTR("Physics Frame %"));
+	display_mode->add_item(TTRC("Frame Time (ms)"));
+	display_mode->add_item(TTRC("Average Time (ms)"));
+	display_mode->add_item(TTRC("Frame %"));
+	display_mode->add_item(TTRC("Physics Frame %"));
 	display_mode->connect(SceneStringName(item_selected), callable_mp(this, &EditorProfiler::_combo_changed));
 
 	hb_measure->add_child(display_mode);
@@ -706,19 +716,19 @@ EditorProfiler::EditorProfiler() {
 	hb_time->add_theme_constant_override(SNAME("separation"), 2 * EDSCALE);
 	container->add_child(hb_time);
 
-	hb_time->add_child(memnew(Label(TTR("Time:"))));
+	hb_time->add_child(memnew(Label(TTRC("Time:"))));
 
 	display_time = memnew(OptionButton);
 	display_time->set_accessibility_name(TTRC("Time:"));
 	// TRANSLATORS: This is an option in the profiler to display the time spent in a function, including the time spent in other functions called by that function.
-	display_time->add_item(TTR("Inclusive"));
+	display_time->add_item(TTRC("Inclusive"));
 	// TRANSLATORS: This is an option in the profiler to display the time spent in a function, exincluding the time spent in other functions called by that function.
-	display_time->add_item(TTR("Self"));
-	display_time->set_tooltip_text(TTR("Inclusive: Includes time from other functions called by this function.\nUse this to spot bottlenecks.\n\nSelf: Only count the time spent in the function itself, not in other functions called by that function.\nUse this to find individual functions to optimize."));
+	display_time->add_item(TTRC("Self"));
+	display_time->set_tooltip_text(TTRC("Inclusive: Includes time from other functions called by this function.\nUse this to spot bottlenecks.\n\nSelf: Only count the time spent in the function itself, not in other functions called by that function.\nUse this to find individual functions to optimize."));
 	display_time->connect(SceneStringName(item_selected), callable_mp(this, &EditorProfiler::_combo_changed));
 	hb_time->add_child(display_time);
 
-	display_internal_profiles = memnew(CheckButton(TTR("Display internal functions")));
+	display_internal_profiles = memnew(CheckButton(TTRC("Display internal functions")));
 	display_internal_profiles->set_visible(EDITOR_GET("debugger/profile_native_calls"));
 	display_internal_profiles->set_pressed(false);
 	display_internal_profiles->connect(SceneStringName(pressed), callable_mp(this, &EditorProfiler::_internal_profiles_pressed));
@@ -729,7 +739,7 @@ EditorProfiler::EditorProfiler() {
 	hb_frame->set_v_size_flags(SIZE_SHRINK_BEGIN);
 	hb->add_child(hb_frame);
 
-	hb_frame->add_child(memnew(Label(TTR("Frame #:"))));
+	hb_frame->add_child(memnew(Label(TTRC("Frame #:"))));
 
 	cursor_metric_edit = memnew(SpinBox);
 	cursor_metric_edit->set_accessibility_name(TTRC("Frame #:"));
@@ -744,22 +754,21 @@ EditorProfiler::EditorProfiler() {
 	h_split->set_v_size_flags(SIZE_EXPAND_FILL);
 
 	variables = memnew(Tree);
-	variables->set_auto_translate_mode(AUTO_TRANSLATE_MODE_DISABLED);
 	variables->set_custom_minimum_size(Size2(320, 0) * EDSCALE);
 	variables->set_hide_folding(true);
 	h_split->add_child(variables);
 	variables->set_hide_root(true);
 	variables->set_columns(3);
 	variables->set_column_titles_visible(true);
-	variables->set_column_title(0, TTR("Name"));
+	variables->set_column_title(0, TTRC("Name"));
 	variables->set_column_expand(0, true);
 	variables->set_column_clip_content(0, true);
 	variables->set_column_custom_minimum_width(0, 60);
-	variables->set_column_title(1, TTR("Time"));
+	variables->set_column_title(1, TTRC("Time"));
 	variables->set_column_expand(1, false);
 	variables->set_column_clip_content(1, true);
 	variables->set_column_custom_minimum_width(1, 75 * EDSCALE);
-	variables->set_column_title(2, TTR("Calls"));
+	variables->set_column_title(2, TTRC("Calls"));
 	variables->set_column_expand(2, false);
 	variables->set_column_clip_content(2, true);
 	variables->set_column_custom_minimum_width(2, 50 * EDSCALE);
