@@ -31,15 +31,16 @@
 #pragma once
 
 #include "core/os/thread.h"
-#include "scene/gui/box_container.h"
+#include "editor/docks/editor_dock.h"
 #include "scene/gui/button.h"
 #include "scene/gui/line_edit.h"
 #include "scene/gui/rich_text_label.h"
 
+class Timer;
 class UndoRedo;
 
-class EditorLog : public HBoxContainer {
-	GDCLASS(EditorLog, HBoxContainer);
+class EditorLog : public EditorDock {
+	GDCLASS(EditorLog, EditorDock);
 
 public:
 	enum MessageType {
@@ -139,8 +140,8 @@ private:
 	Button *show_search_button = nullptr;
 	LineEdit *search_box = nullptr;
 
-	// Reference to the "Output" button on the toolbar so we can update its icon when warnings or errors are encountered.
-	Button *tool_button = nullptr;
+	// Reusable RichTextLabel for BBCode parsing during search
+	RichTextLabel *bbcode_parser = nullptr;
 
 	bool is_loading_state = false; // Used to disable saving requests while loading (some signals from buttons will try to trigger a save, which happens during loading).
 	Timer *save_state_timer = nullptr;
@@ -165,6 +166,7 @@ private:
 
 	void _process_message(const String &p_msg, MessageType p_type, bool p_clear);
 	void _reset_message_counts();
+	void _set_dock_tab_icon(Ref<Texture2D> p_icon);
 
 	void _set_collapse(bool p_collapse);
 
@@ -180,7 +182,6 @@ protected:
 
 public:
 	void add_message(const String &p_msg, MessageType p_type = MSG_TYPE_STD);
-	void set_tool_button(Button *p_tool_button);
 	void register_undo_redo(UndoRedo *p_undo_redo);
 	void deinit();
 
