@@ -46,6 +46,7 @@ class OptionButton;
 class PopupMenu;
 class ProjectExportDialog;
 class RichTextLabel;
+class SpinBox;
 class TabContainer;
 class Tree;
 class TreeItem;
@@ -78,13 +79,15 @@ class ProjectExportDialog : public ConfirmationDialog {
 	Button *delete_preset = nullptr;
 	ItemList *presets = nullptr;
 
+	VBoxContainer *settings_vb = nullptr;
 	LineEdit *name = nullptr;
 	EditorPropertyPath *export_path = nullptr;
 	EditorInspector *parameters = nullptr;
 	CheckButton *runnable = nullptr;
 	CheckButton *advanced_options = nullptr;
 
-	Button *button_export = nullptr;
+	Label *empty_label = nullptr;
+
 	bool updating = false;
 
 	RichTextLabel *result_dialog_log = nullptr;
@@ -107,6 +110,11 @@ class ProjectExportDialog : public ConfirmationDialog {
 
 	RBSet<String> feature_set;
 
+	CheckButton *patch_delta_encoding = nullptr;
+	SpinBox *patch_delta_zstd_level = nullptr;
+	SpinBox *patch_delta_min_reduction = nullptr;
+	LineEdit *patch_delta_include_filter = nullptr;
+	LineEdit *patch_delta_exclude_filter = nullptr;
 	Tree *patches = nullptr;
 	int patch_index = -1;
 	EditorFileDialog *patch_dialog = nullptr;
@@ -117,6 +125,7 @@ class ProjectExportDialog : public ConfirmationDialog {
 	RichTextLabel *custom_feature_display = nullptr;
 
 	LineEdit *script_key = nullptr;
+	Button *show_script_key = nullptr;
 	Label *script_key_error = nullptr;
 
 	ProjectExportTextureFormatError *export_texture_format_error = nullptr;
@@ -132,6 +141,7 @@ class ProjectExportDialog : public ConfirmationDialog {
 	void _runnable_pressed();
 	void _update_parameters(const String &p_edited_property);
 	void _name_changed(const String &p_string);
+	void _name_editing_finished();
 	void _export_path_changed(const StringName &p_property, const Variant &p_value, const String &p_field, bool p_changing);
 	void _add_preset(int p_platform);
 	void _edit_preset(int p_index);
@@ -156,6 +166,12 @@ class ProjectExportDialog : public ConfirmationDialog {
 	void _tree_popup_edited(bool p_arrow_clicked);
 	void _set_file_export_mode(int p_id);
 
+	bool updating_patch_delta_filters = false;
+	void _patch_delta_encoding_changed(bool p_pressed);
+	void _patch_delta_include_filter_changed(const String &p_filter);
+	void _patch_delta_exclude_filter_changed(const String &p_filter);
+	void _patch_delta_zstd_level_changed(double p_value);
+	void _patch_delta_min_reduction_changed(double p_value);
 	void _patch_tree_button_clicked(Object *p_item, int p_column, int p_id, int p_mouse_button_index);
 	void _patch_tree_item_edited();
 	void _patch_file_selected(const String &p_path);
@@ -182,7 +198,6 @@ class ProjectExportDialog : public ConfirmationDialog {
 	void _export_pck_zip();
 	void _export_pck_zip_selected(const String &p_path);
 
-	void _validate_export_path(const String &p_path);
 	void _export_project();
 	void _export_project_to_path(const String &p_path);
 	void _export_all_dialog();
@@ -200,9 +215,10 @@ class ProjectExportDialog : public ConfirmationDialog {
 	void _enc_filters_changed(const String &p_text);
 	void _seed_input_changed(const String &p_text);
 	void _script_encryption_key_changed(const String &p_key);
+	void _script_encryption_key_visibility_changed(bool p_visible);
 	bool _validate_script_encryption_key(const String &p_key);
 
-	void _script_export_mode_changed(int p_mode);
+	void _script_export_mode_changed(EditorExportPreset::ScriptExportMode p_mode);
 
 	void _open_key_help_link();
 
