@@ -41,7 +41,6 @@ class EditorSpinSlider : public Range {
 	String suffix;
 	int updown_offset = -1;
 	bool hover_updown = false;
-	bool mouse_hover = false;
 
 	TextureRect *grabber = nullptr;
 	int grabber_range = 1;
@@ -63,12 +62,20 @@ class EditorSpinSlider : public Range {
 	Vector2 grabbing_spinner_mouse_pos;
 	double pre_grab_value = 0.0;
 
-	Control *value_input_popup = nullptr;
 	LineEdit *value_input = nullptr;
 	uint64_t value_input_closed_frame = 0;
 	bool value_input_dirty = false;
+	bool value_input_focus_visible = false;
 
-	bool hide_slider = false;
+public:
+	enum ControlState {
+		CONTROL_STATE_DEFAULT,
+		CONTROL_STATE_PREFER_SLIDER,
+		CONTROL_STATE_HIDE,
+	};
+
+private:
+	ControlState control_state = CONTROL_STATE_DEFAULT;
 	bool flat = false;
 	bool editing_integer = false;
 
@@ -76,7 +83,7 @@ class EditorSpinSlider : public Range {
 	void _grab_end();
 
 	void _grabber_gui_input(const Ref<InputEvent> &p_event);
-	void _value_input_closed();
+	void _value_input_hidden();
 	void _value_input_submitted(const String &);
 	void _value_focus_exited();
 	void _value_input_gui_input(const Ref<InputEvent> &p_event);
@@ -84,7 +91,7 @@ class EditorSpinSlider : public Range {
 	void _evaluate_input_text();
 
 	void _update_value_input_stylebox();
-	void _ensure_input_popup();
+	void _ensure_value_input();
 	void _draw_spin_slider();
 
 	struct ThemeCache {
@@ -98,11 +105,13 @@ protected:
 	static void _bind_methods();
 	void _grabber_mouse_entered();
 	void _grabber_mouse_exited();
-	void _focus_entered();
+	void _focus_entered(bool p_hide_focus = false);
 
 public:
 	String get_tooltip(const Point2 &p_pos) const override;
 
+	virtual Size2 get_minimum_size() const override;
+
 	String get_text_value() const;
 	void set_label(const String &p_label);
 	String get_label() const;
@@ -110,8 +119,13 @@ public:
 	void set_suffix(const String &p_suffix);
 	String get_suffix() const;
 
+	void set_control_state(ControlState p_type);
+	ControlState get_control_state() const;
+
+#ifndef DISABLE_DEPRECATED
 	void set_hide_slider(bool p_hide);
 	bool is_hiding_slider() const;
+#endif
 
 	void set_editing_integer(bool p_editing_integer);
 	bool is_editing_integer() const;
@@ -127,6 +141,7 @@ public:
 	void setup_and_show() { _focus_entered(); }
 	LineEdit *get_line_edit();
 
-	virtual Size2 get_minimum_size() const override;
 	EditorSpinSlider();
 };
+
+VARIANT_ENUM_CAST(EditorSpinSlider::ControlState)
