@@ -76,7 +76,7 @@ bool TileSetScenesCollectionSourceEditor::TileSetScenesCollectionProxyObject::_s
 }
 
 bool TileSetScenesCollectionSourceEditor::TileSetScenesCollectionProxyObject::_get(const StringName &p_name, Variant &r_ret) const {
-	if (!tile_set_scenes_collection_source) {
+	if (tile_set_scenes_collection_source.is_null()) {
 		return false;
 	}
 	String name = p_name;
@@ -114,7 +114,7 @@ void TileSetScenesCollectionSourceEditor::TileSetScenesCollectionProxyObject::ed
 	}
 
 	// Disconnect to changes.
-	if (tile_set_scenes_collection_source) {
+	if (tile_set_scenes_collection_source.is_valid()) {
 		tile_set_scenes_collection_source->disconnect(CoreStringName(property_list_changed), callable_mp((Object *)this, &Object::notify_property_list_changed));
 	}
 
@@ -123,7 +123,7 @@ void TileSetScenesCollectionSourceEditor::TileSetScenesCollectionProxyObject::ed
 	source_id = p_source_id;
 
 	// Connect to changes.
-	if (tile_set_scenes_collection_source) {
+	if (tile_set_scenes_collection_source.is_valid()) {
 		if (!tile_set_scenes_collection_source->is_connected(CoreStringName(property_list_changed), callable_mp((Object *)this, &Object::notify_property_list_changed))) {
 			tile_set_scenes_collection_source->connect(CoreStringName(property_list_changed), callable_mp((Object *)this, &Object::notify_property_list_changed));
 		}
@@ -134,7 +134,7 @@ void TileSetScenesCollectionSourceEditor::TileSetScenesCollectionProxyObject::ed
 
 // -- Proxy object used by the tile inspector --
 bool TileSetScenesCollectionSourceEditor::SceneTileProxyObject::_set(const StringName &p_name, const Variant &p_value) {
-	if (!tile_set_scenes_collection_source) {
+	if (tile_set_scenes_collection_source.is_null()) {
 		return false;
 	}
 
@@ -166,7 +166,7 @@ bool TileSetScenesCollectionSourceEditor::SceneTileProxyObject::_set(const Strin
 }
 
 bool TileSetScenesCollectionSourceEditor::SceneTileProxyObject::_get(const StringName &p_name, Variant &r_ret) const {
-	if (!tile_set_scenes_collection_source) {
+	if (tile_set_scenes_collection_source.is_null()) {
 		return false;
 	}
 
@@ -185,7 +185,7 @@ bool TileSetScenesCollectionSourceEditor::SceneTileProxyObject::_get(const Strin
 }
 
 void TileSetScenesCollectionSourceEditor::SceneTileProxyObject::_get_property_list(List<PropertyInfo> *p_list) const {
-	if (!tile_set_scenes_collection_source) {
+	if (tile_set_scenes_collection_source.is_null()) {
 		return;
 	}
 
@@ -222,11 +222,9 @@ void TileSetScenesCollectionSourceEditor::_tile_set_scenes_collection_source_cha
 	tile_set_scenes_collection_source_changed_needs_update = true;
 }
 
-void TileSetScenesCollectionSourceEditor::_scene_thumbnail_done(const String &p_path, const Ref<Texture2D> &p_preview, const Ref<Texture2D> &p_small_preview, const Variant &p_ud) {
-	int index = p_ud;
-
-	if (index >= 0 && index < scene_tiles_list->get_item_count()) {
-		scene_tiles_list->set_item_icon(index, p_preview);
+void TileSetScenesCollectionSourceEditor::_scene_thumbnail_done(const String &p_path, const Ref<Texture2D> &p_preview, const Ref<Texture2D> &p_small_preview, int p_idx) {
+	if (p_idx >= 0 && p_idx < scene_tiles_list->get_item_count()) {
+		scene_tiles_list->set_item_icon(p_idx, p_preview);
 	}
 }
 
@@ -257,12 +255,11 @@ void TileSetScenesCollectionSourceEditor::_scene_file_selected(const String &p_p
 	int scene_id = tile_set_scenes_collection_source->get_next_scene_tile_id();
 	EditorUndoRedoManager *undo_redo = EditorUndoRedoManager::get_singleton();
 	undo_redo->create_action(TTR("Add a Scene Tile"));
-	undo_redo->add_do_method(tile_set_scenes_collection_source, "create_scene_tile", scene, scene_id);
-	undo_redo->add_undo_method(tile_set_scenes_collection_source, "remove_scene_tile", scene_id);
+	undo_redo->add_do_method(*tile_set_scenes_collection_source, "create_scene_tile", scene, scene_id);
+	undo_redo->add_do_method(this, "_update_all");
+	undo_redo->add_undo_method(*tile_set_scenes_collection_source, "remove_scene_tile", scene_id);
+	undo_redo->add_undo_method(this, "_update_all");
 	undo_redo->commit_action();
-	_update_scenes_list();
-	_update_action_buttons();
-	_update_tile_inspector();
 }
 
 void TileSetScenesCollectionSourceEditor::_source_delete_pressed() {
@@ -272,17 +269,16 @@ void TileSetScenesCollectionSourceEditor::_source_delete_pressed() {
 
 	EditorUndoRedoManager *undo_redo = EditorUndoRedoManager::get_singleton();
 	undo_redo->create_action(TTR("Remove a Scene Tile"));
-	undo_redo->add_do_method(tile_set_scenes_collection_source, "remove_scene_tile", scene_id);
-	undo_redo->add_undo_method(tile_set_scenes_collection_source, "create_scene_tile", tile_set_scenes_collection_source->get_scene_tile_scene(scene_id), scene_id);
+	undo_redo->add_do_method(*tile_set_scenes_collection_source, "remove_scene_tile", scene_id);
+	undo_redo->add_do_method(this, "_update_all");
+	undo_redo->add_undo_method(*tile_set_scenes_collection_source, "create_scene_tile", tile_set_scenes_collection_source->get_scene_tile_scene(scene_id), scene_id);
+	undo_redo->add_undo_method(this, "_update_all");
 	undo_redo->commit_action();
-	_update_scenes_list();
-	_update_action_buttons();
-	_update_tile_inspector();
 }
 
 void TileSetScenesCollectionSourceEditor::_update_source_inspector() {
 	// Update the proxy object.
-	scenes_collection_source_proxy_object->edit(tile_set, tile_set_scenes_collection_source, tile_set_source_id);
+	scenes_collection_source_proxy_object->edit(tile_set, *tile_set_scenes_collection_source, tile_set_source_id);
 }
 
 void TileSetScenesCollectionSourceEditor::_update_tile_inspector() {
@@ -292,7 +288,7 @@ void TileSetScenesCollectionSourceEditor::_update_tile_inspector() {
 	// Update the proxy object.
 	if (has_atlas_tile_selected) {
 		int scene_id = scene_tiles_list->get_item_metadata(selected_indices[0]);
-		tile_proxy_object->edit(tile_set_scenes_collection_source, scene_id);
+		tile_proxy_object->edit(*tile_set_scenes_collection_source, scene_id);
 	}
 
 	// Update visibility.
@@ -305,8 +301,14 @@ void TileSetScenesCollectionSourceEditor::_update_action_buttons() {
 	scene_tile_delete_button->set_disabled(selected_indices.is_empty() || read_only);
 }
 
+void TileSetScenesCollectionSourceEditor::_update_all() {
+	_update_scenes_list();
+	_update_action_buttons();
+	_update_tile_inspector();
+}
+
 void TileSetScenesCollectionSourceEditor::_update_scenes_list() {
-	if (!tile_set_scenes_collection_source) {
+	if (tile_set_scenes_collection_source.is_null()) {
 		return;
 	}
 
@@ -327,8 +329,7 @@ void TileSetScenesCollectionSourceEditor::_update_scenes_list() {
 		int item_index = 0;
 		if (scene.is_valid()) {
 			item_index = scene_tiles_list->add_item(vformat("%s (path:%s id:%d)", scene->get_path().get_file().get_basename(), scene->get_path(), scene_id));
-			Variant udata = i;
-			EditorResourcePreview::get_singleton()->queue_edited_resource_preview(scene, this, "_scene_thumbnail_done", udata);
+			EditorResourcePreview::get_singleton()->queue_edited_resource_preview(scene, callable_mp(this, &TileSetScenesCollectionSourceEditor::_scene_thumbnail_done).bind(i));
 		} else {
 			item_index = scene_tiles_list->add_item(TTR("Tile with Invalid Scene"), get_editor_theme_icon(SNAME("PackedScene")));
 		}
@@ -408,12 +409,12 @@ void TileSetScenesCollectionSourceEditor::edit(Ref<TileSet> p_tile_set, TileSetS
 		new_read_only_state = EditorNode::get_singleton()->is_resource_read_only(p_tile_set);
 	}
 
-	if (p_tile_set == tile_set && p_tile_set_scenes_collection_source == tile_set_scenes_collection_source && p_source_id == tile_set_source_id && new_read_only_state == read_only) {
+	if (p_tile_set == tile_set && p_tile_set_scenes_collection_source == *tile_set_scenes_collection_source && p_source_id == tile_set_source_id && new_read_only_state == read_only) {
 		return;
 	}
 
 	// Remove listener for old objects.
-	if (tile_set_scenes_collection_source) {
+	if (tile_set_scenes_collection_source.is_valid()) {
 		tile_set_scenes_collection_source->disconnect_changed(callable_mp(this, &TileSetScenesCollectionSourceEditor::_tile_set_scenes_collection_source_changed));
 	}
 
@@ -433,7 +434,7 @@ void TileSetScenesCollectionSourceEditor::edit(Ref<TileSet> p_tile_set, TileSetS
 	}
 
 	// Add the listener again.
-	if (tile_set_scenes_collection_source) {
+	if (tile_set_scenes_collection_source.is_valid()) {
 		tile_set_scenes_collection_source->connect_changed(callable_mp(this, &TileSetScenesCollectionSourceEditor::_tile_set_scenes_collection_source_changed));
 	}
 
@@ -452,22 +453,20 @@ void TileSetScenesCollectionSourceEditor::_drop_data_fw(const Point2 &p_point, c
 	if (p_from == scene_tiles_list) {
 		// Handle dropping a texture in the list of atlas resources.
 		Dictionary d = p_data;
-		Vector<String> files = d["files"];
-		for (int i = 0; i < files.size(); i++) {
-			Ref<PackedScene> resource = ResourceLoader::load(files[i]);
+		const Vector<String> files = d["files"];
+		for (const String &file : files) {
+			const Ref<PackedScene> resource = ResourceLoader::load(file);
 			if (resource.is_valid()) {
 				int scene_id = tile_set_scenes_collection_source->get_next_scene_tile_id();
 				EditorUndoRedoManager *undo_redo = EditorUndoRedoManager::get_singleton();
 				undo_redo->create_action(TTR("Add a Scene Tile"));
-				undo_redo->add_do_method(tile_set_scenes_collection_source, "create_scene_tile", resource, scene_id);
-				undo_redo->add_undo_method(tile_set_scenes_collection_source, "remove_scene_tile", scene_id);
+				undo_redo->add_do_method(*tile_set_scenes_collection_source, "create_scene_tile", resource, scene_id);
+				undo_redo->add_do_method(this, "_update_all");
+				undo_redo->add_undo_method(*tile_set_scenes_collection_source, "remove_scene_tile", scene_id);
+				undo_redo->add_undo_method(this, "_update_all");
 				undo_redo->commit_action();
 			}
 		}
-
-		_update_scenes_list();
-		_update_action_buttons();
-		_update_tile_inspector();
 	}
 }
 
@@ -502,9 +501,9 @@ bool TileSetScenesCollectionSourceEditor::_can_drop_data_fw(const Point2 &p_poin
 }
 
 void TileSetScenesCollectionSourceEditor::_bind_methods() {
-	ADD_SIGNAL(MethodInfo("source_id_changed", PropertyInfo(Variant::INT, "source_id")));
+	ClassDB::bind_method(D_METHOD("_update_all"), &TileSetScenesCollectionSourceEditor::_update_all);
 
-	ClassDB::bind_method(D_METHOD("_scene_thumbnail_done"), &TileSetScenesCollectionSourceEditor::_scene_thumbnail_done);
+	ADD_SIGNAL(MethodInfo("source_id_changed", PropertyInfo(Variant::INT, "source_id")));
 }
 
 TileSetScenesCollectionSourceEditor::TileSetScenesCollectionSourceEditor() {
@@ -561,6 +560,7 @@ TileSetScenesCollectionSourceEditor::TileSetScenesCollectionSourceEditor() {
 	scene_tiles_list->set_auto_translate_mode(AUTO_TRANSLATE_MODE_DISABLED);
 	scene_tiles_list->set_h_size_flags(SIZE_EXPAND_FILL);
 	scene_tiles_list->set_v_size_flags(SIZE_EXPAND_FILL);
+	scene_tiles_list->set_theme_type_variation("ItemListSecondary");
 	SET_DRAG_FORWARDING_CDU(scene_tiles_list, TileSetScenesCollectionSourceEditor);
 	scene_tiles_list->connect(SceneStringName(item_selected), callable_mp(this, &TileSetScenesCollectionSourceEditor::_update_tile_inspector).unbind(1));
 	scene_tiles_list->connect(SceneStringName(item_selected), callable_mp(this, &TileSetScenesCollectionSourceEditor::_update_action_buttons).unbind(1));
