@@ -63,8 +63,13 @@ void BonePropertiesEditor::create_editors() {
 	section->get_vbox()->add_child(enabled_checkbox);
 
 	// Position property.
+	EditorPropertyRangeHint large_range_hint;
+	large_range_hint.min = -10000;
+	large_range_hint.max = 10000;
+	large_range_hint.step = 0.001;
+
 	position_property = memnew(EditorPropertyVector3());
-	position_property->setup(-10000, 10000, 0.001, true);
+	position_property->setup(large_range_hint);
 	position_property->set_label("Position");
 	position_property->set_selectable(false);
 	position_property->connect("property_changed", callable_mp(this, &BonePropertiesEditor::_value_changed));
@@ -73,7 +78,7 @@ void BonePropertiesEditor::create_editors() {
 
 	// Rotation property.
 	rotation_property = memnew(EditorPropertyQuaternion());
-	rotation_property->setup(-10000, 10000, 0.001, true);
+	rotation_property->setup(large_range_hint);
 	rotation_property->set_label("Rotation");
 	rotation_property->set_selectable(false);
 	rotation_property->connect("property_changed", callable_mp(this, &BonePropertiesEditor::_value_changed));
@@ -82,7 +87,7 @@ void BonePropertiesEditor::create_editors() {
 
 	// Scale property.
 	scale_property = memnew(EditorPropertyVector3());
-	scale_property->setup(-10000, 10000, 0.001, true, true);
+	scale_property->setup(large_range_hint, true);
 	scale_property->set_label("Scale");
 	scale_property->set_selectable(false);
 	scale_property->connect("property_changed", callable_mp(this, &BonePropertiesEditor::_value_changed));
@@ -96,7 +101,7 @@ void BonePropertiesEditor::create_editors() {
 
 	// Transform/Matrix property.
 	rest_matrix = memnew(EditorPropertyTransform3D());
-	rest_matrix->setup(-10000, 10000, 0.001, true);
+	rest_matrix->setup(large_range_hint);
 	rest_matrix->set_label("Transform");
 	rest_matrix->set_selectable(false);
 	rest_section->get_vbox()->add_child(rest_matrix);
@@ -217,9 +222,9 @@ void BonePropertiesEditor::_add_meta_confirm() {
 	undo_redo->commit_action();
 }
 
-BonePropertiesEditor::BonePropertiesEditor(Skeleton3D *p_skeleton) :
-		skeleton(p_skeleton) {
+BonePropertiesEditor::BonePropertiesEditor(Skeleton3D *p_skeleton) {
 	create_editors();
+	set_skeleton(p_skeleton);
 }
 
 void BonePropertiesEditor::set_keyable(const bool p_keyable) {
@@ -228,6 +233,24 @@ void BonePropertiesEditor::set_keyable(const bool p_keyable) {
 	scale_property->set_keying(p_keyable);
 }
 
+void BonePropertiesEditor::set_skeleton(Skeleton3D *p_skeleton) {
+	skeleton = p_skeleton;
+
+	if (skeleton) {
+		return;
+	}
+
+	enabled_checkbox->set_object_and_property(nullptr, String());
+	position_property->set_object_and_property(nullptr, String());
+	rotation_property->set_object_and_property(nullptr, String());
+	scale_property->set_object_and_property(nullptr, String());
+	rest_matrix->set_object_and_property(nullptr, String());
+
+	for (KeyValue<StringName, EditorProperty *> &E : meta_editors) {
+		E.value->set_object_and_property(nullptr, String());
+	}
+}
+
 void BonePropertiesEditor::set_target(const String &p_prop) {
 	enabled_checkbox->set_object_and_property(skeleton, p_prop + "enabled");
 	enabled_checkbox->update_property();
@@ -267,7 +290,7 @@ void BonePropertiesEditor::_property_keyed(const String &p_path, bool p_advance)
 }
 
 void BonePropertiesEditor::_update_properties() {
-	if (!skeleton) {
+	if (!skeleton || !Skeleton3DEditor::get_singleton()) {
 		return;
 	}
 	int selected = Skeleton3DEditor::get_singleton()->get_selected_bone();
@@ -361,9 +384,7 @@ void Skeleton3DEditor::_bind_methods() {
 }
 
 void Skeleton3DEditor::_on_click_skeleton_option(int p_skeleton_option) {
-	if (!skeleton) {
-		return;
-	}
+	ERR_FAIL_COND(!skeleton);
 
 	switch (p_skeleton_option) {
 		case SKELETON_OPTION_RESET_ALL_POSES: {
@@ -394,9 +415,8 @@ void Skeleton3DEditor::_on_click_skeleton_option(int p_skeleton_option) {
 }
 
 void Skeleton3DEditor::reset_pose(const bool p_all_bones) {
-	if (!skeleton) {
-		return;
-	}
+	ERR_FAIL_COND(!skeleton);
+
 	const int bone_count = skeleton->get_bone_count();
 	if (!bone_count) {
 		return;
@@ -429,10 +449,34 @@ void Skeleton3DEditor::reset_pose(const bool p_all_bones) {
 	ur->commit_action();
 }
 
-void Skeleton3DEditor::insert_keys(const bool p_all_bones) {
-	if (!skeleton) {
+void Skeleton3DEditor::insert_keys(const bool p_all_bones, const bool p_enable_modifier) {
+	ERR_FAIL_COND(!skeleton);
+
+	AnimationTrackEditor *te = AnimationPlayerEditor::get_singleton()->get_track_editor();
+	bool is_read_only = te->is_read_only();
+	if (is_read_only) {
+		te->popup_read_only_dialog();
 		return;
 	}
+	if (p_enable_modifier) {
+		if (!skeleton->is_connected(SceneStringName(skeleton_updated), callable_mp(this, &Skeleton3DEditor::_insert_keys).bind(p_all_bones))) {
+			skeleton->connect(SceneStringName(skeleton_updated), callable_mp(this, &Skeleton3DEditor::_insert_keys).bind(p_all_bones), CONNECT_ONE_SHOT);
+		} else {
+			WARN_PRINT_ED("A skeleton_updated signal is already connected with _insert_keys().");
+		}
+		skeleton->force_update_deferred();
+		skeleton->notification(Skeleton3D::NOTIFICATION_UPDATE_SKELETON);
+		// Force disconnecting signal if remain the connecting just in case.
+		if (skeleton->is_connected(SceneStringName(skeleton_updated), callable_mp(this, &Skeleton3DEditor::_insert_keys).bind(p_all_bones))) {
+			skeleton->disconnect(SceneStringName(skeleton_updated), callable_mp(this, &Skeleton3DEditor::_insert_keys).bind(p_all_bones));
+		}
+	} else {
+		_insert_keys(p_all_bones);
+	}
+}
+
+void Skeleton3DEditor::_insert_keys(const bool p_all_bones) {
+	ERR_FAIL_COND(!skeleton);
 
 	bool pos_enabled = key_loc_button->is_pressed();
 	bool rot_enabled = key_rot_button->is_pressed();
@@ -465,9 +509,8 @@ void Skeleton3DEditor::insert_keys(const bool p_all_bones) {
 }
 
 void Skeleton3DEditor::pose_to_rest(const bool p_all_bones) {
-	if (!skeleton) {
-		return;
-	}
+	ERR_FAIL_COND(!skeleton);
+
 	const int bone_count = skeleton->get_bone_count();
 	if (!bone_count) {
 		return;
@@ -999,7 +1042,7 @@ void Skeleton3DEditor::create_editors() {
 	// Create Skeleton Option in Top Menu Bar.
 	skeleton_options = memnew(MenuButton);
 	skeleton_options->set_flat(false);
-	skeleton_options->set_theme_type_variation("FlatMenuButton");
+	skeleton_options->set_theme_type_variation("FlatMenuButtonNoIconTint");
 	topmenu_bar->add_child(skeleton_options);
 
 	skeleton_options->set_text(TTR("Skeleton3D"));
@@ -1043,42 +1086,67 @@ void Skeleton3DEditor::create_editors() {
 	key_loc_button = memnew(Button);
 	key_loc_button->set_theme_type_variation(SceneStringName(FlatButton));
 	key_loc_button->set_toggle_mode(true);
-	key_loc_button->set_pressed(false);
+	key_loc_button->set_pressed(editor_plugin->loc_pressed);
 	key_loc_button->set_focus_mode(FOCUS_ACCESSIBILITY);
 	key_loc_button->set_tooltip_text(TTR("Translation mask for inserting keys."));
 	animation_hb->add_child(key_loc_button);
+	if (!key_loc_button->is_connected(SceneStringName(toggled), callable_mp(this, &Skeleton3DEditor::_loc_toggled))) {
+		key_loc_button->connect(SceneStringName(toggled), callable_mp(this, &Skeleton3DEditor::_loc_toggled));
+	}
 
 	key_rot_button = memnew(Button);
 	key_rot_button->set_theme_type_variation(SceneStringName(FlatButton));
 	key_rot_button->set_toggle_mode(true);
-	key_rot_button->set_pressed(true);
+	key_rot_button->set_pressed(editor_plugin->rot_pressed);
 	key_rot_button->set_focus_mode(FOCUS_ACCESSIBILITY);
 	key_rot_button->set_tooltip_text(TTR("Rotation mask for inserting keys."));
 	animation_hb->add_child(key_rot_button);
+	if (!key_rot_button->is_connected(SceneStringName(toggled), callable_mp(this, &Skeleton3DEditor::_rot_toggled))) {
+		key_rot_button->connect(SceneStringName(toggled), callable_mp(this, &Skeleton3DEditor::_rot_toggled));
+	}
 
 	key_scale_button = memnew(Button);
 	key_scale_button->set_theme_type_variation(SceneStringName(FlatButton));
 	key_scale_button->set_toggle_mode(true);
-	key_scale_button->set_pressed(false);
+	key_scale_button->set_pressed(editor_plugin->scl_pressed);
 	key_scale_button->set_focus_mode(FOCUS_ACCESSIBILITY);
 	key_scale_button->set_tooltip_text(TTR("Scale mask for inserting keys."));
 	animation_hb->add_child(key_scale_button);
+	if (!key_scale_button->is_connected(SceneStringName(toggled), callable_mp(this, &Skeleton3DEditor::_scl_toggled))) {
+		key_scale_button->connect(SceneStringName(toggled), callable_mp(this, &Skeleton3DEditor::_scl_toggled));
+	}
 
 	key_insert_button = memnew(Button);
 	key_insert_button->set_theme_type_variation(SceneStringName(FlatButton));
 	key_insert_button->set_focus_mode(FOCUS_ACCESSIBILITY);
-	key_insert_button->connect(SceneStringName(pressed), callable_mp(this, &Skeleton3DEditor::insert_keys).bind(false));
+	key_insert_button->connect(SceneStringName(pressed), callable_mp(this, &Skeleton3DEditor::insert_keys).bind(false, false));
 	key_insert_button->set_tooltip_text(TTRC("Insert key (based on mask) for bones with an existing track."));
 	key_insert_button->set_shortcut(ED_SHORTCUT("skeleton_3d_editor/insert_key_to_existing_tracks", TTRC("Insert Key (Existing Tracks)"), Key::INSERT));
 	animation_hb->add_child(key_insert_button);
 
-	key_insert_all_button = memnew(Button);
-	key_insert_all_button->set_theme_type_variation(SceneStringName(FlatButton));
-	key_insert_all_button->set_focus_mode(FOCUS_ACCESSIBILITY);
-	key_insert_all_button->connect(SceneStringName(pressed), callable_mp(this, &Skeleton3DEditor::insert_keys).bind(true));
-	key_insert_all_button->set_tooltip_text(TTRC("Insert key (based on mask) for all bones."));
-	key_insert_all_button->set_shortcut(ED_SHORTCUT("skeleton_3d_editor/insert_key_of_all_bones", TTRC("Insert Key (All Bones)"), KeyModifierMask::CMD_OR_CTRL + Key::INSERT));
-	animation_hb->add_child(key_insert_all_button);
+	key_insert_new_button = memnew(Button);
+	key_insert_new_button->set_theme_type_variation(SceneStringName(FlatButton));
+	key_insert_new_button->set_focus_mode(FOCUS_ACCESSIBILITY);
+	key_insert_new_button->connect(SceneStringName(pressed), callable_mp(this, &Skeleton3DEditor::insert_keys).bind(true, false));
+	key_insert_new_button->set_tooltip_text(TTRC("Insert key (based on mask) for all bones."));
+	key_insert_new_button->set_shortcut(ED_SHORTCUT("skeleton_3d_editor/insert_key_of_all_bones", TTRC("Insert Key (All Bones)"), KeyModifierMask::CMD_OR_CTRL + Key::INSERT));
+	animation_hb->add_child(key_insert_new_button);
+
+	key_mod_insert_button = memnew(Button);
+	key_mod_insert_button->set_theme_type_variation(SceneStringName(FlatButton));
+	key_mod_insert_button->set_focus_mode(FOCUS_ACCESSIBILITY);
+	key_mod_insert_button->connect(SceneStringName(pressed), callable_mp(this, &Skeleton3DEditor::insert_keys).bind(false, true));
+	key_mod_insert_button->set_tooltip_text(TTRC("Insert key (based on mask) for modified bones with an existing track."));
+	key_mod_insert_button->set_shortcut(ED_SHORTCUT("skeleton_3d_editor/insert_key_to_existing_tracks", TTRC("Insert Key (Existing Tracks)"), Key::INSERT));
+	animation_hb->add_child(key_mod_insert_button);
+
+	key_mod_insert_new_button = memnew(Button);
+	key_mod_insert_new_button->set_theme_type_variation(SceneStringName(FlatButton));
+	key_mod_insert_new_button->set_focus_mode(FOCUS_ACCESSIBILITY);
+	key_mod_insert_new_button->connect(SceneStringName(pressed), callable_mp(this, &Skeleton3DEditor::insert_keys).bind(true, true));
+	key_mod_insert_new_button->set_tooltip_text(TTRC("Insert new key (based on mask) for all modified bones."));
+	key_mod_insert_new_button->set_shortcut(ED_SHORTCUT("skeleton_3d_editor/insert_key_of_all_bones", TTRC("Insert Key (All Bones)"), KeyModifierMask::CMD_OR_CTRL + Key::INSERT));
+	animation_hb->add_child(key_mod_insert_new_button);
 
 	// Bone tree.
 	bones_section = memnew(EditorInspectorSection);
@@ -1112,6 +1180,25 @@ void Skeleton3DEditor::create_editors() {
 	set_keyable(te->has_keying());
 }
 
+void Skeleton3DEditor::_loc_toggled(bool p_toggled_on) {
+	if (!editor_plugin) {
+		return;
+	}
+	editor_plugin->loc_pressed = p_toggled_on;
+}
+void Skeleton3DEditor::_rot_toggled(bool p_toggled_on) {
+	if (!editor_plugin) {
+		return;
+	}
+	editor_plugin->rot_pressed = p_toggled_on;
+}
+void Skeleton3DEditor::_scl_toggled(bool p_toggled_on) {
+	if (!editor_plugin) {
+		return;
+	}
+	editor_plugin->scl_pressed = p_toggled_on;
+}
+
 void Skeleton3DEditor::_notification(int p_what) {
 	switch (p_what) {
 		case NOTIFICATION_ENTER_TREE: {
@@ -1126,7 +1213,7 @@ void Skeleton3DEditor::_notification(int p_what) {
 			skeleton->connect(SceneStringName(bone_enabled_changed), callable_mp(this, &Skeleton3DEditor::_bone_enabled_changed));
 			skeleton->connect(SceneStringName(show_rest_only_changed), callable_mp(this, &Skeleton3DEditor::_update_gizmo_visible));
 
-			get_tree()->connect("node_removed", callable_mp(this, &Skeleton3DEditor::_node_removed), Object::CONNECT_ONE_SHOT);
+			get_tree()->connect("node_removed", callable_mp(this, &Skeleton3DEditor::_node_removed));
 		} break;
 		case NOTIFICATION_READY: {
 			// Will trigger NOTIFICATION_THEME_CHANGED, but won't cause any loops if called here.
@@ -1138,8 +1225,10 @@ void Skeleton3DEditor::_notification(int p_what) {
 			key_loc_button->set_button_icon(get_editor_theme_icon(SNAME("KeyPosition")));
 			key_rot_button->set_button_icon(get_editor_theme_icon(SNAME("KeyRotation")));
 			key_scale_button->set_button_icon(get_editor_theme_icon(SNAME("KeyScale")));
-			key_insert_button->set_button_icon(get_editor_theme_icon(SNAME("Key")));
-			key_insert_all_button->set_button_icon(get_editor_theme_icon(SNAME("NewKey")));
+			key_insert_button->set_button_icon(get_editor_theme_icon(SNAME("InsertKey")));
+			key_insert_new_button->set_button_icon(get_editor_theme_icon(SNAME("NewKey")));
+			key_mod_insert_button->set_button_icon(get_editor_theme_icon(SNAME("InsertModKey")));
+			key_mod_insert_new_button->set_button_icon(get_editor_theme_icon(SNAME("NewModKey")));
 			bones_section->set_bg_color(get_theme_color(SNAME("prop_subsection"), EditorStringName(Editor)));
 
 			update_joint_tree();
@@ -1147,10 +1236,7 @@ void Skeleton3DEditor::_notification(int p_what) {
 		case NOTIFICATION_PREDELETE: {
 			if (skeleton) {
 				select_bone(-1); // Requires that the joint_tree has not been deleted.
-				skeleton->disconnect(SceneStringName(show_rest_only_changed), callable_mp(this, &Skeleton3DEditor::_update_gizmo_visible));
-				skeleton->disconnect(SceneStringName(bone_enabled_changed), callable_mp(this, &Skeleton3DEditor::_bone_enabled_changed));
-				skeleton->disconnect(SceneStringName(pose_updated), callable_mp(this, &Skeleton3DEditor::_draw_gizmo));
-				skeleton->disconnect(SceneStringName(pose_updated), callable_mp(this, &Skeleton3DEditor::_update_properties));
+				_disconnect_from_skeleton();
 				skeleton->set_transform_gizmo_visible(true);
 
 				if (handles_mesh_instance->get_parent()) {
@@ -1164,6 +1250,12 @@ void Skeleton3DEditor::_notification(int p_what) {
 
 void Skeleton3DEditor::_node_removed(Node *p_node) {
 	if (skeleton && p_node == skeleton) {
+		_disconnect_from_skeleton();
+		if (pose_editor) {
+			pose_editor->set_skeleton(nullptr);
+			pose_editor->set_visible(false);
+		}
+		edit_mode = false;
 		skeleton = nullptr;
 		skeleton_options->hide();
 	}
@@ -1171,6 +1263,25 @@ void Skeleton3DEditor::_node_removed(Node *p_node) {
 	_update_properties();
 }
 
+void Skeleton3DEditor::_disconnect_from_skeleton() {
+	if (!skeleton) {
+		return;
+	}
+
+	if (skeleton->is_connected(SceneStringName(show_rest_only_changed), callable_mp(this, &Skeleton3DEditor::_update_gizmo_visible))) {
+		skeleton->disconnect(SceneStringName(show_rest_only_changed), callable_mp(this, &Skeleton3DEditor::_update_gizmo_visible));
+	}
+	if (skeleton->is_connected(SceneStringName(bone_enabled_changed), callable_mp(this, &Skeleton3DEditor::_bone_enabled_changed))) {
+		skeleton->disconnect(SceneStringName(bone_enabled_changed), callable_mp(this, &Skeleton3DEditor::_bone_enabled_changed));
+	}
+	if (skeleton->is_connected(SceneStringName(pose_updated), callable_mp(this, &Skeleton3DEditor::_draw_gizmo))) {
+		skeleton->disconnect(SceneStringName(pose_updated), callable_mp(this, &Skeleton3DEditor::_draw_gizmo));
+	}
+	if (skeleton->is_connected(SceneStringName(pose_updated), callable_mp(this, &Skeleton3DEditor::_update_properties))) {
+		skeleton->disconnect(SceneStringName(pose_updated), callable_mp(this, &Skeleton3DEditor::_update_properties));
+	}
+}
+
 void Skeleton3DEditor::edit_mode_toggled(const bool pressed) {
 	edit_mode = pressed;
 	_update_gizmo_visible();
@@ -1407,15 +1518,32 @@ EditorPlugin::AfterGUIInput Skeleton3DEditorPlugin::forward_3d_gui_input(Camera3
 	Node3DEditor *ne = Node3DEditor::get_singleton();
 	if (se && se->is_edit_mode()) {
 		const Ref<InputEventMouseButton> mb = p_event;
+		if (mb.is_valid() && mb->get_button_index() == MouseButton::LEFT && mb->is_pressed()) {
+			Skeleton3D *skeleton = se->get_skeleton();
+			if (skeleton) {
+				int closest_idx = Skeleton3DGizmoPlugin::skeleton_intersect_ray(skeleton, p_camera, mb->get_position());
+				if (closest_idx >= 0) {
+					se->select_bone(closest_idx);
+					se->update_bone_original();
+
+					Vector<Ref<Node3DGizmo>> gizmos = skeleton->get_gizmos();
+					for (Ref<EditorNode3DGizmo> seg : gizmos) {
+						if (seg.is_valid()) {
+							Transform3D bone_xform = seg->get_subgizmo_transform(closest_idx);
+							ne->call("_set_subgizmo_selection", skeleton, seg, closest_idx, bone_xform);
+							break;
+						}
+					}
+					return EditorPlugin::AFTER_GUI_INPUT_STOP;
+				}
+			}
+		}
 		if (mb.is_valid() && mb->get_button_index() == MouseButton::LEFT) {
 			if (ne->get_tool_mode() != Node3DEditor::TOOL_MODE_SELECT) {
 				if (!ne->is_gizmo_visible()) {
 					return EditorPlugin::AFTER_GUI_INPUT_STOP;
 				}
 			}
-			if (mb->is_pressed()) {
-				se->update_bone_original();
-			}
 		}
 		return EditorPlugin::AFTER_GUI_INPUT_CUSTOM;
 	}
@@ -1431,6 +1559,10 @@ void Skeleton3DEditor::_bone_enabled_changed(const int p_bone_id) {
 }
 
 void Skeleton3DEditor::_update_gizmo_visible() {
+	if (!skeleton) {
+		return;
+	}
+
 	_subgizmo_selection_change();
 	if (edit_mode) {
 		if (selected_bone == -1) {
@@ -1469,14 +1601,16 @@ Skeleton3DGizmoPlugin::Skeleton3DGizmoPlugin() {
 
 shader_type spatial;
 render_mode unshaded, shadows_disabled;
+
 void vertex() {
 	if (!OUTPUT_IS_SRGB) {
-		COLOR.rgb = mix( pow((COLOR.rgb + vec3(0.055)) * (1.0 / (1.0 + 0.055)), vec3(2.4)), COLOR.rgb* (1.0 / 12.92), lessThan(COLOR.rgb,vec3(0.04045)) );
+		COLOR.rgb = mix(pow((COLOR.rgb + vec3(0.055)) * (1.0 / (1.0 + 0.055)), vec3(2.4)), COLOR.rgb * (1.0 / 12.92), lessThan(COLOR.rgb,vec3(0.04045)));
 	}
 	VERTEX = VERTEX;
 	POSITION = PROJECTION_MATRIX * VIEW_MATRIX * MODEL_MATRIX * vec4(VERTEX.xyz, 1.0);
 	POSITION.z = mix(POSITION.z, POSITION.w, 0.998);
 }
+
 void fragment() {
 	ALBEDO = COLOR.rgb;
 	ALPHA = COLOR.a;
@@ -1512,19 +1646,27 @@ int Skeleton3DGizmoPlugin::subgizmos_intersect_ray(const EditorNode3DGizmo *p_gi
 		return -1;
 	}
 
-	if (Node3DEditor::get_singleton()->get_tool_mode() != Node3DEditor::TOOL_MODE_SELECT) {
-		return -1;
+	int closest_idx = skeleton_intersect_ray(skeleton, p_camera, p_point);
+
+	if (closest_idx >= 0) {
+		se->select_bone(closest_idx);
+		se->update_bone_original();
+		return closest_idx;
 	}
 
-	// Select bone.
-	real_t grab_threshold = 4 * EDSCALE;
+	se->select_bone(-1);
+	return -1;
+}
+
+int Skeleton3DGizmoPlugin::skeleton_intersect_ray(const Skeleton3D *p_skeleton, Camera3D *p_camera, const Vector2 &p_point) {
+	real_t grab_threshold = 8 * EDSCALE;
 	Vector3 ray_from = p_camera->get_global_transform().origin;
-	Transform3D gt = skeleton->get_global_transform();
+	Transform3D gt = p_skeleton->get_global_transform();
 	int closest_idx = -1;
 	real_t closest_dist = 1e10;
-	const int bone_count = skeleton->get_bone_count();
+	const int bone_count = p_skeleton->get_bone_count();
 	for (int i = 0; i < bone_count; i++) {
-		Vector3 joint_pos_3d = gt.xform(skeleton->get_bone_global_pose(i).origin);
+		Vector3 joint_pos_3d = gt.xform(p_skeleton->get_bone_global_pose(i).origin);
 		Vector2 joint_pos_2d = p_camera->unproject_position(joint_pos_3d);
 		real_t dist_3d = ray_from.distance_to(joint_pos_3d);
 		real_t dist_2d = p_point.distance_to(joint_pos_2d);
@@ -1533,14 +1675,7 @@ int Skeleton3DGizmoPlugin::subgizmos_intersect_ray(const EditorNode3DGizmo *p_gi
 			closest_idx = i;
 		}
 	}
-
-	if (closest_idx >= 0) {
-		se->select_bone(closest_idx);
-		return closest_idx;
-	}
-
-	se->select_bone(-1);
-	return -1;
+	return closest_idx;
 }
 
 Transform3D Skeleton3DGizmoPlugin::get_subgizmo_transform(const EditorNode3DGizmo *p_gizmo, int p_id) const {
@@ -1588,25 +1723,35 @@ void Skeleton3DGizmoPlugin::commit_subgizmos(const EditorNode3DGizmo *p_gizmo, c
 
 	EditorUndoRedoManager *ur = EditorUndoRedoManager::get_singleton();
 	ur->create_action(TTR("Set Bone Transform"));
-	if (ne->get_tool_mode() == Node3DEditor::TOOL_MODE_SELECT || ne->get_tool_mode() == Node3DEditor::TOOL_MODE_MOVE) {
-		for (int i = 0; i < p_ids.size(); i++) {
-			ur->add_do_method(skeleton, "set_bone_pose_position", p_ids[i], skeleton->get_bone_pose_position(p_ids[i]));
-			ur->add_undo_method(skeleton, "set_bone_pose_position", p_ids[i], se->get_bone_original_position());
+
+	if (p_cancel) {
+		for (int id : p_ids) {
+			ur->add_do_method(skeleton, "set_bone_pose_position", id, se->get_bone_original_position());
+			ur->add_do_method(skeleton, "set_bone_pose_rotation", id, se->get_bone_original_rotation());
+			ur->add_do_method(skeleton, "set_bone_pose_scale", id, se->get_bone_original_scale());
+			ur->add_undo_method(skeleton, "set_bone_pose_position", id, skeleton->get_bone_pose_position(id));
+			ur->add_undo_method(skeleton, "set_bone_pose_rotation", id, skeleton->get_bone_pose_rotation(id));
+			ur->add_undo_method(skeleton, "set_bone_pose_scale", id, skeleton->get_bone_pose_scale(id));
 		}
-	}
-	if (ne->get_tool_mode() == Node3DEditor::TOOL_MODE_SELECT || ne->get_tool_mode() == Node3DEditor::TOOL_MODE_ROTATE) {
-		for (int i = 0; i < p_ids.size(); i++) {
-			ur->add_do_method(skeleton, "set_bone_pose_rotation", p_ids[i], skeleton->get_bone_pose_rotation(p_ids[i]));
-			ur->add_undo_method(skeleton, "set_bone_pose_rotation", p_ids[i], se->get_bone_original_rotation());
+	} else {
+		Node3DEditor::ToolMode tool_mode = ne->get_tool_mode();
+		if (tool_mode == Node3DEditor::TOOL_MODE_SELECT || tool_mode == Node3DEditor::TOOL_MODE_MOVE || tool_mode == Node3DEditor::TOOL_MODE_TRANSFORM) {
+			for (int id : p_ids) {
+				ur->add_do_method(skeleton, "set_bone_pose_position", id, skeleton->get_bone_pose_position(id));
+				ur->add_undo_method(skeleton, "set_bone_pose_position", id, se->get_bone_original_position());
+			}
 		}
-	}
-	if (ne->get_tool_mode() == Node3DEditor::TOOL_MODE_SCALE) {
-		for (int i = 0; i < p_ids.size(); i++) {
-			// If the axis is swapped by scaling, the rotation can be changed.
-			ur->add_do_method(skeleton, "set_bone_pose_rotation", p_ids[i], skeleton->get_bone_pose_rotation(p_ids[i]));
-			ur->add_undo_method(skeleton, "set_bone_pose_rotation", p_ids[i], se->get_bone_original_rotation());
-			ur->add_do_method(skeleton, "set_bone_pose_scale", p_ids[i], skeleton->get_bone_pose_scale(p_ids[i]));
-			ur->add_undo_method(skeleton, "set_bone_pose_scale", p_ids[i], se->get_bone_original_scale());
+		if (tool_mode == Node3DEditor::TOOL_MODE_SELECT || tool_mode == Node3DEditor::TOOL_MODE_ROTATE || tool_mode == Node3DEditor::TOOL_MODE_TRANSFORM) {
+			for (int id : p_ids) {
+				ur->add_do_method(skeleton, "set_bone_pose_rotation", id, skeleton->get_bone_pose_rotation(id));
+				ur->add_undo_method(skeleton, "set_bone_pose_rotation", id, se->get_bone_original_rotation());
+			}
+		}
+		if (tool_mode == Node3DEditor::TOOL_MODE_SCALE || tool_mode == Node3DEditor::TOOL_MODE_TRANSFORM) {
+			for (int id : p_ids) {
+				ur->add_do_method(skeleton, "set_bone_pose_scale", id, skeleton->get_bone_pose_scale(id));
+				ur->add_undo_method(skeleton, "set_bone_pose_scale", id, se->get_bone_original_scale());
+			}
 		}
 	}
 
@@ -1706,12 +1851,12 @@ Ref<ArrayMesh> Skeleton3DGizmoPlugin::get_bones_mesh(Skeleton3D *p_skeleton, int
 				case 0: { // Wire shape.
 					surface_tool->set_color(current_bone_color);
 					bones[0] = current_bone_idx;
-					surface_tool->set_bones(bones);
-					surface_tool->set_weights(weights);
+					surface_tool->set_bones(Vector<int>(bones));
+					surface_tool->set_weights(Vector<float>(weights));
 					surface_tool->add_vertex(v0);
 					bones[0] = child_bone_idx;
-					surface_tool->set_bones(bones);
-					surface_tool->set_weights(weights);
+					surface_tool->set_bones(Vector<int>(bones));
+					surface_tool->set_weights(Vector<float>(weights));
 					surface_tool->add_vertex(v1);
 				} break;
 
@@ -1737,19 +1882,19 @@ Ref<ArrayMesh> Skeleton3DGizmoPlugin::get_bones_mesh(Skeleton3D *p_skeleton, int
 							point += axis * dist * 0.1;
 
 							bones[0] = current_bone_idx;
-							surface_tool->set_bones(bones);
-							surface_tool->set_weights(weights);
+							surface_tool->set_bones(Vector<int>(bones));
+							surface_tool->set_weights(Vector<float>(weights));
 							surface_tool->add_vertex(v0);
-							surface_tool->set_bones(bones);
-							surface_tool->set_weights(weights);
+							surface_tool->set_bones(Vector<int>(bones));
+							surface_tool->set_weights(Vector<float>(weights));
 							surface_tool->add_vertex(point);
 
-							surface_tool->set_bones(bones);
-							surface_tool->set_weights(weights);
+							surface_tool->set_bones(Vector<int>(bones));
+							surface_tool->set_weights(Vector<float>(weights));
 							surface_tool->add_vertex(point);
 							bones[0] = child_bone_idx;
-							surface_tool->set_bones(bones);
-							surface_tool->set_weights(weights);
+							surface_tool->set_bones(Vector<int>(bones));
+							surface_tool->set_weights(Vector<float>(weights));
 							surface_tool->add_vertex(v1);
 							points[point_idx++] = point;
 						}
@@ -1758,11 +1903,11 @@ Ref<ArrayMesh> Skeleton3DGizmoPlugin::get_bones_mesh(Skeleton3D *p_skeleton, int
 					SWAP(points[1], points[2]);
 					bones[0] = current_bone_idx;
 					for (int j = 0; j < 6; j++) {
-						surface_tool->set_bones(bones);
-						surface_tool->set_weights(weights);
+						surface_tool->set_bones(Vector<int>(bones));
+						surface_tool->set_weights(Vector<float>(weights));
 						surface_tool->add_vertex(points[j]);
-						surface_tool->set_bones(bones);
-						surface_tool->set_weights(weights);
+						surface_tool->set_bones(Vector<int>(bones));
+						surface_tool->set_weights(Vector<float>(weights));
 						surface_tool->add_vertex(points[(j + 1) % 6]);
 					}
 				} break;
@@ -1772,11 +1917,11 @@ Ref<ArrayMesh> Skeleton3DGizmoPlugin::get_bones_mesh(Skeleton3D *p_skeleton, int
 			for (int j = 0; j < 3; j++) {
 				bones[0] = current_bone_idx;
 				surface_tool->set_color(axis_colors[j]);
-				surface_tool->set_bones(bones);
-				surface_tool->set_weights(weights);
+				surface_tool->set_bones(Vector<int>(bones));
+				surface_tool->set_weights(Vector<float>(weights));
 				surface_tool->add_vertex(v0);
-				surface_tool->set_bones(bones);
-				surface_tool->set_weights(weights);
+				surface_tool->set_bones(Vector<int>(bones));
+				surface_tool->set_weights(Vector<float>(weights));
 				surface_tool->add_vertex(v0 + (p_skeleton->get_bone_global_rest(current_bone_idx).basis.inverse())[j].normalized() * dist * bone_axis_length);
 
 				if (j == closest) {
@@ -1789,11 +1934,11 @@ Ref<ArrayMesh> Skeleton3DGizmoPlugin::get_bones_mesh(Skeleton3D *p_skeleton, int
 				for (int j = 0; j < 3; j++) {
 					bones[0] = child_bone_idx;
 					surface_tool->set_color(axis_colors[j]);
-					surface_tool->set_bones(bones);
-					surface_tool->set_weights(weights);
+					surface_tool->set_bones(Vector<int>(bones));
+					surface_tool->set_weights(Vector<float>(weights));
 					surface_tool->add_vertex(v1);
-					surface_tool->set_bones(bones);
-					surface_tool->set_weights(weights);
+					surface_tool->set_bones(Vector<int>(bones));
+					surface_tool->set_weights(Vector<float>(weights));
 					surface_tool->add_vertex(v1 + (p_skeleton->get_bone_global_rest(child_bone_idx).basis.inverse())[j].normalized() * dist * bone_axis_length);
 
 					if (j == closest) {
