@@ -33,8 +33,8 @@
 #include "core/config/project_settings.h"
 #include "core/templates/hash_set.h"
 #include "editor/doc/editor_help.h"
-#include "editor/docks/node_dock.h"
 #include "editor/docks/scene_tree_dock.h"
+#include "editor/docks/signals_dock.h"
 #include "editor/editor_main_screen.h"
 #include "editor/editor_node.h"
 #include "editor/editor_string_names.h"
@@ -113,6 +113,33 @@ public:
 		}
 	}
 
+	void update_base_node_relative(Node *p_node) {
+		Node *old_base = nullptr;
+		if (has_meta("__base_node_relative")) {
+			old_base = Object::cast_to<Node>(get_meta("__base_node_relative"));
+		}
+
+		if (old_base == p_node) {
+			return;
+		}
+		// The cdbinds is a proxy object, so we want the node path to be relative to the target node.
+		set_meta("__base_node_relative", p_node);
+
+		if (!old_base) {
+			return;
+		}
+
+		// Update existing outdated node paths.
+		for (int i = 0; i < params.size(); i++) {
+			if (params[i].get_type() != Variant::NODE_PATH) {
+				continue;
+			}
+			StringName property_name = "bind/argument_" + itos(i + 1);
+			Node *n = old_base->get_node(get(property_name));
+			set(property_name, p_node ? p_node->get_path_to(n) : NodePath());
+		}
+	}
+
 	void notify_changed() {
 		notify_property_list_changed();
 	}
@@ -172,33 +199,24 @@ void ConnectDialog::_tree_node_selected() {
 		return;
 	}
 
-	dst_path = source->get_path_to(current);
+	Node *source_node = Object::cast_to<Node>(source);
+	if (source_node) {
+		dst_path = source_node->get_path_to(current);
+	}
+
 	if (!edit_mode) {
 		set_dst_method(generate_method_callback_name(source, signal, current));
 	}
+
+	cdbinds->update_base_node_relative(current);
+
 	_update_method_tree();
 	_update_warning_label();
 	_update_ok_enabled();
 }
 
 void ConnectDialog::_focus_currently_connected() {
-	tree->set_selected(source);
-}
-
-void ConnectDialog::_unbind_count_changed(double p_count) {
-	for (Control *control : bind_controls) {
-		BaseButton *b = Object::cast_to<BaseButton>(control);
-		if (b) {
-			b->set_disabled(p_count > 0);
-		}
-
-		EditorInspector *e = Object::cast_to<EditorInspector>(control);
-		if (e) {
-			e->set_read_only(p_count > 0);
-		}
-	}
-
-	append_source->set_disabled(p_count > 0);
+	tree->set_selected(Object::cast_to<Node>(source));
 }
 
 void ConnectDialog::_method_selected() {
@@ -237,8 +255,9 @@ void ConnectDialog::_remove_bind() {
 /*
  * Automatically generates a name for the callback method.
  */
-StringName ConnectDialog::generate_method_callback_name(Node *p_source, const String &p_signal_name, Node *p_target) {
-	String node_name = p_source->get_name();
+StringName ConnectDialog::generate_method_callback_name(Object *p_source, const String &p_signal_name, Object *p_target) {
+	String node_name = p_source->call("get_name");
+
 	for (int i = 0; i < node_name.length(); i++) { // TODO: Regex filter may be cleaner.
 		char32_t c = node_name[i];
 		if ((i == 0 && !is_unicode_identifier_start(c)) || (i > 0 && !is_unicode_identifier_continue(c))) {
@@ -295,10 +314,9 @@ List<MethodInfo> ConnectDialog::_filter_method_list(const List<MethodInfo> &p_me
 		PropertyInfo pi = p_signal.arguments[i];
 		effective_args.push_back(Pair(pi.type, pi.class_name));
 	}
-	if (unbind == 0) {
-		for (const Variant &variant : get_binds()) {
-			effective_args.push_back(Pair(variant.get_type(), StringName()));
-		}
+
+	for (const Variant &variant : get_binds()) {
+		effective_args.push_back(Pair(variant.get_type(), StringName()));
 	}
 
 	for (const MethodInfo &mi : p_methods) {
@@ -462,7 +480,8 @@ void ConnectDialog::_update_ok_enabled() {
 }
 
 void ConnectDialog::_update_warning_label() {
-	Node *dst = source->get_node(dst_path);
+	Node *dst = Object::cast_to<Node>(source)->get_node(dst_path);
+
 	if (dst == nullptr) {
 		warning_label->set_visible(false);
 		return;
@@ -485,7 +504,7 @@ void ConnectDialog::_update_warning_label() {
 }
 
 void ConnectDialog::_post_popup() {
-	callable_mp((Control *)dst_method, &Control::grab_focus).call_deferred();
+	callable_mp((Control *)dst_method, &Control::grab_focus).call_deferred(false);
 	callable_mp(dst_method, &LineEdit::select_all).call_deferred();
 }
 
@@ -507,7 +526,7 @@ void ConnectDialog::_bind_methods() {
 	ADD_SIGNAL(MethodInfo("connected"));
 }
 
-Node *ConnectDialog::get_source() const {
+Object *ConnectDialog::get_source() const {
 	return source;
 }
 
@@ -663,15 +682,15 @@ void ConnectDialog::shortcut_input(const Ref<InputEvent> &p_event) {
 void ConnectDialog::init(const ConnectionData &p_cd, const PackedStringArray &p_signal_args, bool p_edit) {
 	set_hide_on_ok(false);
 
-	source = static_cast<Node *>(p_cd.source);
+	source = p_cd.source;
 	signal = p_cd.signal;
 	signal_args = p_signal_args;
 
 	tree->set_selected(nullptr);
-	tree->set_marked(source);
+	tree->set_marked(Object::cast_to<Node>(source));
 
 	if (p_cd.target) {
-		set_dst_node(static_cast<Node *>(p_cd.target));
+		set_dst_node(Object::cast_to<Node>(p_cd.target));
 		set_dst_method(p_cd.method);
 	}
 
@@ -687,7 +706,6 @@ void ConnectDialog::init(const ConnectionData &p_cd, const PackedStringArray &p_
 
 	unbind_count->set_max(p_signal_args.size());
 	unbind_count->set_value(p_cd.unbinds);
-	_unbind_count_changed(p_cd.unbinds);
 
 	cdbinds->params.clear();
 	cdbinds->params = p_cd.binds;
@@ -761,6 +779,7 @@ ConnectDialog::ConnectDialog() {
 	tree->get_scene_tree()->connect("item_activated", callable_mp(this, &ConnectDialog::_item_activated));
 	tree->connect("node_selected", callable_mp(this, &ConnectDialog::_tree_node_selected));
 	tree->set_connect_to_script_mode(true);
+	tree->get_scene_tree()->set_theme_type_variation("TreeSecondary");
 
 	HBoxContainer *hbc_filter = memnew(HBoxContainer);
 
@@ -865,6 +884,7 @@ ConnectDialog::ConnectDialog() {
 
 	bind_editor = memnew(EditorInspector);
 	bind_editor->set_accessibility_name(TTRC("Extra Call Arguments:"));
+	bind_editor->set_theme_type_variation("ScrollContainerSecondary");
 	bind_controls.push_back(bind_editor);
 
 	vbc_right->add_margin_child(TTR("Extra Call Arguments:"), bind_editor, true);
@@ -872,7 +892,6 @@ ConnectDialog::ConnectDialog() {
 	unbind_count = memnew(SpinBox);
 	unbind_count->set_tooltip_text(TTR("Allows to drop arguments sent by signal emitter."));
 	unbind_count->set_accessibility_name(TTRC("Unbind Signal Arguments:"));
-	unbind_count->connect(SceneStringName(value_changed), callable_mp(this, &ConnectDialog::_unbind_count_changed));
 
 	vbc_right->add_margin_child(TTR("Unbind Signal Arguments:"), unbind_count);
 
@@ -935,7 +954,7 @@ Control *ConnectionsDockTree::make_custom_tooltip(const String &p_text) const {
 		return nullptr;
 	}
 
-	return EditorHelpBitTooltip::show_tooltip(const_cast<ConnectionsDockTree *>(this), p_text);
+	return EditorHelpBitTooltip::make_tooltip(const_cast<ConnectionsDockTree *>(this), p_text);
 }
 
 struct _ConnectionsDockMethodInfoSort {
@@ -954,7 +973,8 @@ void ConnectionsDock::_filter_changed(const String &p_text) {
  */
 void ConnectionsDock::_make_or_edit_connection() {
 	NodePath dst_path = connect_dialog->get_dst_path();
-	Node *target = selected_node->get_node(dst_path);
+	Node *target = Object::cast_to<Node>(selected_object)->get_node(dst_path);
+
 	ERR_FAIL_NULL(target);
 
 	ConnectDialog::ConnectionData cd;
@@ -963,9 +983,8 @@ void ConnectionsDock::_make_or_edit_connection() {
 	cd.signal = connect_dialog->get_signal_name();
 	cd.method = connect_dialog->get_dst_method_name();
 	cd.unbinds = connect_dialog->get_unbinds();
-	if (cd.unbinds == 0) {
-		cd.binds = connect_dialog->get_binds();
-	}
+	cd.binds = connect_dialog->get_binds();
+
 	bool b_deferred = connect_dialog->get_deferred();
 	bool b_oneshot = connect_dialog->get_one_shot();
 	bool b_append_source = connect_dialog->get_append_source();
@@ -1047,7 +1066,7 @@ void ConnectionsDock::_make_or_edit_connection() {
 			script_function_args.push_back("extra_arg_" + itos(i) + ": " + Variant::get_type_name(cd.binds[i].get_type()));
 		}
 
-		EditorNode::get_singleton()->emit_signal(SNAME("script_add_function_request"), target, cd.method, script_function_args);
+		EditorNode::get_singleton()->emit_signal(SNAME("script_add_function_request"), cd.target, cd.method, script_function_args);
 	}
 
 	if (connect_dialog->is_editing()) {
@@ -1064,7 +1083,7 @@ void ConnectionsDock::_make_or_edit_connection() {
  * Creates single connection w/ undo-redo functionality.
  */
 void ConnectionsDock::_connect(const ConnectDialog::ConnectionData &p_cd) {
-	Node *source = Object::cast_to<Node>(p_cd.source);
+	Object *source = p_cd.source;
 	Node *target = Object::cast_to<Node>(p_cd.target);
 
 	if (!source || !target) {
@@ -1088,14 +1107,14 @@ void ConnectionsDock::_connect(const ConnectDialog::ConnectionData &p_cd) {
  * Break single connection w/ undo-redo functionality.
  */
 void ConnectionsDock::_disconnect(const ConnectDialog::ConnectionData &p_cd) {
-	ERR_FAIL_COND(p_cd.source != selected_node); // Shouldn't happen but... Bugcheck.
+	ERR_FAIL_COND(p_cd.source != selected_object); // Shouldn't happen but... Bugcheck.
 
 	EditorUndoRedoManager *undo_redo = EditorUndoRedoManager::get_singleton();
 	undo_redo->create_action(vformat(TTR("Disconnect '%s' from '%s'"), p_cd.signal, p_cd.method));
 
 	Callable callable = p_cd.get_callable();
-	undo_redo->add_do_method(selected_node, "disconnect", p_cd.signal, callable);
-	undo_redo->add_undo_method(selected_node, "connect", p_cd.signal, callable, p_cd.flags);
+	undo_redo->add_do_method(selected_object, "disconnect", p_cd.signal, callable);
+	undo_redo->add_undo_method(selected_object, "connect", p_cd.signal, callable, p_cd.flags);
 	undo_redo->add_do_method(this, "update_tree");
 	undo_redo->add_undo_method(this, "update_tree");
 	undo_redo->add_do_method(SceneTreeDock::get_singleton()->get_tree_editor(), "update_tree"); // To force redraw of scene tree.
@@ -1123,8 +1142,8 @@ void ConnectionsDock::_disconnect_all() {
 		Connection connection = child->get_metadata(0);
 		if (!_is_connection_inherited(connection)) {
 			ConnectDialog::ConnectionData cd = connection;
-			undo_redo->add_do_method(selected_node, "disconnect", cd.signal, cd.get_callable());
-			undo_redo->add_undo_method(selected_node, "connect", cd.signal, cd.get_callable(), cd.flags);
+			undo_redo->add_do_method(selected_object, "disconnect", cd.signal, cd.get_callable());
+			undo_redo->add_undo_method(selected_object, "connect", cd.signal, cd.get_callable(), cd.flags);
 		}
 		child = child->get_next();
 	}
@@ -1142,7 +1161,7 @@ void ConnectionsDock::_tree_item_selected() {
 	if (item && _get_item_type(*item) == TREE_ITEM_TYPE_SIGNAL) {
 		connect_button->set_text(TTR("Connect..."));
 		connect_button->set_button_icon(get_editor_theme_icon(SNAME("Instance")));
-		connect_button->set_disabled(false);
+		connect_button->set_disabled(is_editing_resource);
 	} else if (item && _get_item_type(*item) == TREE_ITEM_TYPE_CONNECTION) {
 		connect_button->set_text(TTR("Disconnect"));
 		connect_button->set_button_icon(get_editor_theme_icon(SNAME("Unlinked")));
@@ -1189,19 +1208,24 @@ bool ConnectionsDock::_is_connection_inherited(Connection &p_connection) {
  * Open connection dialog with TreeItem data to CREATE a brand-new connection.
  */
 void ConnectionsDock::_open_connection_dialog(TreeItem &p_item) {
+	if (is_editing_resource) {
+		return;
+	}
+
 	const Dictionary sinfo = p_item.get_metadata(0);
 	const StringName signal_name = sinfo["name"];
 	const PackedStringArray signal_args = sinfo["args"];
 
+	ConnectDialog::ConnectionData cd;
+
+	Node *selected_node = Object::cast_to<Node>(selected_object);
 	Node *dst_node = selected_node->get_owner() ? selected_node->get_owner() : selected_node;
 	if (!dst_node || dst_node->get_script().is_null()) {
 		dst_node = _find_first_script(get_tree()->get_edited_scene_root(), get_tree()->get_edited_scene_root());
 	}
-
-	ConnectDialog::ConnectionData cd;
-	cd.source = selected_node;
-	cd.signal = signal_name;
+	cd.source = selected_object;
 	cd.target = dst_node;
+	cd.signal = signal_name;
 	cd.method = ConnectDialog::generate_method_callback_name(cd.source, signal_name, cd.target);
 	connect_dialog->init(cd, signal_args);
 	connect_dialog->set_title(TTR("Connect a Signal to a Method"));
@@ -1218,8 +1242,8 @@ void ConnectionsDock::_open_edit_connection_dialog(TreeItem &p_item) {
 	Connection connection = p_item.get_metadata(0);
 	ConnectDialog::ConnectionData cd = connection;
 
-	Node *src = Object::cast_to<Node>(cd.source);
-	Node *dst = Object::cast_to<Node>(cd.target);
+	Object *src = cd.source;
+	Object *dst = cd.target;
 
 	if (src && dst) {
 		const StringName &signal_name = cd.signal;
@@ -1241,7 +1265,7 @@ void ConnectionsDock::_go_to_method(TreeItem &p_item) {
 
 	Connection connection = p_item.get_metadata(0);
 	ConnectDialog::ConnectionData cd = connection;
-	ERR_FAIL_COND(cd.source != selected_node); // Shouldn't happen but... bugcheck.
+	ERR_FAIL_COND(cd.source != selected_object); // Shouldn't happen but... bugcheck.
 
 	if (!cd.target) {
 		return;
@@ -1312,6 +1336,7 @@ void ConnectionsDock::_signal_menu_about_to_popup() {
 		}
 	}
 
+	signal_menu->set_item_disabled(signal_menu->get_item_index(SIGNAL_MENU_CONNECT), is_editing_resource);
 	signal_menu->set_item_disabled(signal_menu->get_item_index(SIGNAL_MENU_DISCONNECT_ALL), disable_disconnect_all);
 	signal_menu->set_item_disabled(signal_menu->get_item_index(SIGNAL_MENU_OPEN_DOCS), String(meta["class"]).is_empty());
 }
@@ -1479,8 +1504,16 @@ void ConnectionsDock::_bind_methods() {
 	ClassDB::bind_method("update_tree", &ConnectionsDock::update_tree);
 }
 
-void ConnectionsDock::set_node(Node *p_node) {
-	selected_node = p_node;
+void ConnectionsDock::set_object(Object *p_object) {
+	if (p_object == nullptr) {
+		select_an_object->show();
+		holder->hide();
+	} else {
+		select_an_object->hide();
+		holder->show();
+	}
+	selected_object = p_object;
+	is_editing_resource = (Object::cast_to<Resource>(selected_object) != nullptr);
 	update_tree();
 }
 
@@ -1491,15 +1524,15 @@ void ConnectionsDock::update_tree() {
 	}
 	tree->clear();
 
-	if (!selected_node) {
+	if (!selected_object) {
 		return;
 	}
 
 	TreeItem *root = tree->create_item();
 	DocTools *doc_data = EditorHelp::get_doc_data();
 	EditorData &editor_data = EditorNode::get_editor_data();
-	StringName native_base = selected_node->get_class();
-	Ref<Script> script_base = selected_node->get_script();
+	StringName native_base = selected_object->get_class();
+	Ref<Script> script_base = selected_object->get_script();
 
 	while (native_base != StringName()) {
 		String class_name;
@@ -1582,6 +1615,7 @@ void ConnectionsDock::update_tree() {
 			section_item->set_selectable(0, false);
 			section_item->set_editable(0, false);
 			section_item->set_custom_bg_color(0, get_theme_color(SNAME("prop_subsection"), EditorStringName(Editor)));
+			section_item->set_custom_stylebox(0, get_theme_stylebox(SNAME("prop_subsection_stylebox"), EditorStringName(Editor)));
 			section_item->set_metadata(0, doc_class_name);
 		}
 
@@ -1613,7 +1647,7 @@ void ConnectionsDock::update_tree() {
 
 			// List existing connections.
 			List<Object::Connection> existing_connections;
-			selected_node->get_signal_connection_list(signal_name, &existing_connections);
+			selected_object->get_signal_connection_list(signal_name, &existing_connections);
 
 			for (const Object::Connection &F : existing_connections) {
 				Connection connection = F;
@@ -1627,19 +1661,21 @@ void ConnectionsDock::update_tree() {
 					continue;
 				}
 
-				String path = String(selected_node->get_path_to(target)) + " :: " + cd.method + "()";
+				String path = String(Object::cast_to<Node>(selected_object)->get_path_to(target)) + " :: " + cd.method + "()";
 				if (cd.flags & CONNECT_DEFERRED) {
 					path += " (deferred)";
 				}
 				if (cd.flags & CONNECT_ONE_SHOT) {
 					path += " (one-shot)";
 				}
+				if (cd.unbinds > 0) {
+					path += " unbinds(" + itos(cd.unbinds) + ")";
+				}
+				// CONNECT_APPEND_SOURCE_OBJECT is not affected by unbinds, list it between unbinds/binds to better indicate the final order.
 				if (cd.flags & CONNECT_APPEND_SOURCE_OBJECT) {
 					path += " (source)";
 				}
-				if (cd.unbinds > 0) {
-					path += " unbinds(" + itos(cd.unbinds) + ")";
-				} else if (!cd.binds.is_empty()) {
+				if (!cd.binds.is_empty()) {
 					path += " binds(";
 					for (int i = 0; i < cd.binds.size(); i++) {
 						if (i > 0) {
@@ -1664,7 +1700,7 @@ void ConnectionsDock::update_tree() {
 		}
 	}
 
-	connect_button->set_text(TTR("Connect..."));
+	connect_button->set_text(TTRC("Connect..."));
 	connect_button->set_button_icon(get_editor_theme_icon(SNAME("Instance")));
 	connect_button->set_disabled(true);
 }
@@ -1672,7 +1708,10 @@ void ConnectionsDock::update_tree() {
 ConnectionsDock::ConnectionsDock() {
 	set_name(TTR("Signals"));
 
-	VBoxContainer *vbc = this;
+	holder = memnew(VBoxContainer);
+	holder->set_v_size_flags(SIZE_EXPAND_FILL);
+	holder->hide();
+	add_child(holder);
 
 	search_box = memnew(LineEdit);
 	search_box->set_h_size_flags(Control::SIZE_EXPAND_FILL);
@@ -1680,7 +1719,12 @@ ConnectionsDock::ConnectionsDock() {
 	search_box->set_accessibility_name(TTRC("Filter Signals"));
 	search_box->set_clear_button_enabled(true);
 	search_box->connect(SceneStringName(text_changed), callable_mp(this, &ConnectionsDock::_filter_changed));
-	vbc->add_child(search_box);
+	holder->add_child(search_box);
+
+	MarginContainer *mc = memnew(MarginContainer);
+	mc->set_theme_type_variation("NoBorderHorizontal");
+	mc->set_v_size_flags(SIZE_EXPAND_FILL);
+	holder->add_child(mc);
 
 	tree = memnew(ConnectionsDockTree);
 	tree->set_accessibility_name(TTRC("Connections"));
@@ -1688,25 +1732,25 @@ ConnectionsDock::ConnectionsDock() {
 	tree->set_columns(1);
 	tree->set_select_mode(Tree::SELECT_ROW);
 	tree->set_hide_root(true);
-	tree->set_column_clip_content(0, true);
-	vbc->add_child(tree);
-	tree->set_v_size_flags(Control::SIZE_EXPAND_FILL);
 	tree->set_allow_rmb_select(true);
+	tree->set_column_clip_content(0, true);
+	tree->set_scroll_hint_mode(Tree::SCROLL_HINT_MODE_BOTH);
+	mc->add_child(tree);
 
 	connect_button = memnew(Button);
 	connect_button->set_accessibility_name(TTRC("Connect"));
 	HBoxContainer *hb = memnew(HBoxContainer);
-	vbc->add_child(hb);
+	holder->add_child(hb);
 	hb->add_spacer();
 	hb->add_child(connect_button);
 	connect_button->connect(SceneStringName(pressed), callable_mp(this, &ConnectionsDock::_connect_pressed));
 
 	connect_dialog = memnew(ConnectDialog);
 	connect_dialog->set_process_shortcut_input(true);
-	add_child(connect_dialog);
+	holder->add_child(connect_dialog);
 
 	disconnect_all_dialog = memnew(ConfirmationDialog);
-	add_child(disconnect_all_dialog);
+	holder->add_child(disconnect_all_dialog);
 	disconnect_all_dialog->connect(SceneStringName(confirmed), callable_mp(this, &ConnectionsDock::_disconnect_all));
 	disconnect_all_dialog->set_text(TTR("Are you sure you want to remove all connections from this signal?"));
 
@@ -1714,7 +1758,7 @@ ConnectionsDock::ConnectionsDock() {
 	class_menu->connect(SceneStringName(id_pressed), callable_mp(this, &ConnectionsDock::_handle_class_menu_option));
 	class_menu->connect("about_to_popup", callable_mp(this, &ConnectionsDock::_class_menu_about_to_popup));
 	class_menu->add_item(TTR("Open Documentation"), CLASS_MENU_OPEN_DOCS);
-	add_child(class_menu);
+	holder->add_child(class_menu);
 
 	signal_menu = memnew(PopupMenu);
 	signal_menu->connect(SceneStringName(id_pressed), callable_mp(this, &ConnectionsDock::_handle_signal_menu_option));
@@ -1724,7 +1768,7 @@ ConnectionsDock::ConnectionsDock() {
 	signal_menu->add_item(TTR("Copy Name"), SIGNAL_MENU_COPY_NAME);
 	signal_menu->add_separator();
 	signal_menu->add_item(TTR("Open Documentation"), SIGNAL_MENU_OPEN_DOCS);
-	add_child(signal_menu);
+	holder->add_child(signal_menu);
 
 	slot_menu = memnew(PopupMenu);
 	slot_menu->connect(SceneStringName(id_pressed), callable_mp(this, &ConnectionsDock::_handle_slot_menu_option));
@@ -1732,7 +1776,7 @@ ConnectionsDock::ConnectionsDock() {
 	slot_menu->add_item(TTR("Edit..."), SLOT_MENU_EDIT);
 	slot_menu->add_item(TTR("Go to Method"), SLOT_MENU_GO_TO_METHOD);
 	slot_menu->add_shortcut(ED_SHORTCUT("connections_editor/disconnect", TTRC("Disconnect"), Key::KEY_DELETE), SLOT_MENU_DISCONNECT);
-	add_child(slot_menu);
+	holder->add_child(slot_menu);
 
 	connect_dialog->connect("connected", callable_mp(this, &ConnectionsDock::_make_or_edit_connection));
 	tree->connect(SceneStringName(item_selected), callable_mp(this, &ConnectionsDock::_tree_item_selected));
@@ -1740,4 +1784,14 @@ ConnectionsDock::ConnectionsDock() {
 	tree->connect(SceneStringName(gui_input), callable_mp(this, &ConnectionsDock::_tree_gui_input));
 
 	add_theme_constant_override("separation", 3 * EDSCALE);
+
+	select_an_object = memnew(Label);
+	select_an_object->set_focus_mode(FOCUS_ACCESSIBILITY);
+	select_an_object->set_text(TTRC("Select a single node or resource to edit its signals."));
+	select_an_object->set_custom_minimum_size(Size2(100 * EDSCALE, 0));
+	select_an_object->set_v_size_flags(SIZE_EXPAND_FILL);
+	select_an_object->set_vertical_alignment(VERTICAL_ALIGNMENT_CENTER);
+	select_an_object->set_horizontal_alignment(HORIZONTAL_ALIGNMENT_CENTER);
+	select_an_object->set_autowrap_mode(TextServer::AUTOWRAP_WORD_SMART);
+	add_child(select_an_object);
 }
