@@ -41,6 +41,7 @@ class Main {
 		CLI_OPTION_AVAILABILITY_EDITOR,
 		CLI_OPTION_AVAILABILITY_TEMPLATE_DEBUG,
 		CLI_OPTION_AVAILABILITY_TEMPLATE_RELEASE,
+		CLI_OPTION_AVAILABILITY_TEMPLATE_UNSAFE,
 		CLI_OPTION_AVAILABILITY_HIDDEN,
 	};
 
@@ -71,6 +72,7 @@ public:
 	static Error setup(const char *execpath, int argc, char *argv[], bool p_second_phase = true);
 	static Error setup2(bool p_show_boot_logo = true); // The thread calling setup2() will effectively become the main thread.
 	static String get_rendering_driver_name();
+	static String get_locale_override();
 	static void setup_boot_logo();
 #ifdef TESTS_ENABLED
 	static Error test_setup();
@@ -91,6 +93,7 @@ public:
 	bool run_test = false;                                         \
 	int return_code = Main::test_entrypoint(argc, argv, run_test); \
 	if (run_test) {                                                \
+		godot_cleanup_profiler();                                  \
 		return return_code;                                        \
 	}
 
@@ -98,5 +101,6 @@ public:
 	bool run_test = false;                                         \
 	int return_code = Main::test_entrypoint(argc, argv, run_test); \
 	if (run_test) {                                                \
+		godot_cleanup_profiler();                                  \
 		return return_code;                                        \
 	}
