@@ -29,38 +29,42 @@
 /**************************************************************************/
 
 #include "performance.h"
+#include "performance.compat.inc"
 
 #include "core/os/os.h"
 #include "core/variant/typed_array.h"
 #include "scene/main/node.h"
 #include "scene/main/scene_tree.h"
-#include "servers/audio_server.h"
+#include "servers/audio/audio_server.h"
+#include "servers/rendering/rendering_server.h"
+
 #ifndef NAVIGATION_2D_DISABLED
-#include "servers/navigation_server_2d.h"
+#include "servers/navigation_2d/navigation_server_2d.h"
 #endif // NAVIGATION_2D_DISABLED
+
 #ifndef NAVIGATION_3D_DISABLED
-#include "servers/navigation_server_3d.h"
+#include "servers/navigation_3d/navigation_server_3d.h"
 #endif // NAVIGATION_3D_DISABLED
-#include "servers/rendering_server.h"
 
 #ifndef PHYSICS_2D_DISABLED
-#include "servers/physics_server_2d.h"
+#include "servers/physics_2d/physics_server_2d.h"
 #endif // PHYSICS_2D_DISABLED
 
 #ifndef PHYSICS_3D_DISABLED
-#include "servers/physics_server_3d.h"
+#include "servers/physics_3d/physics_server_3d.h"
 #endif // PHYSICS_3D_DISABLED
 
 Performance *Performance::singleton = nullptr;
 
 void Performance::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("get_monitor", "monitor"), &Performance::get_monitor);
-	ClassDB::bind_method(D_METHOD("add_custom_monitor", "id", "callable", "arguments"), &Performance::add_custom_monitor, DEFVAL(Array()));
+	ClassDB::bind_method(D_METHOD("add_custom_monitor", "id", "callable", "arguments", "type"), &Performance::add_custom_monitor, DEFVAL(Array()), DEFVAL(MONITOR_TYPE_QUANTITY));
 	ClassDB::bind_method(D_METHOD("remove_custom_monitor", "id"), &Performance::remove_custom_monitor);
 	ClassDB::bind_method(D_METHOD("has_custom_monitor", "id"), &Performance::has_custom_monitor);
 	ClassDB::bind_method(D_METHOD("get_custom_monitor", "id"), &Performance::get_custom_monitor);
 	ClassDB::bind_method(D_METHOD("get_monitor_modification_time"), &Performance::get_monitor_modification_time);
 	ClassDB::bind_method(D_METHOD("get_custom_monitor_names"), &Performance::get_custom_monitor_names);
+	ClassDB::bind_method(D_METHOD("get_custom_monitor_types"), &Performance::get_custom_monitor_types);
 
 	BIND_ENUM_CONSTANT(TIME_FPS);
 	BIND_ENUM_CONSTANT(TIME_PROCESS);
@@ -130,6 +134,11 @@ void Performance::_bind_methods() {
 	BIND_ENUM_CONSTANT(NAVIGATION_3D_OBSTACLE_COUNT);
 #endif // NAVIGATION_3D_DISABLED
 	BIND_ENUM_CONSTANT(MONITOR_MAX);
+
+	BIND_ENUM_CONSTANT(MONITOR_TYPE_QUANTITY);
+	BIND_ENUM_CONSTANT(MONITOR_TYPE_MEMORY);
+	BIND_ENUM_CONSTANT(MONITOR_TYPE_TIME);
+	BIND_ENUM_CONSTANT(MONITOR_TYPE_PERCENTAGE);
 }
 
 int Performance::_get_node_count() const {
@@ -141,6 +150,16 @@ int Performance::_get_node_count() const {
 	return sml->get_node_count();
 }
 
+int Performance::_get_orphan_node_count() const {
+#ifdef DEBUG_ENABLED
+	const int total_node_count = Node::total_node_count.get();
+	const int orphan_node_count = total_node_count - _get_node_count();
+	return orphan_node_count;
+#else
+	return 0;
+#endif
+}
+
 String Performance::get_monitor_name(Monitor p_monitor) const {
 	ERR_FAIL_INDEX_V(p_monitor, MONITOR_MAX, String());
 	static const char *names[MONITOR_MAX] = {
@@ -210,7 +229,7 @@ String Performance::get_monitor_name(Monitor p_monitor) const {
 		PNAME("navigation_3d/obstacles"),
 #endif // NAVIGATION_3D_DISABLED
 	};
-	static_assert(std::size(names) == MONITOR_MAX);
+	static_assert(std_size(names) == MONITOR_MAX);
 
 	return names[p_monitor];
 }
@@ -240,7 +259,7 @@ double Performance::get_monitor(Monitor p_monitor) const {
 		case OBJECT_NODE_COUNT:
 			return _get_node_count();
 		case OBJECT_ORPHAN_NODE_COUNT:
-			return Node::orphan_node_count;
+			return _get_orphan_node_count();
 		case RENDER_TOTAL_OBJECTS_IN_FRAME:
 			return RS::get_singleton()->get_rendering_info(RS::RENDERING_INFO_TOTAL_OBJECTS_IN_FRAME);
 		case RENDER_TOTAL_PRIMITIVES_IN_FRAME:
@@ -297,6 +316,7 @@ double Performance::get_monitor(Monitor p_monitor) const {
 		case AUDIO_OUTPUT_LATENCY:
 			return AudioServer::get_singleton()->get_output_latency();
 
+			// Deprecated, use the 2D/3D specific ones instead.
 		case NAVIGATION_ACTIVE_MAPS:
 #ifndef NAVIGATION_2D_DISABLED
 			info = NavigationServer2D::get_singleton()->get_process_info(NavigationServer2D::INFO_ACTIVE_MAPS);
@@ -493,6 +513,7 @@ Performance::MonitorType Performance::get_monitor_type(Monitor p_monitor) const
 		MONITOR_TYPE_QUANTITY,
 		MONITOR_TYPE_QUANTITY,
 		MONITOR_TYPE_QUANTITY,
+#ifndef _3D_DISABLED
 		MONITOR_TYPE_QUANTITY,
 		MONITOR_TYPE_QUANTITY,
 		MONITOR_TYPE_QUANTITY,
@@ -503,6 +524,7 @@ Performance::MonitorType Performance::get_monitor_type(Monitor p_monitor) const
 		MONITOR_TYPE_QUANTITY,
 		MONITOR_TYPE_QUANTITY,
 		MONITOR_TYPE_QUANTITY,
+#endif // _3D_DISABLED
 
 	};
 	static_assert((sizeof(types) / sizeof(MonitorType)) == MONITOR_MAX);
@@ -522,9 +544,9 @@ void Performance::set_navigation_process_time(double p_pt) {
 	_navigation_process_time = p_pt;
 }
 
-void Performance::add_custom_monitor(const StringName &p_id, const Callable &p_callable, const Vector<Variant> &p_args) {
+void Performance::add_custom_monitor(const StringName &p_id, const Callable &p_callable, const Vector<Variant> &p_args, MonitorType p_type) {
 	ERR_FAIL_COND_MSG(has_custom_monitor(p_id), "Custom monitor with id '" + String(p_id) + "' already exists.");
-	_monitor_map.insert(p_id, MonitorCall(p_callable, p_args));
+	_monitor_map.insert(p_id, MonitorCall(p_type, p_callable, p_args));
 	_monitor_modification_time = OS::get_singleton()->get_ticks_usec();
 }
 
@@ -561,6 +583,20 @@ TypedArray<StringName> Performance::get_custom_monitor_names() {
 	return return_array;
 }
 
+Vector<int> Performance::get_custom_monitor_types() {
+	if (_monitor_map.is_empty()) {
+		return Vector<int>();
+	}
+	Vector<int> ret;
+	ret.resize(_monitor_map.size());
+	int index = 0;
+	for (const KeyValue<StringName, MonitorCall> &i : _monitor_map) {
+		ret.set(index, (int)i.value.get_monitor_type());
+		index++;
+	}
+	return ret;
+}
+
 uint64_t Performance::get_monitor_modification_time() {
 	return _monitor_modification_time;
 }
@@ -573,7 +609,8 @@ Performance::Performance() {
 	singleton = this;
 }
 
-Performance::MonitorCall::MonitorCall(Callable p_callable, Vector<Variant> p_arguments) {
+Performance::MonitorCall::MonitorCall(Performance::MonitorType p_type, const Callable &p_callable, const Vector<Variant> &p_arguments) {
+	_type = p_type;
 	_callable = p_callable;
 	_arguments = p_arguments;
 }
