@@ -78,6 +78,12 @@ public:
 	~GDScriptParserRef();
 };
 
+#ifdef TESTS_ENABLED
+namespace GDScriptTests {
+class TestGDScriptCacheAccessor;
+}
+#endif // TESTS_ENABLED
+
 class GDScriptCache {
 	// String key is full path.
 	HashMap<String, GDScriptParserRef *> parser_map;
@@ -91,6 +97,9 @@ class GDScriptCache {
 	friend class GDScript;
 	friend class GDScriptParserRef;
 	friend class GDScriptInstance;
+#ifdef TESTS_ENABLED
+	friend class GDScriptTests::TestGDScriptCacheAccessor;
+#endif // TESTS_ENABLED
 
 	static GDScriptCache *singleton;
 
@@ -112,6 +121,11 @@ public:
 	static String get_source_code(const String &p_path);
 	static Vector<uint8_t> get_binary_tokens(const String &p_path);
 	static Ref<GDScript> get_shallow_script(const String &p_path, Error &r_error, const String &p_owner = String());
+	/**
+	 * Returns a fully loaded GDScript using an already cached script if one exists.
+	 *
+	 * The returned instance is present in GDScriptCache and ResourceCache.
+	 */
 	static Ref<GDScript> get_full_script(const String &p_path, Error &r_error, const String &p_owner = String(), bool p_update_from_disk = false);
 	static Ref<GDScript> get_cached_script(const String &p_path);
 	static Error finish_compiling(const String &p_owner);
