@@ -34,10 +34,9 @@
 #include "gdscript_extend_parser.h"
 #include "gdscript_language_protocol.h"
 
-#include "core/os/os.h"
-#include "editor/editor_settings.h"
-#include "editor/plugins/script_text_editor.h"
-#include "servers/display_server.h"
+#include "editor/script/script_text_editor.h"
+#include "editor/settings/editor_settings.h"
+#include "servers/display/display_server.h"
 
 void GDScriptTextDocument::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("didOpen"), &GDScriptTextDocument::didOpen);
@@ -64,29 +63,21 @@ void GDScriptTextDocument::_bind_methods() {
 }
 
 void GDScriptTextDocument::didOpen(const Variant &p_param) {
-	lsp::TextDocumentItem doc = load_document_item(p_param);
-	sync_script_content(doc.uri, doc.text);
-}
-
-void GDScriptTextDocument::didClose(const Variant &p_param) {
-	// Left empty on purpose. Godot does nothing special on closing a document,
-	// but it satisfies LSP clients that require didClose be implemented.
+	GDScriptLanguageProtocol::get_singleton()->lsp_did_open(p_param);
 }
 
 void GDScriptTextDocument::didChange(const Variant &p_param) {
-	lsp::TextDocumentItem doc = load_document_item(p_param);
-	Dictionary dict = p_param;
-	Array contentChanges = dict["contentChanges"];
-	for (int i = 0; i < contentChanges.size(); ++i) {
-		lsp::TextDocumentContentChangeEvent evt;
-		evt.load(contentChanges[i]);
-		doc.text = evt.text;
-	}
-	sync_script_content(doc.uri, doc.text);
+	GDScriptLanguageProtocol::get_singleton()->lsp_did_change(p_param);
+}
+
+void GDScriptTextDocument::didClose(const Variant &p_param) {
+	GDScriptLanguageProtocol::get_singleton()->lsp_did_close(p_param);
 }
 
 void GDScriptTextDocument::willSaveWaitUntil(const Variant &p_param) {
-	lsp::TextDocumentItem doc = load_document_item(p_param);
+	Dictionary dict = p_param;
+	LSP::TextDocumentIdentifier doc;
+	doc.load(dict["textDocument"]);
 
 	String path = GDScriptLanguageProtocol::get_singleton()->get_workspace()->get_file_path(doc.uri);
 	Ref<Script> scr = ResourceLoader::load(path);
@@ -96,12 +87,11 @@ void GDScriptTextDocument::willSaveWaitUntil(const Variant &p_param) {
 }
 
 void GDScriptTextDocument::didSave(const Variant &p_param) {
-	lsp::TextDocumentItem doc = load_document_item(p_param);
 	Dictionary dict = p_param;
+	LSP::TextDocumentIdentifier doc;
+	doc.load(dict["textDocument"]);
 	String text = dict["text"];
 
-	sync_script_content(doc.uri, text);
-
 	String path = GDScriptLanguageProtocol::get_singleton()->get_workspace()->get_file_path(doc.uri);
 	Ref<GDScript> scr = ResourceLoader::load(path);
 	if (scr.is_valid() && (scr->load_source_code(path) == OK)) {
@@ -112,20 +102,22 @@ void GDScriptTextDocument::didSave(const Variant &p_param) {
 		}
 
 		scr->update_exports();
-		ScriptEditor::get_singleton()->reload_scripts(true);
-		ScriptEditor::get_singleton()->update_docs_from_script(scr);
-		ScriptEditor::get_singleton()->trigger_live_script_reload(scr->get_path());
+
+		if (!Thread::is_main_thread()) {
+			callable_mp(this, &GDScriptTextDocument::reload_script).call_deferred(scr);
+		} else {
+			reload_script(scr);
+		}
 	}
 }
 
-lsp::TextDocumentItem GDScriptTextDocument::load_document_item(const Variant &p_param) {
-	lsp::TextDocumentItem doc;
-	Dictionary params = p_param;
-	doc.load(params["textDocument"]);
-	return doc;
+void GDScriptTextDocument::reload_script(Ref<GDScript> p_to_reload_script) {
+	ScriptEditor::get_singleton()->reload_scripts(true);
+	ScriptEditor::get_singleton()->update_docs_from_script(p_to_reload_script);
+	ScriptEditor::get_singleton()->trigger_live_script_reload(p_to_reload_script->get_path());
 }
 
-void GDScriptTextDocument::notify_client_show_symbol(const lsp::DocumentSymbol *symbol) {
+void GDScriptTextDocument::notify_client_show_symbol(const LSP::DocumentSymbol *symbol) {
 	ERR_FAIL_NULL(symbol);
 	GDScriptLanguageProtocol::get_singleton()->notify_client("gdscript/show_native_symbol", symbol->to_json(true));
 }
@@ -135,9 +127,9 @@ void GDScriptTextDocument::initialize() {
 		for (const KeyValue<StringName, ClassMembers> &E : GDScriptLanguageProtocol::get_singleton()->get_workspace()->native_members) {
 			const ClassMembers &members = E.value;
 
-			for (const KeyValue<String, const lsp::DocumentSymbol *> &F : members) {
-				const lsp::DocumentSymbol *symbol = members.get(F.key);
-				lsp::CompletionItem item = symbol->make_completion_item();
+			for (const KeyValue<String, const LSP::DocumentSymbol *> &F : members) {
+				const LSP::DocumentSymbol *symbol = members.get(F.key);
+				LSP::CompletionItem item = symbol->make_completion_item();
 				item.data = JOIN_SYMBOLS(String(E.key), F.key);
 				native_member_completions.push_back(item.to_json());
 			}
@@ -148,10 +140,10 @@ void GDScriptTextDocument::initialize() {
 Variant GDScriptTextDocument::nativeSymbol(const Dictionary &p_params) {
 	Variant ret;
 
-	lsp::NativeSymbolInspectParams params;
+	LSP::NativeSymbolInspectParams params;
 	params.load(p_params);
 
-	if (const lsp::DocumentSymbol *symbol = GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_native_symbol(params)) {
+	if (const LSP::DocumentSymbol *symbol = GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_native_symbol(params)) {
 		ret = symbol->to_json(true);
 		notify_client_show_symbol(symbol);
 	}
@@ -164,8 +156,10 @@ Array GDScriptTextDocument::documentSymbol(const Dictionary &p_params) {
 	String uri = params["uri"];
 	String path = GDScriptLanguageProtocol::get_singleton()->get_workspace()->get_file_path(uri);
 	Array arr;
-	if (HashMap<String, ExtendGDScriptParser *>::ConstIterator parser = GDScriptLanguageProtocol::get_singleton()->get_workspace()->scripts.find(path)) {
-		lsp::DocumentSymbol symbol = parser->value->get_symbols();
+
+	ExtendGDScriptParser *parser = GDScriptLanguageProtocol::get_singleton()->get_parse_result(path);
+	if (parser) {
+		LSP::DocumentSymbol symbol = parser->get_symbols();
 		arr.push_back(symbol.to_json(true));
 	}
 	return arr;
@@ -174,7 +168,7 @@ Array GDScriptTextDocument::documentSymbol(const Dictionary &p_params) {
 Array GDScriptTextDocument::completion(const Dictionary &p_params) {
 	Array arr;
 
-	lsp::CompletionParams params;
+	LSP::CompletionParams params;
 	params.load(p_params);
 	Dictionary request_data = params.to_json();
 
@@ -186,41 +180,41 @@ Array GDScriptTextDocument::completion(const Dictionary &p_params) {
 		arr.resize(options.size());
 
 		for (const ScriptLanguage::CodeCompletionOption &option : options) {
-			lsp::CompletionItem item;
+			LSP::CompletionItem item;
 			item.label = option.display;
 			item.data = request_data;
 			item.insertText = option.insert_text;
 
 			switch (option.kind) {
 				case ScriptLanguage::CODE_COMPLETION_KIND_ENUM:
-					item.kind = lsp::CompletionItemKind::Enum;
+					item.kind = LSP::CompletionItemKind::Enum;
 					break;
 				case ScriptLanguage::CODE_COMPLETION_KIND_CLASS:
-					item.kind = lsp::CompletionItemKind::Class;
+					item.kind = LSP::CompletionItemKind::Class;
 					break;
 				case ScriptLanguage::CODE_COMPLETION_KIND_MEMBER:
-					item.kind = lsp::CompletionItemKind::Property;
+					item.kind = LSP::CompletionItemKind::Property;
 					break;
 				case ScriptLanguage::CODE_COMPLETION_KIND_FUNCTION:
-					item.kind = lsp::CompletionItemKind::Method;
+					item.kind = LSP::CompletionItemKind::Method;
 					break;
 				case ScriptLanguage::CODE_COMPLETION_KIND_SIGNAL:
-					item.kind = lsp::CompletionItemKind::Event;
+					item.kind = LSP::CompletionItemKind::Event;
 					break;
 				case ScriptLanguage::CODE_COMPLETION_KIND_CONSTANT:
-					item.kind = lsp::CompletionItemKind::Constant;
+					item.kind = LSP::CompletionItemKind::Constant;
 					break;
 				case ScriptLanguage::CODE_COMPLETION_KIND_VARIABLE:
-					item.kind = lsp::CompletionItemKind::Variable;
+					item.kind = LSP::CompletionItemKind::Variable;
 					break;
 				case ScriptLanguage::CODE_COMPLETION_KIND_FILE_PATH:
-					item.kind = lsp::CompletionItemKind::File;
+					item.kind = LSP::CompletionItemKind::File;
 					break;
 				case ScriptLanguage::CODE_COMPLETION_KIND_NODE_PATH:
-					item.kind = lsp::CompletionItemKind::Snippet;
+					item.kind = LSP::CompletionItemKind::Snippet;
 					break;
 				case ScriptLanguage::CODE_COMPLETION_KIND_PLAIN_TEXT:
-					item.kind = lsp::CompletionItemKind::Text;
+					item.kind = LSP::CompletionItemKind::Text;
 					break;
 				default: {
 				}
@@ -229,25 +223,12 @@ Array GDScriptTextDocument::completion(const Dictionary &p_params) {
 			arr[i] = item.to_json();
 			i++;
 		}
-	} else if (GDScriptLanguageProtocol::get_singleton()->is_smart_resolve_enabled()) {
-		arr = native_member_completions.duplicate();
-
-		for (KeyValue<String, ExtendGDScriptParser *> &E : GDScriptLanguageProtocol::get_singleton()->get_workspace()->scripts) {
-			ExtendGDScriptParser *scr = E.value;
-			const Array &items = scr->get_member_completions();
-
-			const int start_size = arr.size();
-			arr.resize(start_size + items.size());
-			for (int i = start_size; i < arr.size(); i++) {
-				arr[i] = items[i - start_size];
-			}
-		}
 	}
 	return arr;
 }
 
 Dictionary GDScriptTextDocument::rename(const Dictionary &p_params) {
-	lsp::TextDocumentPositionParams params;
+	LSP::TextDocumentPositionParams params;
 	params.load(p_params);
 	String new_name = p_params["newName"];
 
@@ -255,11 +236,11 @@ Dictionary GDScriptTextDocument::rename(const Dictionary &p_params) {
 }
 
 Variant GDScriptTextDocument::prepareRename(const Dictionary &p_params) {
-	lsp::TextDocumentPositionParams params;
+	LSP::TextDocumentPositionParams params;
 	params.load(p_params);
 
-	lsp::DocumentSymbol symbol;
-	lsp::Range range;
+	LSP::DocumentSymbol symbol;
+	LSP::Range range;
 	if (GDScriptLanguageProtocol::get_singleton()->get_workspace()->can_rename(params, symbol, range)) {
 		return Variant(range.to_json());
 	}
@@ -271,16 +252,16 @@ Variant GDScriptTextDocument::prepareRename(const Dictionary &p_params) {
 Array GDScriptTextDocument::references(const Dictionary &p_params) {
 	Array res;
 
-	lsp::ReferenceParams params;
+	LSP::ReferenceParams params;
 	params.load(p_params);
 
-	const lsp::DocumentSymbol *symbol = GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_symbol(params);
+	const LSP::DocumentSymbol *symbol = GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_symbol(params);
 	if (symbol) {
-		Vector<lsp::Location> usages = GDScriptLanguageProtocol::get_singleton()->get_workspace()->find_all_usages(*symbol);
+		Vector<LSP::Location> usages = GDScriptLanguageProtocol::get_singleton()->get_workspace()->find_all_usages(*symbol);
 		res.resize(usages.size());
 		int declaration_adjustment = 0;
 		for (int i = 0; i < usages.size(); i++) {
-			lsp::Location usage = usages[i];
+			LSP::Location usage = usages[i];
 			if (!params.context.includeDeclaration && usage.range == symbol->range) {
 				declaration_adjustment++;
 				continue;
@@ -297,22 +278,22 @@ Array GDScriptTextDocument::references(const Dictionary &p_params) {
 }
 
 Dictionary GDScriptTextDocument::resolve(const Dictionary &p_params) {
-	lsp::CompletionItem item;
+	LSP::CompletionItem item;
 	item.load(p_params);
 
-	lsp::CompletionParams params;
+	LSP::CompletionParams params;
 	Variant data = p_params["data"];
 
-	const lsp::DocumentSymbol *symbol = nullptr;
+	const LSP::DocumentSymbol *symbol = nullptr;
 
 	if (data.get_type() == Variant::DICTIONARY) {
 		params.load(p_params["data"]);
-		symbol = GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_symbol(params, item.label, item.kind == lsp::CompletionItemKind::Method || item.kind == lsp::CompletionItemKind::Function);
+		symbol = GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_symbol(params, item.label, item.kind == LSP::CompletionItemKind::Method || item.kind == LSP::CompletionItemKind::Function);
 
-	} else if (data.get_type() == Variant::STRING) {
+	} else if (data.is_string()) {
 		String query = data;
 
-		Vector<String> param_symbols = query.split(SYMBOL_SEPERATOR, false);
+		Vector<String> param_symbols = query.split(SYMBOL_SEPARATOR, false);
 
 		if (param_symbols.size() >= 2) {
 			StringName class_name = param_symbols[0];
@@ -323,14 +304,15 @@ Dictionary GDScriptTextDocument::resolve(const Dictionary &p_params) {
 			}
 
 			if (const ClassMembers *members = GDScriptLanguageProtocol::get_singleton()->get_workspace()->native_members.getptr(class_name)) {
-				if (const lsp::DocumentSymbol *const *member = members->getptr(member_name)) {
+				if (const LSP::DocumentSymbol *const *member = members->getptr(member_name)) {
 					symbol = *member;
 				}
 			}
 
 			if (!symbol) {
-				if (HashMap<String, ExtendGDScriptParser *>::ConstIterator E = GDScriptLanguageProtocol::get_singleton()->get_workspace()->scripts.find(class_name)) {
-					symbol = E->value->get_member_symbol(member_name, inner_class_name);
+				ExtendGDScriptParser *parser = GDScriptLanguageProtocol::get_singleton()->get_parse_result(class_name);
+				if (parser) {
+					symbol = parser->get_member_symbol(member_name, inner_class_name);
 				}
 			}
 		}
@@ -340,15 +322,15 @@ Dictionary GDScriptTextDocument::resolve(const Dictionary &p_params) {
 		item.documentation = symbol->render();
 	}
 
-	if (item.kind == lsp::CompletionItemKind::Event) {
-		if (params.context.triggerKind == lsp::CompletionTriggerKind::TriggerCharacter && (params.context.triggerCharacter == "(")) {
+	if (item.kind == LSP::CompletionItemKind::Event) {
+		if (params.context.triggerKind == LSP::CompletionTriggerKind::TriggerCharacter && (params.context.triggerCharacter == "(")) {
 			const String quote_style = EDITOR_GET("text_editor/completion/use_single_quotes") ? "'" : "\"";
 			item.insertText = item.label.quote(quote_style);
 		}
 	}
 
-	if (item.kind == lsp::CompletionItemKind::Method) {
-		bool is_trigger_character = params.context.triggerKind == lsp::CompletionTriggerKind::TriggerCharacter;
+	if (item.kind == LSP::CompletionItemKind::Method) {
+		bool is_trigger_character = params.context.triggerKind == LSP::CompletionTriggerKind::TriggerCharacter;
 		bool is_quote_character = params.context.triggerCharacter == "\"" || params.context.triggerCharacter == "'";
 
 		if (is_trigger_character && is_quote_character && item.insertText.is_quoted()) {
@@ -360,41 +342,38 @@ Dictionary GDScriptTextDocument::resolve(const Dictionary &p_params) {
 }
 
 Array GDScriptTextDocument::foldingRange(const Dictionary &p_params) {
-	Array arr;
-	return arr;
+	return Array();
 }
 
 Array GDScriptTextDocument::codeLens(const Dictionary &p_params) {
-	Array arr;
-	return arr;
+	return Array();
 }
 
 Array GDScriptTextDocument::documentLink(const Dictionary &p_params) {
 	Array ret;
 
-	lsp::DocumentLinkParams params;
+	LSP::DocumentLinkParams params;
 	params.load(p_params);
 
-	List<lsp::DocumentLink> links;
+	List<LSP::DocumentLink> links;
 	GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_document_links(params.textDocument.uri, links);
-	for (const lsp::DocumentLink &E : links) {
+	for (const LSP::DocumentLink &E : links) {
 		ret.push_back(E.to_json());
 	}
 	return ret;
 }
 
 Array GDScriptTextDocument::colorPresentation(const Dictionary &p_params) {
-	Array arr;
-	return arr;
+	return Array();
 }
 
 Variant GDScriptTextDocument::hover(const Dictionary &p_params) {
-	lsp::TextDocumentPositionParams params;
+	LSP::TextDocumentPositionParams params;
 	params.load(p_params);
 
-	const lsp::DocumentSymbol *symbol = GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_symbol(params);
+	const LSP::DocumentSymbol *symbol = GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_symbol(params);
 	if (symbol) {
-		lsp::Hover hover;
+		LSP::Hover hover;
 		hover.contents = symbol->render();
 		hover.range.start = params.position;
 		hover.range.end = params.position;
@@ -403,10 +382,10 @@ Variant GDScriptTextDocument::hover(const Dictionary &p_params) {
 	} else if (GDScriptLanguageProtocol::get_singleton()->is_smart_resolve_enabled()) {
 		Dictionary ret;
 		Array contents;
-		List<const lsp::DocumentSymbol *> list;
-		GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_related_symbols(params, list);
-		for (const lsp::DocumentSymbol *&E : list) {
-			if (const lsp::DocumentSymbol *s = E) {
+		List<const LSP::DocumentSymbol *> list;
+		GDScriptLanguageProtocol::get_singleton()->resolve_related_symbols(params, list);
+		for (const LSP::DocumentSymbol *&E : list) {
+			if (const LSP::DocumentSymbol *s = E) {
 				contents.push_back(s->render().value);
 			}
 		}
@@ -418,38 +397,37 @@ Variant GDScriptTextDocument::hover(const Dictionary &p_params) {
 }
 
 Array GDScriptTextDocument::definition(const Dictionary &p_params) {
-	lsp::TextDocumentPositionParams params;
+	LSP::TextDocumentPositionParams params;
 	params.load(p_params);
-	List<const lsp::DocumentSymbol *> symbols;
-	Array arr = find_symbols(params, symbols);
-	return arr;
+	List<const LSP::DocumentSymbol *> symbols;
+	return find_symbols(params, symbols);
 }
 
 Variant GDScriptTextDocument::declaration(const Dictionary &p_params) {
-	lsp::TextDocumentPositionParams params;
+	LSP::TextDocumentPositionParams params;
 	params.load(p_params);
-	List<const lsp::DocumentSymbol *> symbols;
+	List<const LSP::DocumentSymbol *> symbols;
 	Array arr = find_symbols(params, symbols);
 	if (arr.is_empty() && !symbols.is_empty() && !symbols.front()->get()->native_class.is_empty()) { // Find a native symbol
-		const lsp::DocumentSymbol *symbol = symbols.front()->get();
+		const LSP::DocumentSymbol *symbol = symbols.front()->get();
 		if (GDScriptLanguageProtocol::get_singleton()->is_goto_native_symbols_enabled()) {
 			String id;
 			switch (symbol->kind) {
-				case lsp::SymbolKind::Class:
+				case LSP::SymbolKind::Class:
 					id = "class_name:" + symbol->name;
 					break;
-				case lsp::SymbolKind::Constant:
+				case LSP::SymbolKind::Constant:
 					id = "class_constant:" + symbol->native_class + ":" + symbol->name;
 					break;
-				case lsp::SymbolKind::Property:
-				case lsp::SymbolKind::Variable:
+				case LSP::SymbolKind::Property:
+				case LSP::SymbolKind::Variable:
 					id = "class_property:" + symbol->native_class + ":" + symbol->name;
 					break;
-				case lsp::SymbolKind::Enum:
+				case LSP::SymbolKind::Enum:
 					id = "class_enum:" + symbol->native_class + ":" + symbol->name;
 					break;
-				case lsp::SymbolKind::Method:
-				case lsp::SymbolKind::Function:
+				case LSP::SymbolKind::Method:
+				case LSP::SymbolKind::Function:
 					id = "class_method:" + symbol->native_class + ":" + symbol->name;
 					break;
 				default:
@@ -467,10 +445,10 @@ Variant GDScriptTextDocument::declaration(const Dictionary &p_params) {
 Variant GDScriptTextDocument::signatureHelp(const Dictionary &p_params) {
 	Variant ret;
 
-	lsp::TextDocumentPositionParams params;
+	LSP::TextDocumentPositionParams params;
 	params.load(p_params);
 
-	lsp::SignatureHelp s;
+	LSP::SignatureHelp s;
 	if (OK == GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_signature(params, s)) {
 		ret = s.to_json();
 	}
@@ -482,38 +460,33 @@ GDScriptTextDocument::GDScriptTextDocument() {
 	file_checker = FileAccess::create(FileAccess::ACCESS_RESOURCES);
 }
 
-void GDScriptTextDocument::sync_script_content(const String &p_path, const String &p_content) {
-	String path = GDScriptLanguageProtocol::get_singleton()->get_workspace()->get_file_path(p_path);
-	GDScriptLanguageProtocol::get_singleton()->get_workspace()->parse_script(path, p_content);
-
-	EditorFileSystem::get_singleton()->update_file(path);
-}
-
 void GDScriptTextDocument::show_native_symbol_in_editor(const String &p_symbol_id) {
 	callable_mp(ScriptEditor::get_singleton(), &ScriptEditor::goto_help).call_deferred(p_symbol_id);
 
 	DisplayServer::get_singleton()->window_move_to_foreground();
 }
 
-Array GDScriptTextDocument::find_symbols(const lsp::TextDocumentPositionParams &p_location, List<const lsp::DocumentSymbol *> &r_list) {
+Array GDScriptTextDocument::find_symbols(const LSP::TextDocumentPositionParams &p_location, List<const LSP::DocumentSymbol *> &r_list) {
 	Array arr;
-	const lsp::DocumentSymbol *symbol = GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_symbol(p_location);
+	const LSP::DocumentSymbol *symbol = GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_symbol(p_location);
 	if (symbol) {
-		lsp::Location location;
+		LSP::Location location;
 		location.uri = symbol->uri;
-		location.range = symbol->selectionRange;
-		const String &path = GDScriptLanguageProtocol::get_singleton()->get_workspace()->get_file_path(symbol->uri);
-		if (file_checker->file_exists(path)) {
-			arr.push_back(location.to_json());
+		if (!location.uri.is_empty()) {
+			location.range = symbol->selectionRange;
+			const String &path = GDScriptLanguageProtocol::get_singleton()->get_workspace()->get_file_path(symbol->uri);
+			if (file_checker->file_exists(path)) {
+				arr.push_back(location.to_json());
+			}
 		}
 		r_list.push_back(symbol);
 	} else if (GDScriptLanguageProtocol::get_singleton()->is_smart_resolve_enabled()) {
-		List<const lsp::DocumentSymbol *> list;
-		GDScriptLanguageProtocol::get_singleton()->get_workspace()->resolve_related_symbols(p_location, list);
-		for (const lsp::DocumentSymbol *&E : list) {
-			if (const lsp::DocumentSymbol *s = E) {
+		List<const LSP::DocumentSymbol *> list;
+		GDScriptLanguageProtocol::get_singleton()->resolve_related_symbols(p_location, list);
+		for (const LSP::DocumentSymbol *&E : list) {
+			if (const LSP::DocumentSymbol *s = E) {
 				if (!s->uri.is_empty()) {
-					lsp::Location location;
+					LSP::Location location;
 					location.uri = s->uri;
 					location.range = s->selectionRange;
 					arr.push_back(location.to_json());
