@@ -60,6 +60,8 @@ void GLTFNode::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("append_child_index", "child_index"), &GLTFNode::append_child_index);
 	ClassDB::bind_method(D_METHOD("get_light"), &GLTFNode::get_light);
 	ClassDB::bind_method(D_METHOD("set_light", "light"), &GLTFNode::set_light);
+	ClassDB::bind_method(D_METHOD("get_visible"), &GLTFNode::get_visible);
+	ClassDB::bind_method(D_METHOD("set_visible", "visible"), &GLTFNode::set_visible);
 	ClassDB::bind_method(D_METHOD("get_additional_data", "extension_name"), &GLTFNode::get_additional_data);
 	ClassDB::bind_method(D_METHOD("set_additional_data", "extension_name", "additional_data"), &GLTFNode::set_additional_data);
 	ClassDB::bind_method(D_METHOD("get_scene_node_path", "gltf_state", "handle_skeletons"), &GLTFNode::get_scene_node_path, DEFVAL(true));
@@ -77,12 +79,13 @@ void GLTFNode::_bind_methods() {
 	ADD_PROPERTY(PropertyInfo(Variant::VECTOR3, "scale"), "set_scale", "get_scale"); // Vector3
 	ADD_PROPERTY(PropertyInfo(Variant::PACKED_INT32_ARRAY, "children"), "set_children", "get_children"); // Vector<int>
 	ADD_PROPERTY(PropertyInfo(Variant::INT, "light"), "set_light", "get_light"); // GLTFLightIndex
+	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "visible"), "set_visible", "get_visible"); // bool
 }
 
 String GLTFNode::get_original_name() {
 	return original_name;
 }
-void GLTFNode::set_original_name(String p_name) {
+void GLTFNode::set_original_name(const String &p_name) {
 	original_name = p_name;
 }
 
@@ -106,7 +109,7 @@ Transform3D GLTFNode::get_xform() {
 	return transform;
 }
 
-void GLTFNode::set_xform(Transform3D p_xform) {
+void GLTFNode::set_xform(const Transform3D &p_xform) {
 	transform = p_xform;
 }
 
@@ -146,7 +149,7 @@ Vector3 GLTFNode::get_position() {
 	return transform.origin;
 }
 
-void GLTFNode::set_position(Vector3 p_position) {
+void GLTFNode::set_position(const Vector3 &p_position) {
 	transform.origin = p_position;
 }
 
@@ -154,7 +157,7 @@ Quaternion GLTFNode::get_rotation() {
 	return transform.basis.get_rotation_quaternion();
 }
 
-void GLTFNode::set_rotation(Quaternion p_rotation) {
+void GLTFNode::set_rotation(const Quaternion &p_rotation) {
 	transform.basis.set_quaternion_scale(p_rotation, transform.basis.get_scale());
 }
 
@@ -162,16 +165,16 @@ Vector3 GLTFNode::get_scale() {
 	return transform.basis.get_scale();
 }
 
-void GLTFNode::set_scale(Vector3 p_scale) {
+void GLTFNode::set_scale(const Vector3 &p_scale) {
 	transform.basis = transform.basis.orthonormalized() * Basis::from_scale(p_scale);
 }
 
 Vector<int> GLTFNode::get_children() {
-	return children;
+	return Vector<int>(children);
 }
 
-void GLTFNode::set_children(Vector<int> p_children) {
-	children = p_children;
+void GLTFNode::set_children(const Vector<int> &p_children) {
+	children = Vector<int>(p_children);
 }
 
 void GLTFNode::append_child_index(int p_child_index) {
@@ -186,6 +189,14 @@ void GLTFNode::set_light(GLTFLightIndex p_light) {
 	light = p_light;
 }
 
+bool GLTFNode::get_visible() {
+	return visible;
+}
+
+void GLTFNode::set_visible(bool p_visible) {
+	visible = p_visible;
+}
+
 Variant GLTFNode::get_additional_data(const StringName &p_extension_name) {
 	return additional_data[p_extension_name];
 }
