@@ -118,7 +118,7 @@ namespace GodotTools
 
         private void _ShowDotnetFeatures()
         {
-            _bottomPanelBtn.Show();
+            MSBuildPanel.Open();
             _toolBarBuildButton.Show();
         }
 
@@ -456,12 +456,6 @@ namespace GodotTools
             }
         }
 
-        private void BuildStateChanged()
-        {
-            if (_bottomPanelBtn != null)
-                _bottomPanelBtn.Icon = MSBuildPanel.GetBuildStateIcon();
-        }
-
         public override void _EnablePlugin()
         {
             base._EnablePlugin();
@@ -511,8 +505,7 @@ namespace GodotTools
             _confirmCreateSlnDialog.Confirmed += () => CreateProjectSolution();
 
             MSBuildPanel = new MSBuildPanel();
-            MSBuildPanel.BuildStateChanged += BuildStateChanged;
-            _bottomPanelBtn = AddControlToBottomPanel(MSBuildPanel, "MSBuild".TTR());
+            AddDock(MSBuildPanel);
 
             AddChild(new HotReloadAssemblyWatcher { Name = "HotReloadAssemblyWatcher" });
 
@@ -546,7 +539,7 @@ namespace GodotTools
             }
             else
             {
-                _bottomPanelBtn.Hide();
+                MSBuildPanel.Close();
                 _toolBarBuildButton.Hide();
             }
             _menuPopup.AddItem("Create C# solution".TTR(), (int)MenuOptions.CreateSln);
@@ -643,6 +636,9 @@ namespace GodotTools
             AddInspectorPlugin(inspectorPlugin);
             _inspectorPluginWeak = WeakRef(inspectorPlugin);
 
+            // TranslationParser Plugin
+            AddTranslationParserPlugin(new CsTranslationParserPlugin());
+
             BuildManager.Initialize();
 
             GodotIdeManager = new GodotIdeManager();
@@ -654,9 +650,6 @@ namespace GodotTools
             base._DisablePlugin();
 
             _editorSettings.SettingsChanged -= OnSettingsChanged;
-
-            // Custom signals aren't automatically disconnected currently.
-            MSBuildPanel.BuildStateChanged -= BuildStateChanged;
         }
 
         public override void _ExitTree()
