@@ -1539,7 +1539,7 @@ namespace Godot
                 if (end < 0)
                     end = len;
                 if (allowEmpty || end > from)
-                    ret.Add(float.Parse(instance.AsSpan(from), CultureInfo.InvariantCulture));
+                    ret.Add(float.Parse(instance.AsSpan(from, end - from), CultureInfo.InvariantCulture));
                 if (end == len)
                     break;
 
