@@ -36,11 +36,10 @@
 #include "nav_region_iteration_2d.h"
 
 #include "core/math/geometry_2d.h"
-#include "servers/navigation/navigation_utilities.h"
 
 using namespace Nav2D;
 
-#define THREE_POINTS_CROSS_PRODUCT(m_a, m_b, m_c) (((m_c) - (m_a)).cross((m_b) - (m_a)))
+#define THREE_POINTS_CROSS_PRODUCT(m_a, m_b, m_c) (-((m_c) - (m_a)).cross((m_b) - (m_a)))
 
 bool NavMeshQueries2D::emit_callback(const Callable &p_callback) {
 	ERR_FAIL_COND_V(!p_callback.is_valid(), false);
@@ -151,7 +150,7 @@ void NavMeshQueries2D::map_query_path(NavMap2D *p_map, const Ref<NavigationPathQ
 	ERR_FAIL_COND(p_query_parameters.is_null());
 	ERR_FAIL_COND(p_query_result.is_null());
 
-	using namespace NavigationUtilities;
+	using namespace NavigationDefaults2D;
 
 	NavMeshQueries2D::NavMeshPathQueryTask2D query_task;
 	query_task.start_position = p_query_parameters->get_start_position();
@@ -897,7 +896,7 @@ ClosestPointQueryResult NavMeshQueries2D::map_iteration_get_closest_point_info(c
 	const LocalVector<Ref<NavRegionIteration2D>> &regions = p_map_iteration.region_iterations;
 	for (const Ref<NavRegionIteration2D> &region : regions) {
 		for (const Polygon &polygon : region->get_navmesh_polygons()) {
-			real_t cross = (polygon.vertices[1] - polygon.vertices[0]).cross(polygon.vertices[2] - polygon.vertices[0]);
+			real_t cross = -(polygon.vertices[1] - polygon.vertices[0]).cross(polygon.vertices[2] - polygon.vertices[0]);
 			Vector2 closest_on_polygon;
 			real_t closest = FLT_MAX;
 			bool inside = true;
@@ -905,7 +904,7 @@ ClosestPointQueryResult NavMeshQueries2D::map_iteration_get_closest_point_info(c
 			for (uint32_t point_id = 0; point_id < polygon.vertices.size(); ++point_id) {
 				Vector2 edge = polygon.vertices[point_id] - previous;
 				Vector2 to_point = p_point - previous;
-				real_t edge_to_point_cross = edge.cross(to_point);
+				real_t edge_to_point_cross = -edge.cross(to_point);
 				bool clockwise = (edge_to_point_cross * cross) > 0;
 				// If we are not clockwise, the point will never be inside the polygon and so the closest point will be on an edge.
 				if (!clockwise) {
@@ -1029,7 +1028,7 @@ ClosestPointQueryResult NavMeshQueries2D::polygons_get_closest_point_info(const
 	// TODO: Check for further 2D improvements.
 
 	for (const Polygon &polygon : p_polygons) {
-		real_t cross = (polygon.vertices[1] - polygon.vertices[0]).cross(polygon.vertices[2] - polygon.vertices[0]);
+		real_t cross = -(polygon.vertices[1] - polygon.vertices[0]).cross(polygon.vertices[2] - polygon.vertices[0]);
 		Vector2 closest_on_polygon;
 		real_t closest = FLT_MAX;
 		bool inside = true;
@@ -1037,7 +1036,7 @@ ClosestPointQueryResult NavMeshQueries2D::polygons_get_closest_point_info(const
 		for (uint32_t point_id = 0; point_id < polygon.vertices.size(); ++point_id) {
 			Vector2 edge = polygon.vertices[point_id] - previous;
 			Vector2 to_point = p_point - previous;
-			real_t edge_to_point_cross = edge.cross(to_point);
+			real_t edge_to_point_cross = -edge.cross(to_point);
 			bool clockwise = (edge_to_point_cross * cross) > 0;
 			// If we are not clockwise, the point will never be inside the polygon and so the closest point will be on an edge.
 			if (!clockwise) {
@@ -1156,7 +1155,7 @@ bool NavMeshQueries2D::_query_task_is_connection_owner_usable(const NavMeshPathQ
 
 	if (p_query_task.exclude_regions || p_query_task.include_regions) {
 		switch (p_owner->get_type()) {
-			case NavigationUtilities::PathSegmentType::PATH_SEGMENT_TYPE_REGION: {
+			case NavigationEnums2D::PathSegmentType::PATH_SEGMENT_TYPE_REGION: {
 				if (p_query_task.exclude_regions && p_query_task.excluded_regions.has(p_owner->get_self())) {
 					// Not usable. Exclude region filter is active and this region is excluded.
 					owner_usable = false;
@@ -1165,7 +1164,7 @@ bool NavMeshQueries2D::_query_task_is_connection_owner_usable(const NavMeshPathQ
 					owner_usable = false;
 				}
 			} break;
-			case NavigationUtilities::PathSegmentType::PATH_SEGMENT_TYPE_LINK: {
+			case NavigationEnums2D::PathSegmentType::PATH_SEGMENT_TYPE_LINK: {
 				const LocalVector<Polygon> &link_polygons = p_owner->get_navmesh_polygons();
 				if (link_polygons.size() != 2) {
 					// Not usable. Whatever this is, it is not a valid connected link.
