@@ -31,13 +31,15 @@
 #include "openxr_action_map_editor.h"
 
 #include "core/config/project_settings.h"
+#include "core/io/dir_access.h"
 #include "editor/editor_node.h"
-#include "editor/editor_settings.h"
-#include "editor/gui/editor_bottom_panel.h"
 #include "editor/gui/editor_file_dialog.h"
+#include "editor/settings/editor_command_palette.h"
+#include "editor/settings/editor_settings.h"
 #include "editor/themes/editor_scale.h"
 
-// TODO implement redo/undo system
+HashMap<String, String> OpenXRActionMapEditor::interaction_profile_editors;
+HashMap<String, String> OpenXRActionMapEditor::binding_modifier_editors;
 
 void OpenXRActionMapEditor::_bind_methods() {
 	ClassDB::bind_method("_add_action_set_editor", &OpenXRActionMapEditor::_add_action_set_editor);
@@ -50,12 +52,17 @@ void OpenXRActionMapEditor::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("_do_remove_action_set_editor", "action_set_editor"), &OpenXRActionMapEditor::_do_remove_action_set_editor);
 	ClassDB::bind_method(D_METHOD("_do_add_interaction_profile_editor", "interaction_profile_editor"), &OpenXRActionMapEditor::_do_add_interaction_profile_editor);
 	ClassDB::bind_method(D_METHOD("_do_remove_interaction_profile_editor", "interaction_profile_editor"), &OpenXRActionMapEditor::_do_remove_interaction_profile_editor);
+
+	ClassDB::bind_static_method("OpenXRActionMapEditor", D_METHOD("register_interaction_profile_editor", "interaction_profile_path", "editor_class"), &OpenXRActionMapEditor::register_interaction_profile_editor);
+	ClassDB::bind_static_method("OpenXRActionMapEditor", D_METHOD("register_binding_modifier_editor", "binding_modifier_class", "editor_class"), &OpenXRActionMapEditor::register_binding_modifier_editor);
 }
 
 void OpenXRActionMapEditor::_notification(int p_what) {
 	switch (p_what) {
-		case NOTIFICATION_ENTER_TREE:
 		case NOTIFICATION_THEME_CHANGED: {
+			const String theme_style = EDITOR_GET("interface/theme/style");
+			tabs->set_theme_type_variation(theme_style == "Classic" ? "TabContainerOdd" : "TabContainerInner");
+
 			for (int i = 0; i < tabs->get_child_count(); i++) {
 				Control *tab = Object::cast_to<Control>(tabs->get_child(i));
 				if (tab) {
@@ -71,7 +78,7 @@ void OpenXRActionMapEditor::_notification(int p_what) {
 	}
 }
 
-OpenXRActionSetEditor *OpenXRActionMapEditor::_add_action_set_editor(Ref<OpenXRActionSet> p_action_set) {
+OpenXRActionSetEditor *OpenXRActionMapEditor::_add_action_set_editor(const Ref<OpenXRActionSet> &p_action_set) {
 	ERR_FAIL_COND_V(p_action_set.is_null(), nullptr);
 
 	OpenXRActionSetEditor *action_set_editor = memnew(OpenXRActionSetEditor(action_map, p_action_set));
@@ -93,26 +100,39 @@ void OpenXRActionMapEditor::_create_action_sets() {
 	}
 }
 
-OpenXRInteractionProfileEditorBase *OpenXRActionMapEditor::_add_interaction_profile_editor(Ref<OpenXRInteractionProfile> p_interaction_profile) {
+OpenXRInteractionProfileEditorBase *OpenXRActionMapEditor::_add_interaction_profile_editor(const Ref<OpenXRInteractionProfile> &p_interaction_profile) {
 	ERR_FAIL_COND_V(p_interaction_profile.is_null(), nullptr);
 
 	String profile_path = p_interaction_profile->get_interaction_profile_path();
 
 	// need to instance the correct editor for our profile
 	OpenXRInteractionProfileEditorBase *new_profile_editor = nullptr;
-	if (profile_path == "placeholder_text") {
-		// instance specific editor for this type
-	} else {
+	if (interaction_profile_editors.has(profile_path)) {
+		Object *new_editor = ClassDB::instantiate(interaction_profile_editors[profile_path]);
+		if (new_editor) {
+			new_profile_editor = Object::cast_to<OpenXRInteractionProfileEditorBase>(new_editor);
+			if (!new_profile_editor) {
+				WARN_PRINT("Interaction profile editor type mismatch for " + profile_path);
+				memfree(new_editor);
+			}
+		}
+	}
+	if (!new_profile_editor) {
 		// instance generic editor
-		new_profile_editor = memnew(OpenXRInteractionProfileEditor(action_map, p_interaction_profile));
+		new_profile_editor = memnew(OpenXRInteractionProfileEditor);
 	}
 
 	// now add it in..
 	ERR_FAIL_NULL_V(new_profile_editor, nullptr);
+	new_profile_editor->setup(action_map, p_interaction_profile);
 	tabs->add_child(new_profile_editor);
 	new_profile_editor->add_theme_style_override(SceneStringName(panel), get_theme_stylebox(SceneStringName(panel), SNAME("Tree")));
 	tabs->set_tab_button_icon(tabs->get_tab_count() - 1, get_theme_icon(SNAME("close"), SNAME("TabBar")));
 
+	if (!new_profile_editor->tooltip.is_empty()) {
+		tabs->set_tab_tooltip(tabs->get_tab_count() - 1, new_profile_editor->tooltip);
+	}
+
 	return new_profile_editor;
 }
 
@@ -126,7 +146,7 @@ void OpenXRActionMapEditor::_create_interaction_profiles() {
 	}
 }
 
-OpenXRActionSetEditor *OpenXRActionMapEditor::_add_action_set(String p_name) {
+OpenXRActionSetEditor *OpenXRActionMapEditor::_add_action_set(const String &p_name) {
 	ERR_FAIL_COND_V(action_map.is_null(), nullptr);
 	Ref<OpenXRActionSet> new_action_set;
 
@@ -148,7 +168,7 @@ OpenXRActionSetEditor *OpenXRActionMapEditor::_add_action_set(String p_name) {
 	return action_set_editor;
 }
 
-void OpenXRActionMapEditor::_remove_action_set(String p_name) {
+void OpenXRActionMapEditor::_remove_action_set(const String &p_name) {
 	ERR_FAIL_COND(action_map.is_null());
 	Ref<OpenXRActionSet> action_set = action_map->find_action_set(p_name);
 	ERR_FAIL_COND(action_set.is_null());
@@ -195,8 +215,19 @@ void OpenXRActionMapEditor::_on_remove_action_set(Object *p_action_set_editor) {
 	Ref<OpenXRActionSet> action_set = action_set_editor->get_action_set();
 	ERR_FAIL_COND(action_set.is_null());
 
+	// Remove all actions first.
 	action_set_editor->remove_all_actions();
 
+	// Make sure we update our interaction profiles.
+	for (int i = 0; i < tabs->get_tab_count(); i++) {
+		// First tab won't be an interaction profile editor, but being thorough..
+		OpenXRInteractionProfileEditorBase *interaction_profile_editor = Object::cast_to<OpenXRInteractionProfileEditorBase>(tabs->get_tab_control(i));
+		if (interaction_profile_editor) {
+			interaction_profile_editor->remove_all_for_action_set(action_set);
+		}
+	}
+
+	// And now we can remove our action set.
 	undo_redo->create_action(TTR("Remove action set"));
 	undo_redo->add_do_method(this, "_do_remove_action_set_editor", action_set_editor);
 	undo_redo->add_undo_method(this, "_do_add_action_set_editor", action_set_editor);
@@ -205,12 +236,12 @@ void OpenXRActionMapEditor::_on_remove_action_set(Object *p_action_set_editor) {
 	action_map->set_edited(true);
 }
 
-void OpenXRActionMapEditor::_on_action_removed(Ref<OpenXRAction> p_action) {
+void OpenXRActionMapEditor::_on_action_removed(const Ref<OpenXRAction> &p_action) {
 	for (int i = 0; i < tabs->get_tab_count(); i++) {
 		// First tab won't be an interaction profile editor, but being thorough..
 		OpenXRInteractionProfileEditorBase *interaction_profile_editor = Object::cast_to<OpenXRInteractionProfileEditorBase>(tabs->get_tab_control(i));
 		if (interaction_profile_editor) {
-			interaction_profile_editor->remove_all_bindings_for_action(p_action);
+			interaction_profile_editor->remove_all_for_action(p_action);
 		}
 	}
 }
@@ -227,7 +258,7 @@ void OpenXRActionMapEditor::_on_add_interaction_profile() {
 	select_interaction_profile_dialog->open(already_selected);
 }
 
-void OpenXRActionMapEditor::_on_interaction_profile_selected(const String p_path) {
+void OpenXRActionMapEditor::_on_interaction_profile_selected(const String &p_path) {
 	ERR_FAIL_COND(action_map.is_null());
 
 	Ref<OpenXRInteractionProfile> new_profile;
@@ -246,13 +277,13 @@ void OpenXRActionMapEditor::_on_interaction_profile_selected(const String p_path
 	tabs->set_current_tab(tabs->get_tab_count() - 1);
 }
 
-void OpenXRActionMapEditor::_load_action_map(const String p_path, bool p_create_new_if_missing) {
+void OpenXRActionMapEditor::_load_action_map(const String &p_path, bool p_create_new_if_missing) {
 	Error err = OK;
 	Ref<DirAccess> da = DirAccess::create(DirAccess::ACCESS_RESOURCES);
 	if (da->file_exists(p_path)) {
 		action_map = ResourceLoader::load(p_path, "", ResourceFormatLoader::CACHE_MODE_REUSE, &err);
 		if (err != OK) {
-			EditorNode::get_singleton()->show_warning(vformat(TTR("Error loading %s: %s."), edited_path, error_names[err]));
+			EditorNode::get_singleton()->show_warning(vformat(TTR("Error loading %s: %s."), edited_path, TTR(error_names[err])));
 
 			edited_path = "";
 			header_label->set_text("");
@@ -261,13 +292,12 @@ void OpenXRActionMapEditor::_load_action_map(const String p_path, bool p_create_
 	} else if (p_create_new_if_missing) {
 		action_map.instantiate();
 		action_map->create_default_action_sets();
-		action_map->set_path(p_path);
 
 		// Save it immediately
 		err = ResourceSaver::save(action_map, p_path);
 		if (err != OK) {
 			// Show warning but continue.
-			EditorNode::get_singleton()->show_warning(vformat(TTR("Error saving file %s: %s"), edited_path, error_names[err]));
+			EditorNode::get_singleton()->show_warning(vformat(TTR("Error saving file %s: %s"), p_path, TTR(error_names[err])));
 		}
 	}
 
@@ -278,7 +308,7 @@ void OpenXRActionMapEditor::_load_action_map(const String p_path, bool p_create_
 void OpenXRActionMapEditor::_on_save_action_map() {
 	Error err = ResourceSaver::save(action_map, edited_path);
 	if (err != OK) {
-		EditorNode::get_singleton()->show_warning(vformat(TTR("Error saving file %s: %s"), edited_path, error_names[err]));
+		EditorNode::get_singleton()->show_warning(vformat(TTR("Error saving file %s: %s"), edited_path, TTR(error_names[err])));
 		return;
 	}
 
@@ -357,8 +387,8 @@ void OpenXRActionMapEditor::_do_remove_interaction_profile_editor(OpenXRInteract
 	action_map->remove_interaction_profile(interaction_profile);
 }
 
-void OpenXRActionMapEditor::open_action_map(String p_path) {
-	EditorNode::get_bottom_panel()->make_item_visible(this);
+void OpenXRActionMapEditor::open_action_map(const String &p_path) {
+	make_visible();
 
 	// out with the old...
 	_clear_action_map();
@@ -387,12 +417,37 @@ void OpenXRActionMapEditor::_clear_action_map() {
 	}
 }
 
+void OpenXRActionMapEditor::register_interaction_profile_editor(const String &p_for_path, const String &p_editor_class) {
+	interaction_profile_editors[p_for_path] = p_editor_class;
+}
+
+void OpenXRActionMapEditor::register_binding_modifier_editor(const String &p_binding_modifier_class, const String &p_editor_class) {
+	binding_modifier_editors[p_binding_modifier_class] = p_editor_class;
+}
+
+String OpenXRActionMapEditor::get_binding_modifier_editor_class(const String &p_binding_modifier_class) {
+	if (binding_modifier_editors.has(p_binding_modifier_class)) {
+		return binding_modifier_editors[p_binding_modifier_class];
+	}
+
+	return OpenXRBindingModifierEditor::get_class_static();
+}
+
 OpenXRActionMapEditor::OpenXRActionMapEditor() {
+	set_name(TTRC("OpenXR Action Map"));
+	set_icon_name("OpenXRActionMap");
+	set_dock_shortcut(ED_SHORTCUT_AND_COMMAND("bottom_panels/toggle_openxr_action_map_bottom_panel", TTRC("Toggle OpenXR Action Map Dock")));
+	set_default_slot(EditorDock::DOCK_SLOT_BOTTOM);
+	set_available_layouts(EditorDock::DOCK_LAYOUT_HORIZONTAL | EditorDock::DOCK_LAYOUT_FLOATING);
+	set_custom_minimum_size(Size2(0.0, 300.0 * EDSCALE));
+
 	undo_redo = EditorUndoRedoManager::get_singleton();
-	set_custom_minimum_size(Size2(0.0, 300.0));
+
+	VBoxContainer *main_vb = memnew(VBoxContainer);
+	add_child(main_vb);
 
 	top_hb = memnew(HBoxContainer);
-	add_child(top_hb);
+	main_vb->add_child(top_hb);
 
 	header_label = memnew(Label);
 	header_label->set_text(String(TTR("Action Map")));
@@ -430,10 +485,9 @@ OpenXRActionMapEditor::OpenXRActionMapEditor() {
 	tabs = memnew(TabContainer);
 	tabs->set_h_size_flags(SIZE_EXPAND_FILL);
 	tabs->set_v_size_flags(SIZE_EXPAND_FILL);
-	tabs->set_theme_type_variation("TabContainerOdd");
 	tabs->connect("tab_changed", callable_mp(this, &OpenXRActionMapEditor::_on_tabs_tab_changed));
 	tabs->connect("tab_button_pressed", callable_mp(this, &OpenXRActionMapEditor::_on_tab_button_pressed));
-	add_child(tabs);
+	main_vb->add_child(tabs);
 
 	actionsets_scroll = memnew(ScrollContainer);
 	actionsets_scroll->set_h_size_flags(SIZE_EXPAND_FILL);
@@ -452,8 +506,5 @@ OpenXRActionMapEditor::OpenXRActionMapEditor() {
 
 	// Our Action map editor is only shown if openxr is enabled in project settings
 	// So load our action map and if it doesn't exist, create it right away.
-	_load_action_map(GLOBAL_GET("xr/openxr/default_action_map"), true);
-}
-
-OpenXRActionMapEditor::~OpenXRActionMapEditor() {
+	_load_action_map(ResourceUID::ensure_path(GLOBAL_GET("xr/openxr/default_action_map")), true);
 }
