@@ -36,6 +36,7 @@
 #include "openxr_select_interaction_profile_dialog.h"
 
 #include "core/templates/hash_map.h"
+#include "editor/docks/editor_dock.h"
 #include "editor/editor_undo_redo_manager.h"
 #include "editor/plugins/editor_plugin.h"
 #include "scene/gui/box_container.h"
@@ -44,8 +45,8 @@
 #include "scene/gui/scroll_container.h"
 #include "scene/gui/tab_container.h"
 
-class OpenXRActionMapEditor : public VBoxContainer {
-	GDCLASS(OpenXRActionMapEditor, VBoxContainer);
+class OpenXRActionMapEditor : public EditorDock {
+	GDCLASS(OpenXRActionMapEditor, EditorDock);
 
 private:
 	static HashMap<String, String> interaction_profile_editors; // interaction profile path, interaction profile editor
@@ -67,23 +68,23 @@ private:
 	VBoxContainer *actionsets_vb = nullptr;
 	OpenXRSelectInteractionProfileDialog *select_interaction_profile_dialog = nullptr;
 
-	OpenXRActionSetEditor *_add_action_set_editor(Ref<OpenXRActionSet> p_action_set);
+	OpenXRActionSetEditor *_add_action_set_editor(const Ref<OpenXRActionSet> &p_action_set);
 	void _create_action_sets();
-	OpenXRInteractionProfileEditorBase *_add_interaction_profile_editor(Ref<OpenXRInteractionProfile> p_interaction_profile);
+	OpenXRInteractionProfileEditorBase *_add_interaction_profile_editor(const Ref<OpenXRInteractionProfile> &p_interaction_profile);
 	void _create_interaction_profiles();
 
-	OpenXRActionSetEditor *_add_action_set(String p_name);
-	void _remove_action_set(String p_name);
+	OpenXRActionSetEditor *_add_action_set(const String &p_name);
+	void _remove_action_set(const String &p_name);
 
 	void _on_add_action_set();
 	void _set_focus_on_action_set(OpenXRActionSetEditor *p_action_set_editor);
 	void _on_remove_action_set(Object *p_action_set_editor);
-	void _on_action_removed(Ref<OpenXRAction> p_action);
+	void _on_action_removed(const Ref<OpenXRAction> &p_action);
 
 	void _on_add_interaction_profile();
-	void _on_interaction_profile_selected(const String p_path);
+	void _on_interaction_profile_selected(const String &p_path);
 
-	void _load_action_map(const String p_path, bool p_create_new_if_missing = false);
+	void _load_action_map(const String &p_path, bool p_create_new_if_missing = false);
 	void _on_save_action_map();
 	void _on_reset_to_default_layout();
 
@@ -107,7 +108,7 @@ public:
 	static void register_binding_modifier_editor(const String &p_binding_modifier_class, const String &p_editor_class);
 	static String get_binding_modifier_editor_class(const String &p_binding_modifier_class);
 
-	void open_action_map(String p_path);
+	void open_action_map(const String &p_path);
 
 	OpenXRActionMapEditor();
 };
