@@ -72,13 +72,13 @@ public:
 	virtual void _theme_changed();
 
 	void _do_update_interaction_profile();
-	void _add_binding(const String p_action, const String p_path);
-	void _remove_binding(const String p_action, const String p_path);
+	void _add_binding(const String &p_action, const String &p_path);
+	void _remove_binding(const String &p_action, const String &p_path);
 
-	void remove_all_for_action_set(Ref<OpenXRActionSet> p_action_set);
-	void remove_all_for_action(Ref<OpenXRAction> p_action);
+	void remove_all_for_action_set(const Ref<OpenXRActionSet> &p_action_set);
+	void remove_all_for_action(const Ref<OpenXRAction> &p_action);
 
-	virtual void setup(Ref<OpenXRActionMap> p_action_map, Ref<OpenXRInteractionProfile> p_interaction_profile);
+	virtual void setup(const Ref<OpenXRActionMap> &p_action_map, const Ref<OpenXRInteractionProfile> &p_interaction_profile);
 
 	OpenXRInteractionProfileEditorBase();
 };
@@ -95,13 +95,13 @@ private:
 	void _add_io_path(VBoxContainer *p_container, const OpenXRInteractionProfileMetadata::IOPath *p_io_path);
 
 public:
-	void select_action_for(const String p_io_path);
-	void _on_action_selected(const String p_action);
-	void _on_remove_pressed(const String p_action, const String p_for_io_path);
+	void select_action_for(const String &p_io_path);
+	void _on_action_selected(const String &p_action);
+	void _on_remove_pressed(const String &p_action, const String &p_for_io_path);
 
 	virtual void _update_interaction_profile() override;
 	virtual void _theme_changed() override;
-	virtual void setup(Ref<OpenXRActionMap> p_action_map, Ref<OpenXRInteractionProfile> p_interaction_profile) override;
+	virtual void setup(const Ref<OpenXRActionMap> &p_action_map, const Ref<OpenXRInteractionProfile> &p_interaction_profile) override;
 
 	OpenXRInteractionProfileEditor();
 };
