@@ -51,8 +51,8 @@ protected:
 	void _notification(int p_what);
 
 public:
-	void _on_select_interaction_profile(const String p_interaction_profile);
-	void open(PackedStringArray p_do_not_include);
+	void _on_select_interaction_profile(const String &p_interaction_profile);
+	void open(const PackedStringArray &p_do_not_include);
 	virtual void ok_pressed() override;
 
 	OpenXRSelectInteractionProfileDialog();
