@@ -44,7 +44,7 @@ public:
 	OpenXREyeGazeInteractionExtension();
 	~OpenXREyeGazeInteractionExtension();
 
-	virtual HashMap<String, bool *> get_requested_extensions() override;
+	virtual HashMap<String, bool *> get_requested_extensions(XrVersion p_version) override;
 	virtual void *set_system_properties_and_get_next_pointer(void *p_next_pointer) override;
 
 	PackedStringArray get_suggested_tracker_names() override;
