@@ -30,6 +30,8 @@
 
 #include "openxr_local_floor_extension.h"
 
+#include "../openxr_api.h"
+
 OpenXRLocalFloorExtension *OpenXRLocalFloorExtension::singleton = nullptr;
 
 OpenXRLocalFloorExtension *OpenXRLocalFloorExtension::get_singleton() {
@@ -44,10 +46,13 @@ OpenXRLocalFloorExtension::~OpenXRLocalFloorExtension() {
 	singleton = nullptr;
 }
 
-HashMap<String, bool *> OpenXRLocalFloorExtension::get_requested_extensions() {
+HashMap<String, bool *> OpenXRLocalFloorExtension::get_requested_extensions(XrVersion p_version) {
 	HashMap<String, bool *> request_extensions;
 
-	request_extensions[XR_EXT_LOCAL_FLOOR_EXTENSION_NAME] = &available;
+	if (p_version < XR_API_VERSION_1_1_0) {
+		// Extension was promoted in OpenXR 1.1, only include it in OpenXR 1.0.
+		request_extensions[XR_EXT_LOCAL_FLOOR_EXTENSION_NAME] = &available;
+	}
 
 	return request_extensions;
 }
