@@ -35,6 +35,8 @@
 #include "core/debugger/remote_debugger_peer.h"
 
 class RemoteDebuggerPeerWebSocket : public RemoteDebuggerPeer {
+	GDSOFTCLASS(RemoteDebuggerPeerWebSocket, RemoteDebuggerPeer);
+
 	Ref<WebSocketPeer> ws_peer;
 	List<Array> in_queue;
 	List<Array> out_queue;
@@ -55,5 +57,5 @@ public:
 	void poll() override;
 	bool can_block() const override;
 
-	RemoteDebuggerPeerWebSocket(Ref<WebSocketPeer> p_peer = Ref<WebSocketPeer>());
+	RemoteDebuggerPeerWebSocket(const Ref<WebSocketPeer> &p_peer = Ref<WebSocketPeer>());
 };
