@@ -38,17 +38,11 @@
 
 #include <jni.h>
 
-struct jvalret {
-	jobject obj;
-	jvalue val;
-	jvalret() { obj = nullptr; }
-};
-
-jvalret _variant_to_jvalue(JNIEnv *env, Variant::Type p_type, const Variant *p_arg, bool force_jobject = false);
+jobject _variant_to_jobject(JNIEnv *env, Variant::Type p_type, const Variant *p_arg, int p_depth = 0);
 
 String _get_class_name(JNIEnv *env, jclass cls, bool *array);
 
-Variant _jobject_to_variant(JNIEnv *env, jobject obj);
+Variant _jobject_to_variant(JNIEnv *env, jobject obj, int p_depth = 0);
 
 Variant::Type get_jni_type(const String &p_type);
 
