@@ -35,7 +35,7 @@
 #include "core/os/os.h"
 #include "core/templates/local_vector.h"
 #include "drivers/egl/egl_manager.h"
-#include "servers/display_server.h"
+#include "servers/display/display_server.h"
 
 #include <X11/Xlib.h>
 
