@@ -34,7 +34,7 @@
 #include "core/templates/hash_map.h"
 #include "core/templates/list.h"
 #include "core/variant/array.h"
-#include "servers/display_server.h"
+#include "servers/display/display_server.h"
 
 #import <AppKit/AppKit.h>
 
@@ -47,12 +47,12 @@
 @interface TTS_MacOS : NSObject <AVSpeechSynthesizerDelegate> {
 	// AVSpeechSynthesizer
 	bool speaking;
-	HashMap<id, int> ids;
+	HashMap<id, int64_t> ids;
 
 	// NSSpeechSynthesizer
 	bool paused;
 	bool have_utterance;
-	int last_utterance;
+	int64_t last_utterance;
 
 	id synth; // NSSpeechSynthesizer or AVSpeechSynthesizer
 	List<DisplayServer::TTSUtterance> queue;
@@ -63,6 +63,6 @@
 - (void)stopSpeaking;
 - (bool)isSpeaking;
 - (bool)isPaused;
-- (void)speak:(const String &)text voice:(const String &)voice volume:(int)volume pitch:(float)pitch rate:(float)rate utterance_id:(int)utterance_id interrupt:(bool)interrupt;
+- (void)speak:(const String &)text voice:(const String &)voice volume:(int)volume pitch:(float)pitch rate:(float)rate utterance_id:(int64_t)utterance_id interrupt:(bool)interrupt;
 - (Array)getVoices;
 @end
