@@ -42,6 +42,8 @@
 #include "editor/export/editor_export_platform.h"
 #include "main/splash.gen.h"
 
+class ImageTexture;
+
 class EditorExportPlatformWeb : public EditorExportPlatform {
 	GDCLASS(EditorExportPlatformWeb, EditorExportPlatform);
 
@@ -148,6 +150,6 @@ public:
 
 	String get_debug_protocol() const override { return "ws://"; }
 
-	EditorExportPlatformWeb();
+	virtual void initialize() override;
 	~EditorExportPlatformWeb();
 };
