@@ -396,42 +396,25 @@ void CPUParticles2D::set_param_curve(Parameter p_param, const Ref<Curve> &p_curv
 	curve_parameters[p_param] = p_curve;
 
 	switch (p_param) {
-		case PARAM_INITIAL_LINEAR_VELOCITY: {
-			//do none for this one
-		} break;
-		case PARAM_ANGULAR_VELOCITY: {
-			_adjust_curve_range(p_curve, -360, 360);
-		} break;
-		case PARAM_ORBIT_VELOCITY: {
-			_adjust_curve_range(p_curve, -500, 500);
-		} break;
-		case PARAM_LINEAR_ACCEL: {
-			_adjust_curve_range(p_curve, -200, 200);
-		} break;
-		case PARAM_RADIAL_ACCEL: {
-			_adjust_curve_range(p_curve, -200, 200);
-		} break;
-		case PARAM_TANGENTIAL_ACCEL: {
-			_adjust_curve_range(p_curve, -200, 200);
-		} break;
-		case PARAM_DAMPING: {
-			_adjust_curve_range(p_curve, 0, 100);
-		} break;
-		case PARAM_ANGLE: {
-			_adjust_curve_range(p_curve, -360, 360);
-		} break;
-		case PARAM_SCALE: {
-		} break;
+		case PARAM_ANGULAR_VELOCITY:
+		case PARAM_ORBIT_VELOCITY:
+		case PARAM_LINEAR_ACCEL:
+		case PARAM_RADIAL_ACCEL:
+		case PARAM_TANGENTIAL_ACCEL:
+		case PARAM_ANGLE:
 		case PARAM_HUE_VARIATION: {
 			_adjust_curve_range(p_curve, -1, 1);
 		} break;
-		case PARAM_ANIM_SPEED: {
-			_adjust_curve_range(p_curve, 0, 200);
-		} break;
+		case PARAM_DAMPING:
+		case PARAM_SCALE:
+		case PARAM_ANIM_SPEED:
 		case PARAM_ANIM_OFFSET: {
+			_adjust_curve_range(p_curve, 0, 1);
+		} break;
+		case PARAM_INITIAL_LINEAR_VELOCITY:
+		case PARAM_MAX: {
+			// No curve available.
 		} break;
-		default: {
-		}
 	}
 
 	update_configuration_warnings();
@@ -524,6 +507,14 @@ void CPUParticles2D::set_emission_colors(const Vector<Color> &p_colors) {
 	emission_colors = p_colors;
 }
 
+void CPUParticles2D::set_emission_ring_inner_radius(real_t p_inner_radius) {
+	emission_ring_inner_radius = p_inner_radius;
+}
+
+void CPUParticles2D::set_emission_ring_radius(real_t p_ring_radius) {
+	emission_ring_radius = p_ring_radius;
+}
+
 real_t CPUParticles2D::get_emission_sphere_radius() const {
 	return emission_sphere_radius;
 }
@@ -544,6 +535,14 @@ Vector<Color> CPUParticles2D::get_emission_colors() const {
 	return emission_colors;
 }
 
+real_t CPUParticles2D::get_emission_ring_inner_radius() const {
+	return emission_ring_inner_radius;
+}
+
+real_t CPUParticles2D::get_emission_ring_radius() const {
+	return emission_ring_radius;
+}
+
 CPUParticles2D::EmissionShape CPUParticles2D::get_emission_shape() const {
 	return emission_shape;
 }
@@ -647,6 +646,13 @@ void CPUParticles2D::_validate_property(PropertyInfo &p_property) const {
 		p_property.usage = PROPERTY_USAGE_NONE;
 	}
 
+	if (p_property.name == "emission_ring_inner_radius" && emission_shape != EMISSION_SHAPE_RING) {
+		p_property.usage = PROPERTY_USAGE_NONE;
+	}
+	if (p_property.name == "emission_ring_radius" && emission_shape != EMISSION_SHAPE_RING) {
+		p_property.usage = PROPERTY_USAGE_NONE;
+	}
+
 	if (p_property.name == "seed" && !use_fixed_seed) {
 		p_property.usage = PROPERTY_USAGE_NONE;
 	}
@@ -926,6 +932,13 @@ void CPUParticles2D::_particles_process(double p_delta) {
 						p.base_color = emission_colors.get(random_idx);
 					}
 				} break;
+				case EMISSION_SHAPE_RING: {
+					real_t t = Math::TAU * Math::randf();
+					real_t outer_sq = emission_ring_radius * emission_ring_radius;
+					real_t inner_sq = emission_ring_inner_radius * emission_ring_inner_radius;
+					real_t radius = Math::sqrt(Math::randf() * (outer_sq - inner_sq) + inner_sq);
+					p.transform[2] = Vector2(Math::cos(t), Math::sin(t)) * radius;
+				} break;
 				case EMISSION_SHAPE_MAX: { // Max value for validity check.
 					break;
 				}
@@ -1379,6 +1392,9 @@ void CPUParticles2D::convert_from_particles(Node *p_particles) {
 	Vector2 rect_extents = Vector2(proc_mat->get_emission_box_extents().x, proc_mat->get_emission_box_extents().y);
 	set_emission_rect_extents(rect_extents);
 
+	set_emission_ring_radius(proc_mat->get_emission_ring_radius());
+	set_emission_ring_inner_radius(proc_mat->get_emission_ring_inner_radius());
+
 	Ref<CurveXYZTexture> scale3D = proc_mat->get_param_texture(ParticleProcessMaterial::PARAM_SCALE);
 	if (scale3D.is_valid()) {
 		split_scale = true;
@@ -1527,6 +1543,12 @@ void CPUParticles2D::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("set_emission_colors", "array"), &CPUParticles2D::set_emission_colors);
 	ClassDB::bind_method(D_METHOD("get_emission_colors"), &CPUParticles2D::get_emission_colors);
 
+	ClassDB::bind_method(D_METHOD("set_emission_ring_inner_radius", "inner_radius"), &CPUParticles2D::set_emission_ring_inner_radius);
+	ClassDB::bind_method(D_METHOD("get_emission_ring_inner_radius"), &CPUParticles2D::get_emission_ring_inner_radius);
+
+	ClassDB::bind_method(D_METHOD("set_emission_ring_radius", "radius"), &CPUParticles2D::set_emission_ring_radius);
+	ClassDB::bind_method(D_METHOD("get_emission_ring_radius"), &CPUParticles2D::get_emission_ring_radius);
+
 	ClassDB::bind_method(D_METHOD("get_gravity"), &CPUParticles2D::get_gravity);
 	ClassDB::bind_method(D_METHOD("set_gravity", "accel_vec"), &CPUParticles2D::set_gravity);
 
@@ -1544,12 +1566,14 @@ void CPUParticles2D::_bind_methods() {
 	ADD_SIGNAL(MethodInfo("finished"));
 
 	ADD_GROUP("Emission Shape", "emission_");
-	ADD_PROPERTY(PropertyInfo(Variant::INT, "emission_shape", PROPERTY_HINT_ENUM, "Point,Sphere,Sphere Surface,Rectangle,Points,Directed Points", PROPERTY_USAGE_DEFAULT | PROPERTY_USAGE_UPDATE_ALL_IF_MODIFIED), "set_emission_shape", "get_emission_shape");
+	ADD_PROPERTY(PropertyInfo(Variant::INT, "emission_shape", PROPERTY_HINT_ENUM, "Point,Sphere,Sphere Surface,Rectangle,Points,Directed Points,Ring", PROPERTY_USAGE_DEFAULT | PROPERTY_USAGE_UPDATE_ALL_IF_MODIFIED), "set_emission_shape", "get_emission_shape");
 	ADD_PROPERTY(PropertyInfo(Variant::FLOAT, "emission_sphere_radius", PROPERTY_HINT_RANGE, "0.01,128,0.01,suffix:px"), "set_emission_sphere_radius", "get_emission_sphere_radius");
 	ADD_PROPERTY(PropertyInfo(Variant::VECTOR2, "emission_rect_extents", PROPERTY_HINT_NONE, "suffix:px"), "set_emission_rect_extents", "get_emission_rect_extents");
 	ADD_PROPERTY(PropertyInfo(Variant::PACKED_VECTOR2_ARRAY, "emission_points"), "set_emission_points", "get_emission_points");
 	ADD_PROPERTY(PropertyInfo(Variant::PACKED_VECTOR2_ARRAY, "emission_normals"), "set_emission_normals", "get_emission_normals");
 	ADD_PROPERTY(PropertyInfo(Variant::PACKED_COLOR_ARRAY, "emission_colors"), "set_emission_colors", "get_emission_colors");
+	ADD_PROPERTY(PropertyInfo(Variant::FLOAT, "emission_ring_inner_radius"), "set_emission_ring_inner_radius", "get_emission_ring_inner_radius");
+	ADD_PROPERTY(PropertyInfo(Variant::FLOAT, "emission_ring_radius"), "set_emission_ring_radius", "get_emission_ring_radius");
 	ADD_GROUP("Particle Flags", "particle_flag_");
 	ADD_PROPERTYI(PropertyInfo(Variant::BOOL, "particle_flag_align_y"), "set_particle_flag", "get_particle_flag", PARTICLE_FLAG_ALIGN_Y_TO_VELOCITY);
 	ADD_GROUP("Direction", "");
@@ -1638,6 +1662,7 @@ void CPUParticles2D::_bind_methods() {
 	BIND_ENUM_CONSTANT(EMISSION_SHAPE_RECTANGLE);
 	BIND_ENUM_CONSTANT(EMISSION_SHAPE_POINTS);
 	BIND_ENUM_CONSTANT(EMISSION_SHAPE_DIRECTED_POINTS);
+	BIND_ENUM_CONSTANT(EMISSION_SHAPE_RING);
 	BIND_ENUM_CONSTANT(EMISSION_SHAPE_MAX);
 }
 
@@ -1696,6 +1721,6 @@ CPUParticles2D::CPUParticles2D() {
 
 CPUParticles2D::~CPUParticles2D() {
 	ERR_FAIL_NULL(RenderingServer::get_singleton());
-	RS::get_singleton()->free(multimesh);
-	RS::get_singleton()->free(mesh);
+	RS::get_singleton()->free_rid(multimesh);
+	RS::get_singleton()->free_rid(mesh);
 }
