@@ -37,6 +37,7 @@
 #include "scene/main/viewport.h"
 #include "scene/resources/curve_texture.h"
 #include "scene/resources/gradient_texture.h"
+#include "scene/resources/mesh.h"
 #include "scene/resources/particle_process_material.h"
 
 AABB CPUParticles3D::get_aabb() const {
@@ -335,42 +336,25 @@ void CPUParticles3D::set_param_curve(Parameter p_param, const Ref<Curve> &p_curv
 	curve_parameters[p_param] = p_curve;
 
 	switch (p_param) {
-		case PARAM_INITIAL_LINEAR_VELOCITY: {
-			//do none for this one
-		} break;
-		case PARAM_ANGULAR_VELOCITY: {
-			_adjust_curve_range(p_curve, -360, 360);
-		} break;
-		case PARAM_ORBIT_VELOCITY: {
-			_adjust_curve_range(p_curve, -500, 500);
-		} break;
-		case PARAM_LINEAR_ACCEL: {
-			_adjust_curve_range(p_curve, -200, 200);
-		} break;
-		case PARAM_RADIAL_ACCEL: {
-			_adjust_curve_range(p_curve, -200, 200);
-		} break;
-		case PARAM_TANGENTIAL_ACCEL: {
-			_adjust_curve_range(p_curve, -200, 200);
-		} break;
-		case PARAM_DAMPING: {
-			_adjust_curve_range(p_curve, 0, 100);
-		} break;
-		case PARAM_ANGLE: {
-			_adjust_curve_range(p_curve, -360, 360);
-		} break;
-		case PARAM_SCALE: {
-		} break;
+		case PARAM_ANGULAR_VELOCITY:
+		case PARAM_ORBIT_VELOCITY:
+		case PARAM_LINEAR_ACCEL:
+		case PARAM_RADIAL_ACCEL:
+		case PARAM_TANGENTIAL_ACCEL:
+		case PARAM_ANGLE:
 		case PARAM_HUE_VARIATION: {
 			_adjust_curve_range(p_curve, -1, 1);
 		} break;
-		case PARAM_ANIM_SPEED: {
-			_adjust_curve_range(p_curve, 0, 200);
-		} break;
+		case PARAM_DAMPING:
+		case PARAM_SCALE:
+		case PARAM_ANIM_SPEED:
 		case PARAM_ANIM_OFFSET: {
+			_adjust_curve_range(p_curve, 0, 1);
+		} break;
+		case PARAM_INITIAL_LINEAR_VELOCITY:
+		case PARAM_MAX: {
+			// No curve available.
 		} break;
-		default: {
-		}
 	}
 
 	update_configuration_warnings();
@@ -827,7 +811,7 @@ void CPUParticles3D::_particles_process(double p_delta) {
 				tex_anim_offset = curve_parameters[PARAM_ANGLE]->sample(tv);
 			}
 
-			p.seed = seed + uint32_t(1) + i + cycle;
+			p.seed = seed + uint32_t(1) + i + cycle * pcount;
 			rng->set_seed(p.seed);
 			p.angle_rand = rng->randf();
 			p.scale_rand = rng->randf();
@@ -1822,5 +1806,5 @@ CPUParticles3D::CPUParticles3D() {
 
 CPUParticles3D::~CPUParticles3D() {
 	ERR_FAIL_NULL(RenderingServer::get_singleton());
-	RS::get_singleton()->free(multimesh);
+	RS::get_singleton()->free_rid(multimesh);
 }
