@@ -31,6 +31,7 @@
 #include "collision_object_3d.h"
 
 #include "scene/resources/3d/shape_3d.h"
+#include "scene/resources/mesh.h"
 
 void CollisionObject3D::_notification(int p_what) {
 	switch (p_what) {
@@ -70,7 +71,7 @@ void CollisionObject3D::_notification(int p_what) {
 
 			if (!disabled || (disable_mode != DISABLE_MODE_REMOVE)) {
 				Ref<World3D> world_ref = get_world_3d();
-				ERR_FAIL_COND(!world_ref.is_valid());
+				ERR_FAIL_COND(world_ref.is_null());
 				RID space = world_ref->get_space();
 				if (area) {
 					PhysicsServer3D::get_singleton()->area_set_space(rid, space);
@@ -386,7 +387,7 @@ void CollisionObject3D::_update_debug_shapes() {
 				ShapeData::ShapeBase &s = shape_bases[i];
 				if (s.shape.is_null() || shapedata.disabled) {
 					if (s.debug_shape.is_valid()) {
-						RS::get_singleton()->free(s.debug_shape);
+						RS::get_singleton()->free_rid(s.debug_shape);
 						s.debug_shape = RID();
 						--debug_shapes_count;
 					}
@@ -418,7 +419,7 @@ void CollisionObject3D::_clear_debug_shapes() {
 		for (int i = 0; i < shapedata.shapes.size(); i++) {
 			ShapeData::ShapeBase &s = shape_bases[i];
 			if (s.debug_shape.is_valid()) {
-				RS::get_singleton()->free(s.debug_shape);
+				RS::get_singleton()->free_rid(s.debug_shape);
 				s.debug_shape = RID();
 				if (s.shape.is_valid()) {
 					s.shape->disconnect_changed(callable_mp(this, &CollisionObject3D::_update_shape_data));
@@ -519,7 +520,7 @@ uint32_t CollisionObject3D::create_shape_owner(Object *p_owner) {
 	ShapeData sd;
 	uint32_t id;
 
-	if (shapes.size() == 0) {
+	if (shapes.is_empty()) {
 		id = 0;
 	} else {
 		id = shapes.back()->key() + 1;
@@ -607,9 +608,9 @@ Object *CollisionObject3D::shape_owner_get_owner(uint32_t p_owner) const {
 	return ObjectDB::get_instance(shapes[p_owner].owner_id);
 }
 
-void CollisionObject3D::shape_owner_add_shape(uint32_t p_owner, const Ref<Shape3D> &p_shape) {
+void CollisionObject3D::shape_owner_add_shape(uint32_t p_owner, RequiredParam<Shape3D> rp_shape) {
 	ERR_FAIL_COND(!shapes.has(p_owner));
-	ERR_FAIL_COND(p_shape.is_null());
+	EXTRACT_PARAM_OR_FAIL(p_shape, rp_shape);
 
 	ShapeData &sd = shapes[p_owner];
 	ShapeData::ShapeBase s;
@@ -664,7 +665,7 @@ void CollisionObject3D::shape_owner_remove_shape(uint32_t p_owner, int p_shape)
 	}
 
 	if (s.debug_shape.is_valid()) {
-		RS::get_singleton()->free(s.debug_shape);
+		RS::get_singleton()->free_rid(s.debug_shape);
 		if (s.shape.is_valid()) {
 			s.shape->disconnect_changed(callable_mp(this, &CollisionObject3D::_shape_changed));
 		}
@@ -713,6 +714,7 @@ CollisionObject3D::CollisionObject3D(RID p_rid, bool p_area) {
 	rid = p_rid;
 	area = p_area;
 	set_notify_transform(true);
+	_define_ancestry(AncestralClass::COLLISION_OBJECT_3D);
 
 	if (p_area) {
 		PhysicsServer3D::get_singleton()->area_attach_object_instance_id(rid, get_instance_id());
@@ -731,7 +733,7 @@ bool CollisionObject3D::get_capture_input_on_drag() const {
 }
 
 PackedStringArray CollisionObject3D::get_configuration_warnings() const {
-	PackedStringArray warnings = Node::get_configuration_warnings();
+	PackedStringArray warnings = Node3D::get_configuration_warnings();
 
 	if (shapes.is_empty()) {
 		warnings.push_back(RTR("This node has no shape, so it can't collide or interact with other objects.\nConsider adding a CollisionShape3D or CollisionPolygon3D as a child to define its shape."));
@@ -746,6 +748,8 @@ PackedStringArray CollisionObject3D::get_configuration_warnings() const {
 }
 
 CollisionObject3D::CollisionObject3D() {
+	_define_ancestry(AncestralClass::COLLISION_OBJECT_3D);
+
 	set_notify_transform(true);
 	//owner=
 
@@ -754,5 +758,5 @@ CollisionObject3D::CollisionObject3D() {
 
 CollisionObject3D::~CollisionObject3D() {
 	ERR_FAIL_NULL(PhysicsServer3D::get_singleton());
-	PhysicsServer3D::get_singleton()->free(rid);
+	PhysicsServer3D::get_singleton()->free_rid(rid);
 }
