@@ -260,7 +260,7 @@ VoxelGIData::VoxelGIData() {
 
 VoxelGIData::~VoxelGIData() {
 	ERR_FAIL_NULL(RenderingServer::get_singleton());
-	RS::get_singleton()->free(probe);
+	RS::get_singleton()->free_rid(probe);
 }
 
 //////////////////////
@@ -586,5 +586,5 @@ VoxelGI::VoxelGI() {
 
 VoxelGI::~VoxelGI() {
 	ERR_FAIL_NULL(RenderingServer::get_singleton());
-	RS::get_singleton()->free(voxel_gi);
+	RS::get_singleton()->free_rid(voxel_gi);
 }
