@@ -81,7 +81,7 @@ void Tween::_start_tweeners() {
 		ERR_FAIL_MSG("Tween without commands, aborting.");
 	}
 
-	for (Ref<Tweener> &tweener : tweeners.write[current_step]) {
+	for (Ref<Tweener> &tweener : tweeners[current_step]) {
 		tweener->start();
 	}
 }
@@ -95,8 +95,8 @@ void Tween::_stop_internal(bool p_reset) {
 	}
 }
 
-Ref<PropertyTweener> Tween::tween_property(const Object *p_target, const NodePath &p_property, Variant p_to, double p_duration) {
-	ERR_FAIL_NULL_V(p_target, nullptr);
+RequiredResult<PropertyTweener> Tween::tween_property(RequiredParam<const Object> rp_target, const NodePath &p_property, Variant p_to, double p_duration) {
+	EXTRACT_PARAM_OR_FAIL_V(p_target, rp_target, nullptr);
 	CHECK_VALID();
 
 	Vector<StringName> property_subnames = p_property.get_as_property_path().get_subnames();
@@ -118,7 +118,7 @@ Ref<PropertyTweener> Tween::tween_property(const Object *p_target, const NodePat
 	return tweener;
 }
 
-Ref<IntervalTweener> Tween::tween_interval(double p_time) {
+RequiredResult<IntervalTweener> Tween::tween_interval(double p_time) {
 	CHECK_VALID();
 
 	Ref<IntervalTweener> tweener;
@@ -127,7 +127,7 @@ Ref<IntervalTweener> Tween::tween_interval(double p_time) {
 	return tweener;
 }
 
-Ref<CallbackTweener> Tween::tween_callback(const Callable &p_callback) {
+RequiredResult<CallbackTweener> Tween::tween_callback(const Callable &p_callback) {
 	CHECK_VALID();
 
 	Ref<CallbackTweener> tweener;
@@ -136,7 +136,7 @@ Ref<CallbackTweener> Tween::tween_callback(const Callable &p_callback) {
 	return tweener;
 }
 
-Ref<MethodTweener> Tween::tween_method(const Callable &p_callback, const Variant p_from, Variant p_to, double p_duration) {
+RequiredResult<MethodTweener> Tween::tween_method(const Callable &p_callback, const Variant p_from, Variant p_to, double p_duration) {
 	CHECK_VALID();
 
 	if (!Animation::validate_type_match(p_from, p_to)) {
@@ -149,11 +149,11 @@ Ref<MethodTweener> Tween::tween_method(const Callable &p_callback, const Variant
 	return tweener;
 }
 
-Ref<SubtweenTweener> Tween::tween_subtween(const Ref<Tween> &p_subtween) {
+RequiredResult<SubtweenTweener> Tween::tween_subtween(RequiredParam<Tween> rp_subtween) {
 	CHECK_VALID();
 
 	// Ensure that the subtween being added is not null.
-	ERR_FAIL_COND_V(p_subtween.is_null(), nullptr);
+	EXTRACT_PARAM_OR_FAIL_V(p_subtween, rp_subtween, nullptr);
 
 	Ref<SubtweenTweener> tweener;
 	tweener.instantiate(p_subtween);
@@ -164,6 +164,7 @@ Ref<SubtweenTweener> Tween::tween_subtween(const Ref<Tween> &p_subtween) {
 	if (tweener->subtween->parent_tree != nullptr) {
 		tweener->subtween->parent_tree->remove_tween(tweener->subtween);
 	}
+	subtweens.push_back(p_subtween);
 	append(tweener);
 	return tweener;
 }
@@ -179,7 +180,7 @@ void Tween::append(Ref<Tweener> p_tweener) {
 	parallel_enabled = default_parallel;
 
 	tweeners.resize(current_step + 1);
-	tweeners.write[current_step].push_back(p_tweener);
+	tweeners[current_step].push_back(p_tweener);
 }
 
 void Tween::stop() {
@@ -200,6 +201,11 @@ void Tween::kill() {
 	running = false; // For the sake of is_running().
 	valid = false;
 	dead = true;
+
+	// Kill all subtweens of this tween.
+	for (Ref<Tween> &st : subtweens) {
+		st->kill();
+	}
 }
 
 bool Tween::is_running() {
@@ -215,15 +221,15 @@ void Tween::clear() {
 	tweeners.clear();
 }
 
-Ref<Tween> Tween::bind_node(const Node *p_node) {
-	ERR_FAIL_NULL_V(p_node, this);
+RequiredResult<Tween> Tween::bind_node(RequiredParam<const Node> rp_node) {
+	EXTRACT_PARAM_OR_FAIL_V(p_node, rp_node, this);
 
 	bound_node = p_node->get_instance_id();
 	is_bound = true;
 	return this;
 }
 
-Ref<Tween> Tween::set_process_mode(TweenProcessMode p_mode) {
+RequiredResult<Tween> Tween::set_process_mode(TweenProcessMode p_mode) {
 	process_mode = p_mode;
 	return this;
 }
@@ -232,7 +238,7 @@ Tween::TweenProcessMode Tween::get_process_mode() const {
 	return process_mode;
 }
 
-Ref<Tween> Tween::set_pause_mode(TweenPauseMode p_mode) {
+RequiredResult<Tween> Tween::set_pause_mode(TweenPauseMode p_mode) {
 	pause_mode = p_mode;
 	return this;
 }
@@ -241,7 +247,7 @@ Tween::TweenPauseMode Tween::get_pause_mode() const {
 	return pause_mode;
 }
 
-Ref<Tween> Tween::set_ignore_time_scale(bool p_ignore) {
+RequiredResult<Tween> Tween::set_ignore_time_scale(bool p_ignore) {
 	ignore_time_scale = p_ignore;
 	return this;
 }
@@ -250,13 +256,13 @@ bool Tween::is_ignoring_time_scale() const {
 	return ignore_time_scale;
 }
 
-Ref<Tween> Tween::set_parallel(bool p_parallel) {
+RequiredResult<Tween> Tween::set_parallel(bool p_parallel) {
 	default_parallel = p_parallel;
 	parallel_enabled = p_parallel;
 	return this;
 }
 
-Ref<Tween> Tween::set_loops(int p_loops) {
+RequiredResult<Tween> Tween::set_loops(int p_loops) {
 	loops = p_loops;
 	return this;
 }
@@ -269,12 +275,12 @@ int Tween::get_loops_left() const {
 	}
 }
 
-Ref<Tween> Tween::set_speed_scale(float p_speed) {
+RequiredResult<Tween> Tween::set_speed_scale(float p_speed) {
 	speed_scale = p_speed;
 	return this;
 }
 
-Ref<Tween> Tween::set_trans(TransitionType p_trans) {
+RequiredResult<Tween> Tween::set_trans(TransitionType p_trans) {
 	default_transition = p_trans;
 	return this;
 }
@@ -283,7 +289,7 @@ Tween::TransitionType Tween::get_trans() const {
 	return default_transition;
 }
 
-Ref<Tween> Tween::set_ease(EaseType p_ease) {
+RequiredResult<Tween> Tween::set_ease(EaseType p_ease) {
 	default_ease = p_ease;
 	return this;
 }
@@ -292,12 +298,12 @@ Tween::EaseType Tween::get_ease() const {
 	return default_ease;
 }
 
-Ref<Tween> Tween::parallel() {
+RequiredResult<Tween> Tween::parallel() {
 	parallel_enabled = true;
 	return this;
 }
 
-Ref<Tween> Tween::chain() {
+RequiredResult<Tween> Tween::chain() {
 	parallel_enabled = false;
 	return this;
 }
@@ -365,7 +371,7 @@ bool Tween::step(double p_delta) {
 		double step_delta = rem_delta;
 		step_active = false;
 
-		for (Ref<Tweener> &tweener : tweeners.write[current_step]) {
+		for (Ref<Tweener> &tweener : tweeners[current_step]) {
 			// Modified inside Tweener.step().
 			double temp_delta = rem_delta;
 			// Turns to true if any Tweener returns true (i.e. is still not finished).
@@ -379,7 +385,7 @@ bool Tween::step(double p_delta) {
 			emit_signal(SNAME("step_finished"), current_step);
 			current_step++;
 
-			if (current_step == tweeners.size()) {
+			if (current_step == (int)tweeners.size()) {
 				loops_done++;
 				if (loops_done == loops) {
 					running = false;
@@ -453,8 +459,8 @@ Variant Tween::interpolate_variant(const Variant &p_initial_val, const Variant &
 	return ret;
 }
 
-String Tween::to_string() {
-	String ret = Object::to_string();
+String Tween::_to_string() {
+	String ret = Object::_to_string();
 	Node *node = get_bound_node();
 	if (node) {
 		ret += vformat(" (bound to %s)", node->get_name());
@@ -548,7 +554,7 @@ double PropertyTweener::_get_custom_interpolated_value(const Variant &p_value) {
 	return result;
 }
 
-Ref<PropertyTweener> PropertyTweener::from(const Variant &p_value) {
+RequiredResult<PropertyTweener> PropertyTweener::from(const Variant &p_value) {
 	Ref<Tween> tween = _get_tween();
 	ERR_FAIL_COND_V(tween.is_null(), nullptr);
 
@@ -562,32 +568,32 @@ Ref<PropertyTweener> PropertyTweener::from(const Variant &p_value) {
 	return this;
 }
 
-Ref<PropertyTweener> PropertyTweener::from_current() {
+RequiredResult<PropertyTweener> PropertyTweener::from_current() {
 	do_continue = false;
 	return this;
 }
 
-Ref<PropertyTweener> PropertyTweener::as_relative() {
+RequiredResult<PropertyTweener> PropertyTweener::as_relative() {
 	relative = true;
 	return this;
 }
 
-Ref<PropertyTweener> PropertyTweener::set_trans(Tween::TransitionType p_trans) {
+RequiredResult<PropertyTweener> PropertyTweener::set_trans(Tween::TransitionType p_trans) {
 	trans_type = p_trans;
 	return this;
 }
 
-Ref<PropertyTweener> PropertyTweener::set_ease(Tween::EaseType p_ease) {
+RequiredResult<PropertyTweener> PropertyTweener::set_ease(Tween::EaseType p_ease) {
 	ease_type = p_ease;
 	return this;
 }
 
-Ref<PropertyTweener> PropertyTweener::set_custom_interpolator(const Callable &p_method) {
+RequiredResult<PropertyTweener> PropertyTweener::set_custom_interpolator(const Callable &p_method) {
 	custom_method = p_method;
 	return this;
 }
 
-Ref<PropertyTweener> PropertyTweener::set_delay(double p_delay) {
+RequiredResult<PropertyTweener> PropertyTweener::set_delay(double p_delay) {
 	delay = p_delay;
 	return this;
 }
@@ -725,7 +731,7 @@ IntervalTweener::IntervalTweener() {
 	ERR_FAIL_MSG("IntervalTweener can't be created directly. Use the tween_interval() method in Tween.");
 }
 
-Ref<CallbackTweener> CallbackTweener::set_delay(double p_delay) {
+RequiredResult<CallbackTweener> CallbackTweener::set_delay(double p_delay) {
 	delay = p_delay;
 	return this;
 }
@@ -775,17 +781,17 @@ CallbackTweener::CallbackTweener() {
 	ERR_FAIL_MSG("CallbackTweener can't be created directly. Use the tween_callback() method in Tween.");
 }
 
-Ref<MethodTweener> MethodTweener::set_delay(double p_delay) {
+RequiredResult<MethodTweener> MethodTweener::set_delay(double p_delay) {
 	delay = p_delay;
 	return this;
 }
 
-Ref<MethodTweener> MethodTweener::set_trans(Tween::TransitionType p_trans) {
+RequiredResult<MethodTweener> MethodTweener::set_trans(Tween::TransitionType p_trans) {
 	trans_type = p_trans;
 	return this;
 }
 
-Ref<MethodTweener> MethodTweener::set_ease(Tween::EaseType p_ease) {
+RequiredResult<MethodTweener> MethodTweener::set_ease(Tween::EaseType p_ease) {
 	ease_type = p_ease;
 	return this;
 }
@@ -906,7 +912,7 @@ bool SubtweenTweener::step(double &r_delta) {
 	return true;
 }
 
-Ref<SubtweenTweener> SubtweenTweener::set_delay(double p_delay) {
+RequiredResult<SubtweenTweener> SubtweenTweener::set_delay(double p_delay) {
 	delay = p_delay;
 	return this;
 }
