@@ -31,7 +31,7 @@
 #pragma once
 
 #include "scene/main/node.h"
-#include "servers/audio_server.h"
+#include "servers/audio/audio_server.h"
 
 struct AudioFrame;
 class AudioStream;
