@@ -213,6 +213,14 @@ bool GraphElement::is_selectable() {
 	return selectable;
 }
 
+void GraphElement::set_scaling_menus(bool p_scaling_menus) {
+	scaling_menus = p_scaling_menus;
+}
+
+bool GraphElement::is_scaling_menus() const {
+	return scaling_menus;
+}
+
 void GraphElement::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("set_resizable", "resizable"), &GraphElement::set_resizable);
 	ClassDB::bind_method(D_METHOD("is_resizable"), &GraphElement::is_resizable);
@@ -226,6 +234,9 @@ void GraphElement::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("set_selected", "selected"), &GraphElement::set_selected);
 	ClassDB::bind_method(D_METHOD("is_selected"), &GraphElement::is_selected);
 
+	ClassDB::bind_method(D_METHOD("set_scaling_menus", "scaling_menus"), &GraphElement::set_scaling_menus);
+	ClassDB::bind_method(D_METHOD("is_scaling_menus"), &GraphElement::is_scaling_menus);
+
 	ClassDB::bind_method(D_METHOD("set_position_offset", "offset"), &GraphElement::set_position_offset);
 	ClassDB::bind_method(D_METHOD("get_position_offset"), &GraphElement::get_position_offset);
 
@@ -234,6 +245,7 @@ void GraphElement::_bind_methods() {
 	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "draggable"), "set_draggable", "is_draggable");
 	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "selectable"), "set_selectable", "is_selectable");
 	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "selected"), "set_selected", "is_selected");
+	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "scaling_menus"), "set_scaling_menus", "is_scaling_menus");
 
 	ADD_SIGNAL(MethodInfo("node_selected"));
 	ADD_SIGNAL(MethodInfo("node_deselected"));
