@@ -37,6 +37,10 @@
 #include "scene/resources/image_texture.h"
 #include "scene/resources/text_paragraph.h"
 
+#ifdef TOOLS_ENABLED
+#include "editor/themes/editor_scale.h"
+#endif
+
 class CharFXTransform;
 class RichTextEffect;
 
@@ -153,12 +157,16 @@ protected:
 private:
 	struct Item;
 
-	struct Line {
-		Item *from = nullptr;
+	struct Line { // Line is a paragraph.
+		Item *from = nullptr; // `from` is main if this Line is the first Line in the doc, otherwise `from` is the previous Item in the doc of any type.
 
 		Ref<TextLine> text_prefix;
+		Color prefix_color = Color(0, 0, 0, 0);
+		int prefix_outline_size = -1;
+		Color prefix_outline_color = Color(0, 0, 0, 0);
 		float prefix_width = 0;
 		Ref<TextParagraph> text_buf;
+		Ref<TextParagraph> text_buf_disp;
 
 		RID accessibility_line_element;
 		RID accessibility_text_element;
@@ -185,24 +193,24 @@ private:
 		}
 
 		_FORCE_INLINE_ float get_height(float p_line_separation, float p_paragraph_separation) const {
-			return offset.y + text_buf->get_size().y + (text_buf->get_line_count() - 1) * p_line_separation + p_paragraph_separation;
+			return offset.y + text_buf->get_size().y + text_buf->get_line_count() * p_line_separation + p_paragraph_separation;
 		}
 	};
 
 	struct Item {
 		int index = 0;
 		int char_ofs = 0;
-		Item *parent = nullptr;
+		Item *parent = nullptr; // "parent" means "enclosing item tag", if any. It is an interval predecessor, not a hierarchical parent.
 		ItemType type = ITEM_FRAME;
 		List<Item *> subitems;
 		List<Item *>::Element *E = nullptr;
 		ObjectID owner;
-		int line = 0;
+		int line = 0; // `line` is the index number of the paragraph (Line) this item is inside of (zero if the first paragraph).
 		RID rid;
 
 		RID accessibility_item_element;
 
-		void _clear_children() {
+		void _clear_children() { // Only ever called on main or a paragraph (Line).
 			RichTextLabel *owner_rtl = ObjectDB::get_instance<RichTextLabel>(owner);
 			while (subitems.size()) {
 				Item *subitem = subitems.front()->get();
@@ -503,6 +511,29 @@ private:
 	struct ItemContext : public Item {
 		ItemContext() { type = ITEM_CONTEXT; }
 	};
+	const Array formatting_items = {
+		// all ITEM types affecting glyph appearance.
+		ITEM_FONT,
+		ITEM_FONT_SIZE,
+		ITEM_FONT_FEATURES,
+		ITEM_COLOR,
+		ITEM_OUTLINE_SIZE,
+		ITEM_OUTLINE_COLOR,
+		ITEM_UNDERLINE,
+		ITEM_STRIKETHROUGH,
+		ITEM_FADE,
+		ITEM_SHAKE,
+		ITEM_WAVE,
+		ITEM_TORNADO,
+		ITEM_RAINBOW,
+		ITEM_BGCOLOR,
+		ITEM_FGCOLOR,
+		ITEM_META,
+		ITEM_HINT,
+		ITEM_CUSTOMFX,
+		ITEM_LANGUAGE,
+		ITEM_PULSE,
+	};
 
 	ItemFrame *main = nullptr;
 	Item *current = nullptr;
@@ -561,6 +592,8 @@ private:
 	void _invalidate_accessibility();
 	void _invalidate_current_line(ItemFrame *p_frame);
 
+	void _prepare_scroll_anchor();
+
 	void _thread_function(void *p_userdata);
 	void _thread_end();
 	void _stop_thread();
@@ -680,6 +713,17 @@ private:
 	void _scroll_changed(double);
 	int _find_first_line(int p_from, int p_to, int p_vofs) const;
 
+#ifdef TOOLS_ENABLED
+	const real_t auto_scroll_speed = EDSCALE * 2.0f;
+#else
+	const real_t auto_scroll_speed = 2.0f;
+#endif
+	Vector2 last_clamped_mouse_pos;
+	Timer *click_select_held = nullptr;
+	Vector2 local_mouse_pos;
+	bool is_selecting_text = false;
+	void _update_selection();
+
 	_FORCE_INLINE_ float _calculate_line_vertical_offset(const Line &line) const;
 
 	virtual void gui_input(const Ref<InputEvent> &p_event) override;
@@ -696,6 +740,7 @@ private:
 	Size2 _get_image_size(const Ref<Texture2D> &p_image, int p_width = 0, int p_height = 0, const Rect2 &p_region = Rect2());
 
 	String _get_prefix(Item *p_item, const Vector<int> &p_list_index, const Vector<ItemList *> &p_list_items);
+	void _add_list_prefixes(ItemFrame *p_frame, int p_line, Line &r_l);
 
 	static int _find_unquoted(const String &p_src, char32_t p_chr, int p_from);
 	static Vector<String> _split_unquoted(const String &p_src, char32_t p_splitter);
@@ -960,7 +1005,7 @@ public:
 	void set_structured_text_bidi_override(TextServer::StructuredTextParser p_parser);
 	TextServer::StructuredTextParser get_structured_text_bidi_override() const;
 
-	void set_structured_text_bidi_override_options(Array p_args);
+	void set_structured_text_bidi_override_options(const Array &p_args);
 	Array get_structured_text_bidi_override_options() const;
 
 	void set_visible_characters(int p_visible);
@@ -976,7 +1021,7 @@ public:
 	TextServer::VisibleCharactersBehavior get_visible_characters_behavior() const;
 	void set_visible_characters_behavior(TextServer::VisibleCharactersBehavior p_behavior);
 
-	void set_effects(Array p_effects);
+	void set_effects(const Array &p_effects);
 	Array get_effects();
 
 	void install_effect(const Variant effect);
