@@ -52,7 +52,7 @@ class AudioStreamPlaybackWAV : public AudioStreamPlaybackResampled {
 
 	struct QOA_State {
 		qoa_desc desc = {};
-		uint32_t data_ofs = 0;
+		uint64_t data_ofs = 0;
 		uint32_t frame_len = 0;
 		TightLocalVector<int16_t> dec;
 		uint32_t dec_len = 0;
@@ -118,10 +118,10 @@ private:
 	Format format = FORMAT_8_BITS;
 	LoopMode loop_mode = LOOP_DISABLED;
 	bool stereo = false;
-	int loop_begin = 0;
-	int loop_end = 0;
-	int mix_rate = 44100;
-	TightLocalVector<uint8_t> data;
+	int64_t loop_begin = 0;
+	int64_t loop_end = 0;
+	uint32_t mix_rate = 44100;
+	TightLocalVector<uint8_t, uint64_t> data;
 	uint32_t data_bytes = 0;
 
 	Dictionary tags;
