@@ -2127,6 +2127,14 @@ PackedVector3Array Curve3D::get_baked_up_vectors() const {
 	return baked_up_vector_cache;
 }
 
+Vector<real_t> Curve3D::get_baked_dist_cache() const {
+	if (baked_cache_dirty) {
+		_bake();
+	}
+
+	return baked_dist_cache;
+}
+
 Vector3 Curve3D::get_closest_point(const Vector3 &p_to_point) const {
 	// Brute force method.
 
