@@ -31,7 +31,7 @@
 #include "navigation_mesh.h"
 
 #ifdef DEBUG_ENABLED
-#include "servers/navigation_server_3d.h"
+#include "servers/navigation_3d/navigation_server_3d.h"
 #endif // DEBUG_ENABLED
 
 void NavigationMesh::create_from_mesh(const Ref<Mesh> &p_mesh) {
