@@ -105,5 +105,5 @@ Sky::Sky() {
 
 Sky::~Sky() {
 	ERR_FAIL_NULL(RenderingServer::get_singleton());
-	RS::get_singleton()->free(sky);
+	RS::get_singleton()->free_rid(sky);
 }
