@@ -32,7 +32,7 @@
 
 #include "core/templates/local_vector.h"
 #include "scene/resources/font.h"
-#include "servers/text_server.h"
+#include "servers/text/text_server.h"
 
 /*************************************************************************/
 
@@ -85,6 +85,8 @@ public:
 
 	void clear();
 
+	Ref<TextParagraph> duplicate() const;
+
 	void set_direction(TextServer::Direction p_direction);
 	TextServer::Direction get_direction() const;
 	TextServer::Direction get_inferred_direction() const;
@@ -109,6 +111,7 @@ public:
 	bool add_string(const String &p_text, const Ref<Font> &p_font, int p_font_size, const String &p_language = "", const Variant &p_meta = Variant());
 	bool add_object(Variant p_key, const Size2 &p_size, InlineAlignment p_inline_align = INLINE_ALIGNMENT_CENTER, int p_length = 1, float p_baseline = 0.0);
 	bool resize_object(Variant p_key, const Size2 &p_size, InlineAlignment p_inline_align = INLINE_ALIGNMENT_CENTER, float p_baseline = 0.0);
+	bool has_object(Variant p_key) const;
 
 	void set_alignment(HorizontalAlignment p_alignment);
 	HorizontalAlignment get_alignment() const;
