@@ -29,7 +29,7 @@
 /**************************************************************************/
 
 #include "audio_effect_phaser.h"
-#include "servers/audio_server.h"
+#include "servers/audio/audio_server.h"
 
 void AudioEffectPhaserInstance::process(const AudioFrame *p_src_frames, AudioFrame *p_dst_frames, int p_frame_count) {
 	float sampling_rate = AudioServer::get_singleton()->get_mix_rate();
