@@ -28,8 +28,7 @@
 /* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
 /**************************************************************************/
 
-#ifndef NATIVE_MENU_H
-#define NATIVE_MENU_H
+#pragma once
 
 #include "core/input/input.h"
 #include "core/variant/callable.h"
@@ -72,6 +71,9 @@ public:
 	virtual RID get_system_menu(SystemMenus p_menu_id) const;
 	virtual String get_system_menu_name(SystemMenus p_menu_id) const;
 
+	virtual String get_system_menu_text(SystemMenus p_menu_id) const;
+	virtual void set_system_menu_text(SystemMenus p_menu_id, const String &p_name);
+
 	virtual RID create_menu();
 	virtual bool has_menu(const RID &p_rid) const;
 	virtual void free_menu(const RID &p_rid);
@@ -156,5 +158,3 @@ public:
 
 VARIANT_ENUM_CAST(NativeMenu::Feature);
 VARIANT_ENUM_CAST(NativeMenu::SystemMenus);
-
-#endif // NATIVE_MENU_H
