@@ -31,7 +31,7 @@
 #pragma once
 
 #include "servers/rendering/rendering_method.h"
-#include "servers/rendering_server.h"
+#include "servers/rendering/rendering_server.h"
 
 class RendererCanvasRender {
 public:
