@@ -31,9 +31,33 @@
 #pragma once
 
 #include "core/templates/rid_owner.h"
-#include "servers/rendering_server.h"
+#include "servers/rendering/rendering_server.h"
 
 class RendererEnvironmentStorage {
+public:
+	union TonemapParameters {
+		// Shader vec4:
+		float tonemapper_params[4];
+
+		// Reinhard:
+		struct {
+			float white_squared;
+		};
+
+		// Filmic and ACES:
+		struct {
+			float white_tonemapped;
+		};
+
+		// AgX:
+		struct {
+			float awp_contrast;
+			float awp_toe_a;
+			float awp_slope;
+			float awp_w;
+		};
+	};
+
 private:
 	static RendererEnvironmentStorage *singleton;
 
@@ -62,6 +86,7 @@ private:
 		RS::EnvironmentToneMapper tone_mapper;
 		float exposure = 1.0;
 		float white = 1.0;
+		float tonemap_agx_contrast = 1.25; // Default to approximately Blender's AgX contrast
 
 		// Fog
 		bool fog_enabled = false;
@@ -98,11 +123,11 @@ private:
 		// Glow
 		bool glow_enabled = false;
 		Vector<float> glow_levels;
-		float glow_intensity = 0.8;
+		float glow_intensity = 0.3;
 		float glow_strength = 1.0;
 		float glow_bloom = 0.0;
 		float glow_mix = 0.01;
-		RS::EnvironmentGlowBlendMode glow_blend_mode = RS::ENV_GLOW_BLEND_MODE_SOFTLIGHT;
+		RS::EnvironmentGlowBlendMode glow_blend_mode = RS::ENV_GLOW_BLEND_MODE_SCREEN;
 		float glow_hdr_bleed_threshold = 1.0;
 		float glow_hdr_luminance_cap = 12.0;
 		float glow_hdr_bleed_scale = 2.0;
@@ -114,7 +139,7 @@ private:
 		int ssr_max_steps = 64;
 		float ssr_fade_in = 0.15;
 		float ssr_fade_out = 2.0;
-		float ssr_depth_tolerance = 0.2;
+		float ssr_depth_tolerance = 0.5;
 
 		// SSAO
 		bool ssao_enabled = false;
@@ -202,7 +227,10 @@ public:
 	void environment_set_tonemap(RID p_env, RS::EnvironmentToneMapper p_tone_mapper, float p_exposure, float p_white);
 	RS::EnvironmentToneMapper environment_get_tone_mapper(RID p_env) const;
 	float environment_get_exposure(RID p_env) const;
-	float environment_get_white(RID p_env) const;
+	float environment_get_white(RID p_env, bool p_limit_agx_white) const;
+	void environment_set_tonemap_agx_contrast(RID p_env, float p_agx_contrast);
+	float environment_get_tonemap_agx_contrast(RID p_env) const;
+	TonemapParameters environment_get_tonemap_parameters(RID p_env, bool p_limit_agx_white) const;
 
 	// Fog
 	void environment_set_fog(RID p_env, bool p_enable, const Color &p_light_color, float p_light_energy, float p_sun_scatter, float p_density, float p_height, float p_height_density, float p_aerial_perspective, float p_sky_affect, RS::EnvironmentFogMode p_mode);
