@@ -43,6 +43,7 @@ void TextServerExtension::_bind_methods() {
 	GDVIRTUAL_BIND(_get_support_data_info);
 	GDVIRTUAL_BIND(_save_support_data, "filename");
 	GDVIRTUAL_BIND(_get_support_data);
+	GDVIRTUAL_BIND(_is_locale_using_support_data, "locale");
 
 	GDVIRTUAL_BIND(_is_locale_right_to_left, "locale");
 
@@ -252,6 +253,7 @@ void TextServerExtension::_bind_methods() {
 	GDVIRTUAL_BIND(_create_shaped_text, "direction", "orientation");
 
 	GDVIRTUAL_BIND(_shaped_text_clear, "shaped");
+	GDVIRTUAL_BIND(_shaped_text_duplicate, "shaped");
 
 	GDVIRTUAL_BIND(_shaped_text_set_direction, "shaped", "direction");
 	GDVIRTUAL_BIND(_shaped_text_get_direction, "shaped");
@@ -280,6 +282,7 @@ void TextServerExtension::_bind_methods() {
 	GDVIRTUAL_BIND(_shaped_text_add_string, "shaped", "text", "fonts", "size", "opentype_features", "language", "meta");
 	GDVIRTUAL_BIND(_shaped_text_add_object, "shaped", "key", "size", "inline_align", "length", "baseline");
 	GDVIRTUAL_BIND(_shaped_text_resize_object, "shaped", "key", "size", "inline_align", "baseline");
+	GDVIRTUAL_BIND(_shaped_text_has_object, "shaped", "key");
 	GDVIRTUAL_BIND(_shaped_get_text, "shaped");
 
 	GDVIRTUAL_BIND(_shaped_get_span_count, "shaped");
@@ -363,9 +366,11 @@ void TextServerExtension::_bind_methods() {
 	GDVIRTUAL_BIND(_shaped_text_prev_character_pos, "shaped", "pos");
 	GDVIRTUAL_BIND(_shaped_text_closest_character_pos, "shaped", "pos");
 
+#ifndef DISABLE_DEPRECATED
 	GDVIRTUAL_BIND(_format_number, "number", "language");
 	GDVIRTUAL_BIND(_parse_number, "number", "language");
 	GDVIRTUAL_BIND(_percent_sign, "language");
+#endif
 
 	GDVIRTUAL_BIND(_strip_diacritics, "string");
 	GDVIRTUAL_BIND(_is_valid_identifier, "string");
@@ -444,6 +449,12 @@ PackedByteArray TextServerExtension::get_support_data() const {
 	return ret;
 }
 
+bool TextServerExtension::is_locale_using_support_data(const String &p_locale) const {
+	bool ret = false;
+	GDVIRTUAL_CALL(_is_locale_using_support_data, p_locale, ret);
+	return ret;
+}
+
 bool TextServerExtension::is_locale_right_to_left(const String &p_locale) const {
 	bool ret = false;
 	GDVIRTUAL_CALL(_is_locale_right_to_left, p_locale, ret);
@@ -1148,6 +1159,12 @@ void TextServerExtension::shaped_text_clear(const RID &p_shaped) {
 	GDVIRTUAL_CALL(_shaped_text_clear, p_shaped);
 }
 
+RID TextServerExtension::shaped_text_duplicate(const RID &p_shaped) {
+	RID ret;
+	GDVIRTUAL_CALL(_shaped_text_duplicate, p_shaped, ret);
+	return ret;
+}
+
 void TextServerExtension::shaped_text_set_direction(const RID &p_shaped, TextServer::Direction p_direction) {
 	GDVIRTUAL_CALL(_shaped_text_set_direction, p_shaped, p_direction);
 }
@@ -1246,6 +1263,12 @@ bool TextServerExtension::shaped_text_resize_object(const RID &p_shaped, const V
 	return ret;
 }
 
+bool TextServerExtension::shaped_text_has_object(const RID &p_shaped, const Variant &p_key) const {
+	bool ret = false;
+	GDVIRTUAL_CALL(_shaped_text_has_object, p_shaped, p_key, ret);
+	return ret;
+}
+
 String TextServerExtension::shaped_get_text(const RID &p_shaped) const {
 	String ret;
 	GDVIRTUAL_CALL(_shaped_get_text, p_shaped, ret);
@@ -1638,12 +1661,13 @@ int64_t TextServerExtension::shaped_text_closest_character_pos(const RID &p_shap
 	return TextServer::shaped_text_closest_character_pos(p_shaped, p_pos);
 }
 
+#ifndef DISABLE_DEPRECATED
 String TextServerExtension::format_number(const String &p_string, const String &p_language) const {
 	String ret;
 	if (GDVIRTUAL_CALL(_format_number, p_string, p_language, ret)) {
 		return ret;
 	}
-	return p_string;
+	return TextServer::format_number(p_string, p_language);
 }
 
 String TextServerExtension::parse_number(const String &p_string, const String &p_language) const {
@@ -1651,14 +1675,17 @@ String TextServerExtension::parse_number(const String &p_string, const String &p
 	if (GDVIRTUAL_CALL(_parse_number, p_string, p_language, ret)) {
 		return ret;
 	}
-	return p_string;
+	return TextServer::parse_number(p_string, p_language);
 }
 
 String TextServerExtension::percent_sign(const String &p_language) const {
 	String ret = "%";
-	GDVIRTUAL_CALL(_percent_sign, p_language, ret);
-	return ret;
+	if (GDVIRTUAL_CALL(_percent_sign, p_language, ret)) {
+		return ret;
+	}
+	return TextServer::percent_sign(p_language);
 }
+#endif // DISABLE_DEPRECATED
 
 bool TextServerExtension::is_valid_identifier(const String &p_string) const {
 	bool ret;
