@@ -122,6 +122,20 @@ TEST_CASE("[Image] Saving and loading") {
 			"The BMP image should load successfully.");
 #endif // MODULE_BMP_ENABLED
 
+#ifdef MODULE_EXR_ENABLED
+	// Load EXR
+	Ref<Image> image_exr;
+	image_exr.instantiate();
+	Ref<FileAccess> f_exr = FileAccess::open(TestUtils::get_data_path("images/icon.exr"), FileAccess::READ, &err);
+	REQUIRE(f_exr.is_valid());
+	PackedByteArray data_exr;
+	data_exr.resize(f_exr->get_length() + 1);
+	f_exr->get_buffer(data_exr.ptrw(), f_exr->get_length());
+	CHECK_MESSAGE(
+			image_exr->load_exr_from_buffer(data_exr) == OK,
+			"The EXR image should load successfully.");
+#endif // MODULE_EXR_ENABLED
+
 #ifdef MODULE_JPG_ENABLED
 	// Load JPG
 	Ref<Image> image_jpg = memnew(Image());
@@ -267,11 +281,15 @@ TEST_CASE("[Image] Modifying pixels of an image") {
 
 	// Fill image with color
 	image2->fill(Color(0.5, 0.5, 0.5, 0.5));
-	for (int y = 0; y < image2->get_height(); y++) {
-		for (int x = 0; x < image2->get_width(); x++) {
-			CHECK_MESSAGE(
-					image2->get_pixel(x, y).r > 0.49,
-					"fill() should colorize all pixels of the image.");
+	for (int m = 0; m < image2->get_mipmap_count(); m++) {
+		Ref<Image> mip_image = image2->get_image_from_mipmap(m);
+
+		for (int y = 0; y < mip_image->get_height(); y++) {
+			for (int x = 0; x < mip_image->get_width(); x++) {
+				CHECK_MESSAGE(
+						mip_image->get_pixel(x, y).r > 0.49,
+						"fill() should colorize all pixels of the image.");
+			}
 		}
 	}
 
