@@ -545,57 +545,4 @@ TEST_CASE("[List] Swap adjacent back and front (reverse order of elements)") {
 		}
 	}
 }
-
-static void swap_random(List<int> &p_list, List<int>::Element *r_elements[], size_t p_size, size_t p_iterations) {
-	Math::seed(0);
-
-	for (size_t test_i = 0; test_i < p_iterations; ++test_i) {
-		// A and B elements have corresponding indices as values.
-		const int a_idx = static_cast<int>(Math::rand() % p_size);
-		const int b_idx = static_cast<int>(Math::rand() % p_size);
-		List<int>::Element *a = p_list.find(a_idx); // via find.
-		List<int>::Element *b = r_elements[b_idx]; // via pointer.
-
-		int va = a->get();
-		int vb = b->get();
-
-		p_list.swap(a, b);
-
-		CHECK(va == a->get());
-		CHECK(vb == b->get());
-
-		size_t element_count = 0;
-
-		// Fully traversable after swap?
-		List<int>::Element *it = p_list.front();
-		while (it) {
-			element_count += 1;
-			List<int>::Element *prev_it = it;
-			it = it->next();
-			if (it == prev_it) {
-				FAIL_CHECK("Infinite loop detected.");
-				break;
-			}
-		}
-		// We should not lose anything in the process.
-		if (element_count != p_size) {
-			FAIL_CHECK("Element count mismatch.");
-			break;
-		}
-	}
-}
-
-TEST_CASE("[Stress][List] Swap random 100 elements, 500 iterations.") {
-	List<int> list;
-	List<int>::Element *n[100];
-	populate_integers(list, n, 100);
-	swap_random(list, n, 100, 500);
-}
-
-TEST_CASE("[Stress][List] Swap random 10 elements, 1000 iterations.") {
-	List<int> list;
-	List<int>::Element *n[10];
-	populate_integers(list, n, 10);
-	swap_random(list, n, 10, 1000);
-}
 } // namespace TestList
