@@ -33,7 +33,7 @@
 #ifdef TOOLS_ENABLED
 
 #include "editor/themes/builtin_fonts.gen.h"
-#include "servers/text_server.h"
+#include "servers/text/text_server.h"
 #include "tests/test_macros.h"
 
 namespace TestTextServer {
@@ -50,7 +50,7 @@ TEST_SUITE("[TextServer]") {
 				}
 
 				RID font = ts->create_font();
-				ts->font_set_data_ptr(font, _font_NotoSans_Regular, _font_NotoSans_Regular_size);
+				ts->font_set_data_ptr(font, _font_Inter_Regular, _font_Inter_Regular_size);
 				CHECK_FALSE_MESSAGE(font == RID(), "Loading font failed.");
 				ts->free_rid(font);
 			}
@@ -66,7 +66,7 @@ TEST_SUITE("[TextServer]") {
 				}
 
 				RID font1 = ts->create_font();
-				ts->font_set_data_ptr(font1, _font_NotoSans_Regular, _font_NotoSans_Regular_size);
+				ts->font_set_data_ptr(font1, _font_Inter_Regular, _font_Inter_Regular_size);
 				ts->font_set_allow_system_fallback(font1, false);
 				RID font2 = ts->create_font();
 				ts->font_set_data_ptr(font2, _font_NotoSansThai_Regular, _font_NotoSansThai_Regular_size);
@@ -118,7 +118,7 @@ TEST_SUITE("[TextServer]") {
 				}
 
 				RID font1 = ts->create_font();
-				ts->font_set_data_ptr(font1, _font_NotoSans_Regular, _font_NotoSans_Regular_size);
+				ts->font_set_data_ptr(font1, _font_Inter_Regular, _font_Inter_Regular_size);
 				RID font2 = ts->create_font();
 				ts->font_set_data_ptr(font2, _font_Vazirmatn_Regular, _font_Vazirmatn_Regular_size);
 
@@ -167,7 +167,7 @@ TEST_SUITE("[TextServer]") {
 				}
 
 				RID font1 = ts->create_font();
-				ts->font_set_data_ptr(font1, _font_NotoSans_Regular, _font_NotoSans_Regular_size);
+				ts->font_set_data_ptr(font1, _font_Inter_Regular, _font_Inter_Regular_size);
 				ts->font_set_allow_system_fallback(font1, false);
 				RID font2 = ts->create_font();
 				ts->font_set_data_ptr(font2, _font_NotoSansThai_Regular, _font_NotoSansThai_Regular_size);
@@ -497,7 +497,7 @@ TEST_SUITE("[TextServer]") {
 						{ U"test\r test", { 0, 5, 5, 10 } },
 						{ U"test\r test \r test", { 0, 5, 5, 12, 12, 17 } },
 					};
-					for (size_t j = 0; j < std::size(cases); j++) {
+					for (size_t j = 0; j < std_size(cases); j++) {
 						RID ctx = ts->create_shaped_text();
 						CHECK_FALSE_MESSAGE(ctx == RID(), "Creating text buffer failed.");
 						bool ok = ts->shaped_text_add_string(ctx, cases[j].text, font, 16);
@@ -565,7 +565,7 @@ TEST_SUITE("[TextServer]") {
 				//                   5^  10^
 
 				RID font1 = ts->create_font();
-				ts->font_set_data_ptr(font1, _font_NotoSans_Regular, _font_NotoSans_Regular_size);
+				ts->font_set_data_ptr(font1, _font_Inter_Regular, _font_Inter_Regular_size);
 				RID font2 = ts->create_font();
 				ts->font_set_data_ptr(font2, _font_NotoSansThai_Regular, _font_NotoSansThai_Regular_size);
 
@@ -603,6 +603,59 @@ TEST_SUITE("[TextServer]") {
 
 				ts->free_rid(ctx);
 
+				String test_2 = U"Word Wrap";
+				//                   5^
+
+				ctx = ts->create_shaped_text();
+				CHECK_FALSE_MESSAGE(ctx == RID(), "Creating text buffer failed.");
+				ok = ts->shaped_text_add_string(ctx, test_2, font, 16);
+				CHECK_FALSE_MESSAGE(!ok, "Adding text to the buffer failed.");
+
+				brks = ts->shaped_text_get_line_breaks(ctx, 43);
+				CHECK_FALSE_MESSAGE(brks.size() != 4, "Invalid line breaks number.");
+				if (brks.size() == 4) {
+					CHECK_FALSE_MESSAGE(brks[0] != 0, "Invalid line break position.");
+					CHECK_FALSE_MESSAGE(brks[1] != 5, "Invalid line break position.");
+
+					CHECK_FALSE_MESSAGE(brks[2] != 5, "Invalid line break position.");
+					CHECK_FALSE_MESSAGE(brks[3] != 9, "Invalid line break position.");
+				}
+
+				brks = ts->shaped_text_get_line_breaks(ctx, 43.0, 0, TextServer::BREAK_WORD_BOUND | TextServer::BREAK_MANDATORY | TextServer::BREAK_TRIM_START_EDGE_SPACES | TextServer::BREAK_TRIM_END_EDGE_SPACES);
+				CHECK_FALSE_MESSAGE(brks.size() != 4, "Invalid line breaks number.");
+				if (brks.size() == 4) {
+					CHECK_FALSE_MESSAGE(brks[0] != 0, "Invalid line break position.");
+					CHECK_FALSE_MESSAGE(brks[1] != 4, "Invalid line break position.");
+
+					CHECK_FALSE_MESSAGE(brks[2] != 5, "Invalid line break position.");
+					CHECK_FALSE_MESSAGE(brks[3] != 9, "Invalid line break position.");
+				}
+
+				brks = ts->shaped_text_get_line_breaks(ctx, 43.0, 0, TextServer::BREAK_WORD_BOUND | TextServer::BREAK_ADAPTIVE | TextServer::BREAK_MANDATORY | TextServer::BREAK_TRIM_START_EDGE_SPACES | TextServer::BREAK_TRIM_END_EDGE_SPACES);
+				CHECK_FALSE_MESSAGE(brks.size() != 4, "Invalid line breaks number.");
+				if (brks.size() == 4) {
+					CHECK_FALSE_MESSAGE(brks[0] != 0, "Invalid line break position.");
+					CHECK_FALSE_MESSAGE(brks[1] != 4, "Invalid line break position.");
+
+					CHECK_FALSE_MESSAGE(brks[2] != 5, "Invalid line break position.");
+					CHECK_FALSE_MESSAGE(brks[3] != 9, "Invalid line break position.");
+				}
+
+				brks = ts->shaped_text_get_line_breaks(ctx, 43.0, 0, TextServer::BREAK_WORD_BOUND | TextServer::BREAK_ADAPTIVE | TextServer::BREAK_MANDATORY);
+				CHECK_FALSE_MESSAGE(brks.size() != 6, "Invalid line breaks number.");
+				if (brks.size() == 6) {
+					CHECK_FALSE_MESSAGE(brks[0] != 0, "Invalid line break position.");
+					CHECK_FALSE_MESSAGE(brks[1] != 4, "Invalid line break position.");
+
+					CHECK_FALSE_MESSAGE(brks[2] != 4, "Invalid line break position.");
+					CHECK_FALSE_MESSAGE(brks[3] != 5, "Invalid line break position.");
+
+					CHECK_FALSE_MESSAGE(brks[4] != 5, "Invalid line break position.");
+					CHECK_FALSE_MESSAGE(brks[5] != 9, "Invalid line break position.");
+				}
+
+				ts->free_rid(ctx);
+
 				for (int j = 0; j < font.size(); j++) {
 					ts->free_rid(font[j]);
 				}
@@ -620,7 +673,7 @@ TEST_SUITE("[TextServer]") {
 				}
 
 				RID font1 = ts->create_font();
-				ts->font_set_data_ptr(font1, _font_NotoSans_Regular, _font_NotoSans_Regular_size);
+				ts->font_set_data_ptr(font1, _font_Inter_Regular, _font_Inter_Regular_size);
 				RID font2 = ts->create_font();
 				ts->font_set_data_ptr(font2, _font_Vazirmatn_Regular, _font_Vazirmatn_Regular_size);
 
@@ -931,7 +984,7 @@ TEST_SUITE("[TextServer]") {
 				}
 
 				RID font1 = ts->create_font();
-				ts->font_set_data_ptr(font1, _font_NotoSans_Regular, _font_NotoSans_Regular_size);
+				ts->font_set_data_ptr(font1, _font_Inter_Regular, _font_Inter_Regular_size);
 
 				Array font = { font1 };
 				RID ctx = ts->create_shaped_text();
