@@ -16,19 +16,22 @@ namespace embree
   State::ErrorHandler::~ErrorHandler()
   {
     Lock<MutexSys> lock(errors_mutex);
-    for (size_t i=0; i<thread_errors.size(); i++)
+    for (size_t i=0; i<thread_errors.size(); i++) {
       delete thread_errors[i];
+    }
     destroyTls(thread_error);
     thread_errors.clear();
   }
 
-  RTCError* State::ErrorHandler::error() 
+  RTCErrorMessage* State::ErrorHandler::error()
   {
-    RTCError* stored_error = (RTCError*) getTls(thread_error);
-    if (stored_error) return stored_error;
+    RTCErrorMessage* stored_error = (RTCErrorMessage*) getTls(thread_error);
+    if (stored_error) {
+      return stored_error;
+    }
 
     Lock<MutexSys> lock(errors_mutex);
-    stored_error = new RTCError(RTC_ERROR_NONE);
+    stored_error = new RTCErrorMessage(RTC_ERROR_NONE, "");
     thread_errors.push_back(stored_error);
     setTls(thread_error,stored_error);
     return stored_error;
@@ -84,6 +87,8 @@ namespace embree
     max_spatial_split_replications = 1.2f;
     useSpatialPreSplits = false;
 
+    max_triangles_per_leaf = inf;
+
     tessellation_cache_size = 128*1024*1024;
 
     subdiv_accel = "default";
@@ -194,15 +199,13 @@ namespace embree
   bool State::parseFile(const FileName& fileName)
   { 
     Ref<Stream<int> > file;
-    // -- GODOT start --
-    // try {
+    //try {
       file = new FileStream(fileName);
-    // }
-    // catch (std::runtime_error& e) {
-    //   (void) e;
-    //   return false;
-    // }
-    // -- GODOT end --
+    //}
+    //catch (std::runtime_error& e) {
+    //  (void) e;
+    //  return false;
+    //}
     
     std::vector<std::string> syms;
     for (size_t i=0; i<sizeof(symbols)/sizeof(void*); i++) 
@@ -430,6 +433,9 @@ namespace embree
       else if (tok == Token::Id("max_spatial_split_replications") && cin->trySymbol("="))
         max_spatial_split_replications = cin->get().Float();
 
+      else if (tok == Token::Id("max_triangles_per_leaf") && cin->trySymbol("="))
+        max_triangles_per_leaf = cin->get().Float();
+
       else if (tok == Token::Id("presplits") && cin->trySymbol("="))
         useSpatialPreSplits = cin->get().Int() != 0 ? true : false;
 
