@@ -63,6 +63,8 @@ void SoftBodyMotionProperties::Initialize(const SoftBodyCreationSettings &inSett
 	mNumIterations = inSettings.mNumIterations;
 	mPressure = inSettings.mPressure;
 	mUpdatePosition = inSettings.mUpdatePosition;
+	mFacesDoubleSided = inSettings.mFacesDoubleSided;
+	SetVertexRadius(inSettings.mVertexRadius);
 
 	// Initialize vertices
 	mVertices.resize(inSettings.mSettings->mVertices.size());
@@ -78,6 +80,20 @@ void SoftBodyMotionProperties::Initialize(const SoftBodyCreationSettings &inSett
 		mLocalBounds.Encapsulate(out_vertex.mPosition);
 	}
 
+	// Initialize rods
+	if (!inSettings.mSettings->mRodStretchShearConstraints.empty())
+	{
+		mRodStates.resize(inSettings.mSettings->mRodStretchShearConstraints.size());
+		Quat rotation_q = rotation.GetQuaternion();
+		for (Array<RodState>::size_type r = 0, s = mRodStates.size(); r < s; ++r)
+		{
+			const SoftBodySharedSettings::RodStretchShear &in_rod = inSettings.mSettings->mRodStretchShearConstraints[r];
+			RodState &out_rod = mRodStates[r];
+			out_rod.mRotation = rotation_q * in_rod.mBishop;
+			out_rod.mAngularVelocity = Vec3::sZero();
+		}
+	}
+
 	// Allocate space for skinned vertices
 	if (!inSettings.mSettings->mSkinnedConstraints.empty())
 		mSkinState.resize(mVertices.size());
@@ -170,7 +186,7 @@ void SoftBodyMotionProperties::DetermineCollidingShapes(const SoftBodyUpdateCont
 						Array<LeafShape>	mHits;
 					};
 					LeafShapeCollector collector;
-					body.GetShape()->CollectTransformedShapes(mLocalBounds, com.GetTranslation(), com.GetQuaternion(), Vec3::sOne(), SubShapeIDCreator(), collector, mShapeFilter);
+					body.GetShape()->CollectTransformedShapes(mLocalBounds, com.GetTranslation(), com.GetQuaternion(), Vec3::sOne(), SubShapeIDCreator(), collector, mShapeFilter.GetFilter());
 					if (collector.mHits.empty())
 						return;
 
@@ -222,14 +238,13 @@ void SoftBodyMotionProperties::DetermineCollidingShapes(const SoftBodyUpdateCont
 	// Calculate local bounding box
 	AABox local_bounds = mLocalBounds;
 	local_bounds.Encapsulate(mLocalPredictedBounds);
-	local_bounds.ExpandBy(Vec3::sReplicate(mSettings->mVertexRadius));
+	local_bounds.ExpandBy(Vec3::sReplicate(mVertexRadius));
 
 	// Calculate world space bounding box
 	AABox world_bounds = local_bounds.Transformed(inContext.mCenterOfMassTransform);
 
 	// Create shape filter
-	SimShapeFilterWrapperUnion shape_filter_union(inContext.mSimShapeFilter, inContext.mBody);
-	SimShapeFilterWrapper &shape_filter = shape_filter_union.GetSimShapeFilterWrapper();
+	SimShapeFilterWrapper shape_filter(inContext.mSimShapeFilter, inContext.mBody);
 
 	Collector collector(inContext, inSystem, inBodyLockInterface, local_bounds, shape_filter, mCollidingShapes, mCollidingSensors);
 	ObjectLayer layer = inContext.mBody->GetObjectLayer();
@@ -318,6 +333,7 @@ void SoftBodyMotionProperties::IntegratePositions(const SoftBodyUpdateContext &i
 	Vec3 sub_step_gravity = inContext.mGravity * dt;
 	Vec3 sub_step_impulse = GetAccumulatedForce() * dt / max(float(mVertices.size()), 1.0f);
 	for (Vertex &v : mVertices)
+	{
 		if (v.mInvMass > 0.0f)
 		{
 			// Gravity
@@ -325,17 +341,27 @@ void SoftBodyMotionProperties::IntegratePositions(const SoftBodyUpdateContext &i
 
 			// Damping
 			v.mVelocity *= linear_damping;
+		}
 
-			// Integrate
-			v.mPreviousPosition = v.mPosition;
-			v.mPosition += v.mVelocity * dt;
-		}
-		else
-		{
-			// Integrate
-			v.mPreviousPosition = v.mPosition;
-			v.mPosition += v.mVelocity * dt;
-		}
+		// Integrate
+		Vec3 position = v.mPosition;
+		v.mPreviousPosition = position;
+		v.mPosition = position + v.mVelocity * dt;
+	}
+
+	// Integrate rod orientations
+	float half_dt = 0.5f * dt;
+	for (RodState &r : mRodStates)
+	{
+		// Damping
+		r.mAngularVelocity *= linear_damping;
+
+		// Integrate
+		Quat rotation = r.mRotation;
+		Quat delta_rotation = half_dt * Quat::sMultiplyImaginary(r.mAngularVelocity, rotation);
+		r.mPreviousRotationInternal = rotation; // Overwrites mAngularVelocity
+		r.mRotation = (rotation + delta_rotation).Normalized();
+	}
 }
 
 void SoftBodyMotionProperties::ApplyDihedralBendConstraints(const SoftBodyUpdateContext &inContext, uint inStartIndex, uint inEndIndex)
@@ -574,6 +600,82 @@ void SoftBodyMotionProperties::ApplyEdgeConstraints(const SoftBodyUpdateContext
 	}
 }
 
+void SoftBodyMotionProperties::ApplyRodStretchShearConstraints(const SoftBodyUpdateContext &inContext, uint inStartIndex, uint inEndIndex)
+{
+	JPH_PROFILE_FUNCTION();
+
+	float inv_dt_sq = 1.0f / Square(inContext.mSubStepDeltaTime);
+
+	RodState *rod_state = mRodStates.data() + inStartIndex;
+	for (const RodStretchShear *r = mSettings->mRodStretchShearConstraints.data() + inStartIndex, *r_end = mSettings->mRodStretchShearConstraints.data() + inEndIndex; r < r_end; ++r, ++rod_state)
+	{
+		// Get positions
+		Vertex &v0 = mVertices[r->mVertex[0]];
+		Vertex &v1 = mVertices[r->mVertex[1]];
+
+		// Apply stretch and shear constraint
+		// Equation 37 from "Position and Orientation Based Cosserat Rods" - Kugelstadt and Schoemer - SIGGRAPH 2016
+		float denom = v0.mInvMass + v1.mInvMass + 4.0f * r->mInvMass * Square(r->mLength) + r->mCompliance * inv_dt_sq;
+		if (denom < 1.0e-12f)
+			continue;
+		Vec3 x0 = v0.mPosition;
+		Vec3 x1 = v1.mPosition;
+		Quat rotation = rod_state->mRotation;
+		Vec3 d3 = rotation.RotateAxisZ();
+		Vec3 delta = (x1 - x0 - d3 * r->mLength) / denom;
+		v0.mPosition = x0 + v0.mInvMass * delta;
+		v1.mPosition = x1 - v1.mInvMass * delta;
+		// q * e3_bar = q * (0, 0, -1, 0) = [-qy, qx, -qw, qz]
+		Quat q_e3_bar(rotation.GetXYZW().Swizzle<SWIZZLE_Y, SWIZZLE_X, SWIZZLE_W, SWIZZLE_Z>().FlipSign<-1, 1, -1, 1>());
+		rotation += (2.0f * r->mInvMass * r->mLength) * Quat::sMultiplyImaginary(delta, q_e3_bar);
+
+		// Renormalize
+		rod_state->mRotation = rotation.Normalized();
+	}
+}
+
+void SoftBodyMotionProperties::ApplyRodBendTwistConstraints(const SoftBodyUpdateContext &inContext, uint inStartIndex, uint inEndIndex)
+{
+	JPH_PROFILE_FUNCTION();
+
+	float inv_dt_sq = 1.0f / Square(inContext.mSubStepDeltaTime);
+
+	const Array<RodStretchShear> &rods = mSettings->mRodStretchShearConstraints;
+
+	for (const RodBendTwist *r = mSettings->mRodBendTwistConstraints.data() + inStartIndex, *r_end = mSettings->mRodBendTwistConstraints.data() + inEndIndex; r < r_end; ++r)
+	{
+		uint32 rod1_index = r->mRod[0];
+		uint32 rod2_index = r->mRod[1];
+		const RodStretchShear &rod1 = rods[rod1_index];
+		const RodStretchShear &rod2 = rods[rod2_index];
+		RodState &rod1_state = mRodStates[rod1_index];
+		RodState &rod2_state = mRodStates[rod2_index];
+
+		// Apply bend and twist constraint
+		// Equation 40 from "Position and Orientation Based Cosserat Rods" - Kugelstadt and Schoemer - SIGGRAPH 2016
+		float denom = rod1.mInvMass + rod2.mInvMass + r->mCompliance * inv_dt_sq;
+		if (denom < 1.0e-12f)
+			continue;
+		Quat rotation1 = rod1_state.mRotation;
+		Quat rotation2 = rod2_state.mRotation;
+		Quat omega = rotation1.Conjugated() * rotation2;
+		Quat omega0 = r->mOmega0;
+		Vec4 omega_min_omega0 = (omega - omega0).GetXYZW();
+		Vec4 omega_plus_omega0 = (omega + omega0).GetXYZW();
+		// Take the shortest of the two rotations
+		Quat delta_omega(Vec4::sSelect(omega_min_omega0, omega_plus_omega0, Vec4::sLess(omega_plus_omega0.DotV(omega_plus_omega0), omega_min_omega0.DotV(omega_min_omega0))));
+		delta_omega /= denom;
+		delta_omega.SetW(0.0f); // Scalar part needs to be zero because the real part of the Darboux vector doesn't vanish, see text between eq. 39 and 40.
+		Quat delta_rod2 = rod2.mInvMass * rotation1 * delta_omega;
+		rotation1 += rod1.mInvMass * rotation2 * delta_omega;
+		rotation2 -= delta_rod2;
+
+		// Renormalize
+		rod1_state.mRotation = rotation1.Normalized();
+		rod2_state.mRotation = rotation2.Normalized();
+	}
+}
+
 void SoftBodyMotionProperties::ApplyLRAConstraints(uint inStartIndex, uint inEndIndex)
 {
 	JPH_PROFILE_FUNCTION();
@@ -601,7 +703,7 @@ void SoftBodyMotionProperties::ApplyCollisionConstraintsAndUpdateVelocities(cons
 
 	float dt = inContext.mSubStepDeltaTime;
 	float restitution_threshold = -2.0f * inContext.mGravity.Length() * dt;
-	float vertex_radius = mSettings->mVertexRadius;
+	float vertex_radius = mVertexRadius;
 	for (Vertex &v : mVertices)
 		if (v.mInvMass > 0.0f)
 		{
@@ -714,6 +816,11 @@ void SoftBodyMotionProperties::ApplyCollisionConstraintsAndUpdateVelocities(cons
 				}
 			}
 		}
+
+	// Calculate the new angular velocity for all rods
+	float two_div_dt = 2.0f / dt;
+	for (RodState &r : mRodStates)
+		r.mAngularVelocity = two_div_dt * (r.mRotation * r.mPreviousRotationInternal.Conjugated()).GetXYZ(); // Overwrites mPreviousRotationInternal
 }
 
 void SoftBodyMotionProperties::UpdateSoftBodyState(SoftBodyUpdateContext &ioContext, const PhysicsSettings &inPhysicsSettings)
@@ -920,7 +1027,7 @@ SoftBodyMotionProperties::EStatus SoftBodyMotionProperties::ParallelDetermineSen
 void SoftBodyMotionProperties::ProcessGroup(const SoftBodyUpdateContext &ioContext, uint inGroupIndex)
 {
 	// Determine start and end
-	SoftBodySharedSettings::UpdateGroup start { 0, 0, 0, 0, 0 };
+	SoftBodySharedSettings::UpdateGroup start { 0, 0, 0, 0, 0, 0, 0 };
 	const SoftBodySharedSettings::UpdateGroup &prev = inGroupIndex > 0? mSettings->mUpdateGroups[inGroupIndex - 1] : start;
 	const SoftBodySharedSettings::UpdateGroup &current = mSettings->mUpdateGroups[inGroupIndex];
 
@@ -936,6 +1043,10 @@ void SoftBodyMotionProperties::ProcessGroup(const SoftBodyUpdateContext &ioConte
 	// Process edges
 	ApplyEdgeConstraints(ioContext, prev.mEdgeEndIndex, current.mEdgeEndIndex);
 
+	// Process rods
+	ApplyRodStretchShearConstraints(ioContext, prev.mRodStretchShearEndIndex, current.mRodStretchShearEndIndex);
+	ApplyRodBendTwistConstraints(ioContext, prev.mRodBendTwistEndIndex, current.mRodBendTwistEndIndex);
+
 	// Process LRA constraints
 	ApplyLRAConstraints(prev.mLRAEndIndex, current.mLRAEndIndex);
 }
@@ -1192,6 +1303,62 @@ void SoftBodyMotionProperties::DrawEdgeConstraints(DebugRenderer *inRenderer, RM
 		Color::sWhite);
 }
 
+void SoftBodyMotionProperties::DrawRods(DebugRenderer *inRenderer, RMat44Arg inCenterOfMassTransform, ESoftBodyConstraintColor inConstraintColor) const
+{
+	DrawConstraints(inConstraintColor,
+		[](const SoftBodySharedSettings::UpdateGroup &inGroup) {
+			return inGroup.mRodStretchShearEndIndex;
+		},
+		[this, inRenderer, &inCenterOfMassTransform](uint inIndex, ColorArg inColor) {
+			const RodStretchShear &r = mSettings->mRodStretchShearConstraints[inIndex];
+			inRenderer->DrawLine(inCenterOfMassTransform * mVertices[r.mVertex[0]].mPosition, inCenterOfMassTransform * mVertices[r.mVertex[1]].mPosition, inColor);
+		},
+		Color::sWhite);
+}
+
+void SoftBodyMotionProperties::DrawRodStates(DebugRenderer *inRenderer, RMat44Arg inCenterOfMassTransform, ESoftBodyConstraintColor inConstraintColor) const
+{
+	DrawConstraints(inConstraintColor,
+		[](const SoftBodySharedSettings::UpdateGroup &inGroup) {
+			return inGroup.mRodStretchShearEndIndex;
+		},
+		[this, inRenderer, &inCenterOfMassTransform](uint inIndex, ColorArg inColor) {
+			const RodState &state = mRodStates[inIndex];
+			const RodStretchShear &rod = mSettings->mRodStretchShearConstraints[inIndex];
+
+			RVec3 x0 = inCenterOfMassTransform * mVertices[rod.mVertex[0]].mPosition;
+			RVec3 x1 = inCenterOfMassTransform * mVertices[rod.mVertex[1]].mPosition;
+
+			RMat44 rod_center = inCenterOfMassTransform;
+			rod_center.SetTranslation(0.5_r * (x0 + x1));
+			inRenderer->DrawArrow(rod_center.GetTranslation(), rod_center.GetTranslation() + state.mAngularVelocity, inColor, 0.01f * rod.mLength);
+
+			RMat44 rod_frame = rod_center * RMat44::sRotation(state.mRotation);
+			inRenderer->DrawCoordinateSystem(rod_frame, 0.3f * rod.mLength);
+		},
+		Color::sOrange);
+}
+
+void SoftBodyMotionProperties::DrawRodBendTwistConstraints(DebugRenderer *inRenderer, RMat44Arg inCenterOfMassTransform, ESoftBodyConstraintColor inConstraintColor) const
+{
+	DrawConstraints(inConstraintColor,
+		[](const SoftBodySharedSettings::UpdateGroup &inGroup) {
+			return inGroup.mRodBendTwistEndIndex;
+		},
+		[this, inRenderer, &inCenterOfMassTransform](uint inIndex, ColorArg inColor) {
+			uint r1 = mSettings->mRodBendTwistConstraints[inIndex].mRod[0];
+			uint r2 = mSettings->mRodBendTwistConstraints[inIndex].mRod[1];
+			const RodStretchShear &rod1 = mSettings->mRodStretchShearConstraints[r1];
+			const RodStretchShear &rod2 = mSettings->mRodStretchShearConstraints[r2];
+
+			RVec3 x0 = inCenterOfMassTransform * (0.4f * mVertices[rod1.mVertex[0]].mPosition + 0.6f * mVertices[rod1.mVertex[1]].mPosition);
+			RVec3 x1 = inCenterOfMassTransform * (0.6f * mVertices[rod2.mVertex[0]].mPosition + 0.4f * mVertices[rod2.mVertex[1]].mPosition);
+
+			inRenderer->DrawLine(x0, x1, inColor);
+		},
+		Color::sGreen);
+}
+
 void SoftBodyMotionProperties::DrawBendConstraints(DebugRenderer *inRenderer, RMat44Arg inCenterOfMassTransform, ESoftBodyConstraintColor inConstraintColor) const
 {
 	DrawConstraints(inConstraintColor,
@@ -1279,11 +1446,16 @@ void SoftBodyMotionProperties::SaveState(StateRecorder &inStream) const
 
 	for (const Vertex &v : mVertices)
 	{
-		inStream.Write(v.mPreviousPosition);
 		inStream.Write(v.mPosition);
 		inStream.Write(v.mVelocity);
 	}
 
+	for (const RodState &r : mRodStates)
+	{
+		inStream.Write(r.mRotation);
+		inStream.Write(r.mAngularVelocity);
+	}
+
 	for (const SkinState &s : mSkinState)
 	{
 		inStream.Write(s.mPreviousPosition);
@@ -1303,11 +1475,16 @@ void SoftBodyMotionProperties::RestoreState(StateRecorder &inStream)
 
 	for (Vertex &v : mVertices)
 	{
-		inStream.Read(v.mPreviousPosition);
 		inStream.Read(v.mPosition);
 		inStream.Read(v.mVelocity);
 	}
 
+	for (RodState &r : mRodStates)
+	{
+		inStream.Read(r.mRotation);
+		inStream.Read(r.mAngularVelocity);
+	}
+
 	for (SkinState &s : mSkinState)
 	{
 		inStream.Read(s.mPreviousPosition);
